@@ -8,10 +8,8 @@
 #define LIBBOARDGAME_BASE_GRID_H
 
 #include <algorithm>
-#include <cstring>
 #include <iomanip>
 #include <sstream>
-#include <type_traits>
 #include "Point.h"
 #include "Geometry.h"
 
@@ -24,7 +22,8 @@ using namespace std;
 /** Elements assigned to on-board points.
     The elements must be default-constructible. This class is a POD if the
     element type is a POD.
-    @tparam P An instantiation of libboardgame_base::Point (or compatible class)
+    @tparam P An instantiation of libboardgame_base::Point (or compatible
+    class)
     @tparam T The element type. */
 template<class P, typename T>
 class Grid
@@ -34,8 +33,6 @@ public:
 
     typedef libboardgame_base::Geometry<P> Geometry;
 
-    typedef typename libboardgame_base::Geometry<P>::Iterator Iterator;
-
     T& operator[](const Point& p);
 
     const T& operator[](const Point& p) const;
@@ -48,44 +45,43 @@ public:
 
     string to_string(const Geometry& geo) const;
 
-    /** @pre is_trivially_copyable<T>::value */
-    void memcpy_from(const Grid& grid, const Geometry& geo);
+    void copy_from(const Grid& grid, const Geometry& geo);
 
 private:
-    T m_a[Point::range_onboard_end];
+    T m_a[Point::range_onboard];
 };
 
 template<class P, typename T>
 inline T& Grid<P, T>::operator[](const Point& p)
 {
+    LIBBOARDGAME_ASSERT(! p.is_null());
     return m_a[p.to_int()];
 }
 
 template<class P, typename T>
 inline const T& Grid<P, T>::operator[](const Point& p) const
 {
+    LIBBOARDGAME_ASSERT(! p.is_null());
     return m_a[p.to_int()];
 }
 
 template<class P, typename T>
 inline void Grid<P, T>::fill(const T& val, const Geometry& geo)
 {
-    for (Iterator i(geo); i; ++i)
-        (*this)[*i] = val;
+    std::fill(m_a + Point::begin_onboard, m_a + geo.get_range(), val);
 }
 
 template<class P, typename T>
 inline void Grid<P, T>::fill_all(const T& val)
 {
-    std::fill(m_a, m_a + Point::range_onboard_end, val);
+    std::fill(m_a + Point::begin_onboard, m_a + Point::range_onboard, val);
 }
 
 template<class P, typename T>
-inline void Grid<P, T>::memcpy_from(const Grid& grid, const Geometry& geo)
+inline void Grid<P, T>::copy_from(const Grid& grid, const Geometry& geo)
 {
-    // Uncomment once is_trivially_copyable is implemented in GCC and MSVC
-    //static_assert(is_trivially_copyable<T>::value, "")
-    memcpy(m_a, grid.m_a, geo.get_range());
+    copy(grid.m_a + Point::begin_onboard, grid.m_a + geo.get_range(),
+         m_a + Point::begin_onboard);
 }
 
 template<class P, typename T>
@@ -93,10 +89,10 @@ string Grid<P, T>::to_string(const Geometry& geo) const
 {
     ostringstream buffer;
     size_t max_len = 0;
-    for (Iterator i(geo); i; ++i)
+    for (Point p : geo)
     {
         buffer.str("");
-        buffer << (*this)[*i];
+        buffer << (*this)[p];
         max_len = max(max_len, buffer.str().length());
     }
     buffer.str("");
@@ -107,8 +103,8 @@ string Grid<P, T>::to_string(const Geometry& geo) const
     {
         for (unsigned x = 0; x < width; ++x)
         {
-            Point p(x, y, width);
-            if (geo.is_onboard(p))
+            Point p = geo.get_point(x, y);
+            if (! p.is_null())
                 buffer << setw(int(max_len)) << (*this)[p];
             else
                 buffer << empty;
