@@ -21,12 +21,16 @@ using namespace std;
 /** Geometry of a regular rectangular grid.
     @tparam P An instantiation of libboardgame_base::Point */
 template<class P>
-class RectGeometry
+class RectGeometry final
     : public Geometry<P>
 {
 public:
     typedef P Point;
 
+    using AdjList = typename Geometry<P>::AdjList;
+
+    using DiagList = typename Geometry<P>::DiagList;
+
     /** Create or reuse an already created geometry with a given size. */
     static const RectGeometry& get(unsigned width, unsigned height);
 
@@ -39,10 +43,9 @@ public:
     unsigned get_period_y() const override;
 
 protected:
-    void init_is_onboard(Point p, bool& is_onboard) const;
+    bool init_is_onboard(unsigned x, unsigned y) const override;
 
-    void init_adj_diag(Point p, NullTermList<Point, 4>& adj,
-                       NullTermList<Point, 9>& diag) const;
+    void init_adj_diag(Point p, AdjList& adj, DiagList& diag) const override;
 
 private:
     /** Stores already created geometries by width and height. */
@@ -91,44 +94,39 @@ unsigned RectGeometry<P>::get_point_type(int x, int y) const
 }
 
 template<class P>
-void RectGeometry<P>::init_is_onboard(Point p, bool& is_onboard) const
+bool RectGeometry<P>::init_is_onboard(unsigned x, unsigned y) const
 {
-    LIBBOARDGAME_UNUSED(p);
-    is_onboard = true;
+    LIBBOARDGAME_UNUSED(x);
+    LIBBOARDGAME_UNUSED(y);
+    return true;
 }
 
 template<class P>
-void RectGeometry<P>::init_adj_diag(Point p, NullTermList<Point, 4>& adj,
-                                    NullTermList<Point, 9>& diag) const
+void RectGeometry<P>::init_adj_diag(Point p, AdjList& adj,
+                                    DiagList& diag) const
 {
     auto width = this->get_width();
     auto height = this->get_height();
-    auto x = p.get_x(width);
-    auto y = p.get_y(width);
-    {
-        typename NullTermList<Point, 4>::Init init_adj(adj);
-        if (y > 0)
-            init_adj.push_back(p.get_up(width));
-        if (x > 0)
-            init_adj.push_back(p.get_left());
-        if (x < width - 1)
-            init_adj.push_back(p.get_right());
-        if (y < height - 1)
-            init_adj.push_back(p.get_down(width));
-        init_adj.finish();
-    }
-    {
-        typename NullTermList<Point, 9>::Init init_diag(diag);
-        if (x > 0 && y > 0)
-            init_diag.push_back(p.get_up_left(width));
-        if (x < width - 1 && y > 0)
-            init_diag.push_back(p.get_up_right(width));
-        if (x > 0 && y < height - 1)
-            init_diag.push_back(p.get_down_left(width));
-        if (x < width - 1 && y < height - 1)
-            init_diag.push_back(p.get_down_right(width));
-        init_diag.finish();
-    }
+    auto x = this->get_x(p);
+    auto y = this->get_y(p);
+
+    if (y > 0)
+        adj.push_back(this->get_point(x, y - 1));
+    if (x > 0)
+        adj.push_back(this->get_point(x - 1, y));
+    if (x < width - 1)
+        adj.push_back(this->get_point(x + 1, y));
+    if (y < height - 1)
+        adj.push_back(this->get_point(x, y + 1));
+
+    if (x > 0 && y > 0)
+        diag.push_back(this->get_point(x - 1, y - 1));
+    if (x < width - 1 && y > 0)
+        diag.push_back(this->get_point(x + 1, y - 1));
+    if (x > 0 && y < height - 1)
+        diag.push_back(this->get_point(x - 1, y + 1));
+    if (x < width - 1 && y < height - 1)
+        diag.push_back(this->get_point(x + 1, y + 1));
 }
 
 //-----------------------------------------------------------------------------
