@@ -7,7 +7,7 @@
 #ifndef LIBBOARDGAME_GTP_FAILURE_H
 #define LIBBOARDGAME_GTP_FAILURE_H
 
-#include <string>
+#include <stdexcept>
 
 namespace libboardgame_gtp {
 
@@ -19,26 +19,14 @@ using namespace std;
     Command handlers generate a GTP error response by throwing an instance
     of Failure. */
 class Failure
+    : public runtime_error
 {
 public:
-    Failure(const string& response = "");
-
-    string get_response() const;
-
-private:
-    string m_response;
+    Failure(const string& response)
+        : runtime_error(response)
+    { }
 };
 
-inline Failure::Failure(const string& response)
-    : m_response(response)
-{
-}
-
-inline string Failure::get_response() const
-{
-    return m_response;
-}
-
 //-----------------------------------------------------------------------------
 
 } // namespace libboardgame_gtp
