@@ -21,7 +21,6 @@
 #include "TreeUtil.h"
 #include "libboardgame_util/Abort.h"
 #include "libboardgame_util/Barrier.h"
-#include "libboardgame_util/Exception.h"
 #include "libboardgame_util/IntervalChecker.h"
 #include "libboardgame_util/Log.h"
 #include "libboardgame_util/Statistics.h"
@@ -41,7 +40,6 @@ using libboardgame_util::time_to_string;
 using libboardgame_util::to_string;
 using libboardgame_util::ArrayList;
 using libboardgame_util::Barrier;
-using libboardgame_util::Exception;
 using libboardgame_util::IntervalChecker;
 using libboardgame_util::StatisticsBase;
 using libboardgame_util::StatisticsDirtyLockFree;
@@ -72,6 +70,11 @@ struct SearchParamConstDefault
     /** The maximum length of a game. */
     static const unsigned max_moves = 1000;
 
+    /** Compile with support for multi-threaded search.
+        Disabling this slightly increases the performance if support for a
+        multi-threaded search is not needed. */
+    static const bool multithread = true;
+
     /** Use RAVE. */
     static const bool rave = false;
 
@@ -92,11 +95,25 @@ struct SearchParamConstDefault
 
     /** Enable Last-Good-Reply heuristic.
         @see LastGoodReply */
-    static const bool use_last_good_reply = false;
+    static const bool use_lgr = false;
+
+    /** See LastGoodReply::hash_table_size.
+        Must be greater 0 if use_lgr is true. */
+    static const size_t lgr_hash_table_size = 0;
 
     /** Use virtual loss in multi-threaded mode.
         See Chaslot et al.: Parallel Monte-Carlo Tree Search. 2008. */
     static const bool virtual_loss = false;
+
+    /** The minimum count used in prior knowledge initialization of
+        the children of an expanded node.
+        This should use constexpr Float in the future (not yet supported by
+        MSVC 2013). */
+    static const unsigned child_min_count = 0;
+
+    /** Terminate search early if move is unlikely to change.
+        See implementation of check_cannot_change(). */
+    static const bool use_unlikely_change = true;
 };
 
 //-----------------------------------------------------------------------------
@@ -130,11 +147,11 @@ public:
 
     typedef R SearchParamConst;
 
+    static const bool multithread = SearchParamConst::multithread;
+
     typedef typename SearchParamConst::Float Float;
 
-    typedef libboardgame_mcts::Node<M, Float> Node;
-
-    typedef libboardgame_mcts::ChildIterator<Node> ChildIterator;
+    typedef libboardgame_mcts::Node<M, Float, multithread> Node;
 
     typedef libboardgame_mcts::Tree<Node> Tree;
 
@@ -144,7 +161,11 @@ public:
 
     static const unsigned max_moves = SearchParamConst::max_moves;
 
-    typedef array<StatisticsDirtyLockFree<Float>, max_players> RootStat;
+    static const size_t lgr_hash_table_size =
+            SearchParamConst::lgr_hash_table_size;
+
+    static_assert(! SearchParamConst::use_lgr || lgr_hash_table_size > 0, "");
+
 
     /** Constructor.
         @param nu_threads
@@ -258,10 +279,6 @@ public:
 
     bool get_reuse_tree() const;
 
-    void set_prune_full_tree(bool enable);
-
-    bool get_prune_full_tree() const;
-
     /** Maximum parent visit count for applying RAVE. */
     void set_rave_parent_max(Float value);
 
@@ -341,14 +358,14 @@ public:
     size_t get_nu_simulations() const;
 
     /** Select the move to play.
-        Uses select_child_final() on the root node. */
+        Uses select_final(). */
     bool select_move(Move& mv) const;
 
-    /** Select the best child of a node after the search.
-        Selects child with highest visit count; the value is used as a
+    /** Select the best child of the root node after the search.
+        Selects child with highest number of wins; the value is used as a
         tie-breaker for equal counts (important at very low number of
         simulations, e.g. all children have count 1 or 0). */
-    const Node* select_child_final(const Node& node) const;
+    const Node* select_final() const;
 
     State& get_state(unsigned thread_id);
 
@@ -360,8 +377,16 @@ public:
         of the callback function are: elapsed time, estimated remaining time. */
     void set_callback(function<void(double, double)> callback);
 
-    /** Get mean evaluation for all players at root node. */
-    const RootStat& get_root_val() const;
+    /** Get evaluation for a player at root node. */
+    const StatisticsDirtyLockFree<Float>& get_root_val(PlayerInt player) const;
+
+    /** Get evaluation for get_player() at root node. */
+    const StatisticsDirtyLockFree<Float>& get_root_val() const;
+
+    /** The number of times the root node was visited.
+        This is equal to the number of simulations plus the visit count
+        of a subtree reused from the previous search. */
+    Float get_root_visit_count() const;
 
     /** Create the threads used in the search.
         This cannot be done in the constructor because it uses the virtual
@@ -377,8 +402,6 @@ protected:
     {
         ArrayList<const Node*, max_moves> nodes;
 
-        ArrayList<const Node*, max_moves> last_nodes;
-
         ArrayList<PlayerMove, max_moves> moves;
 
         array<Float, max_players> eval;
@@ -477,13 +500,13 @@ private:
     private:
         SearchFunc m_search_func;
 
-        bool m_quit;
+        bool m_quit = false;
 
-        bool m_start_search_flag;
+        bool m_start_search_flag = false;
 
-        bool m_search_finished_flag;
+        bool m_search_finished_flag = false;
 
-        Barrier m_thread_ready;
+        Barrier m_thread_ready{2};
 
         mutex m_start_search_mutex;
 
@@ -493,7 +516,8 @@ private:
 
         condition_variable m_search_finished_cond;
 
-        unique_lock<mutex> m_search_finished_lock;
+        unique_lock<mutex> m_search_finished_lock{m_search_finished_mutex,
+                                                  defer_lock};
 
         thread m_thread;
 
@@ -503,22 +527,20 @@ private:
     unsigned m_nu_threads;
 
     /** See set_full_select_interval(). */
-    unsigned m_full_select_interval;
+    unsigned m_full_select_interval = 1;
 
     /** See set_full_select_min(). */
-    Float m_full_select_min;
+    Float m_full_select_min = numeric_limits<Float>::max();
 
-    Float m_expand_threshold;
+    Float m_expand_threshold = 0;
 
-    Float m_expand_threshold_inc;
+    Float m_expand_threshold_inc = 0;
 
-    bool m_deterministic;
+    bool m_deterministic = false;
 
-    bool m_reuse_subtree;
+    bool m_reuse_subtree = true;
 
-    bool m_reuse_tree;
-
-    bool m_prune_full_tree;
+    bool m_reuse_tree = false;
 
     /** Player to play at the root node of the search. */
     PlayerInt m_player;
@@ -530,15 +552,15 @@ private:
     /** Time of last search. */
     double m_last_time;
 
-    Float m_prune_count_start;
+    Float m_prune_count_start = 16;
 
-    Float m_rave_parent_max;
+    Float m_rave_parent_max = 50000;
 
-    Float m_rave_child_max;
+    Float m_rave_child_max = 2000;
 
-    Float m_rave_weight;
+    Float m_rave_weight = 0.3f;
 
-    Float m_rave_dist_final;
+    Float m_rave_dist_final = 0;
 
     /** Minimum simulations to perform in the current search.
         This does not include the count of simulations reused from a subtree of
@@ -582,17 +604,12 @@ private:
     Tree m_tree;
 
     /** See get_root_val(). */
-    RootStat m_root_val;
+    array<StatisticsDirtyLockFree<Float>, max_players> m_root_val;
 
-    /** Current position value estimate for prior knowledge initialization.
-        Derived form root values of last search and updated with root values
-        of current search. */
-    RootStat m_init_val;
-
-    LastGoodReply<M, max_players> m_last_good_reply;
+    LastGoodReply<Move, max_players, lgr_hash_table_size, multithread> m_lgr;
 
     /** See get_nu_simulations(). */
-    LIBBOARDGAME_MCTS_ATOMIC(size_t) m_nu_simulations;
+    Atomic<size_t, multithread> m_nu_simulations;
 
     /** Mutex used in log_thread() */
     mutable mutex m_log_mutex;
@@ -612,7 +629,10 @@ private:
 
     void check_create_threads();
 
-    bool check_move_cannot_change(Float count, Float remaining) const;
+    bool check_cannot_change(ThreadState& thread_state, Float remaining) const;
+
+    bool estimate_reused_root_val(Tree& tree, const Node& root, Float& value,
+                                  Float& count);
 
     bool expand_node(ThreadState& thread_state, const Node& node,
                      const Node*& best_child);
@@ -626,13 +646,11 @@ private:
     bool prune(TimeSource& time_source, double time, double max_time,
                Float prune_min_count, Float& new_prune_min_count);
 
-    void restore_root_from_children(Tree& tree, const Node& root);
-
     void search_loop(ThreadState& thread_state);
 
     const Node* select_child(const Node& node);
 
-    void update_last_good_reply(ThreadState& thread_state);
+    void update_lgr(ThreadState& thread_state);
 
     void update_rave(ThreadState& thread_state);
 
@@ -647,14 +665,8 @@ SearchBase<S, M, R>::ThreadState::~ThreadState()
 
 template<class S, class M, class R>
 SearchBase<S, M, R>::Thread::Thread(SearchFunc& search_func)
-    : m_search_func(search_func),
-      m_quit(false),
-      m_start_search_flag(false),
-      m_search_finished_flag(false),
-      m_thread_ready(2),
-      m_search_finished_lock(m_search_finished_mutex, defer_lock)
-{
-}
+    : m_search_func(search_func)
+{ }
 
 template<class S, class M, class R>
 SearchBase<S, M, R>::Thread::~Thread()
@@ -744,19 +756,6 @@ void SearchBase<S, M, R>::AssertionHandler::run()
 template<class S, class M, class R>
 SearchBase<S, M, R>::SearchBase(unsigned nu_threads, size_t memory)
     : m_nu_threads(nu_threads),
-      m_full_select_interval(1),
-      m_full_select_min(numeric_limits<Float>::max()),
-      m_expand_threshold(0),
-      m_expand_threshold_inc(0),
-      m_deterministic(false),
-      m_reuse_subtree(true),
-      m_reuse_tree(false),
-      m_prune_full_tree(true),
-      m_prune_count_start(16),
-      m_rave_parent_max(50000),
-      m_rave_child_max(2000),
-      m_rave_weight(0.3f),
-      m_rave_dist_final(0),
       m_tree_memory(memory == 0 ? 256000000 : memory),
       m_max_nodes(get_max_nodes(m_tree_memory)),
       m_bias_term(0),
@@ -787,14 +786,21 @@ bool SearchBase<S, M, R>::check_abort(const ThreadState& thread_state) const
 }
 
 template<class S, class M, class R>
-bool SearchBase<S, M, R>::check_abort_expensive(ThreadState& thread_state) const
+bool SearchBase<S, M, R>::check_abort_expensive(
+        ThreadState& thread_state) const
 {
     if (get_abort())
     {
         log_thread(thread_state, "Search aborted");
         return true;
     }
-    auto count = m_tree.get_root().get_visit_count();
+    auto& root = m_tree.get_root();
+    if (root.get_nu_children() == 1)
+    {
+        log("Root has only one child");
+        return true;
+    }
+    auto count = root.get_visit_count();
     if (count >= m_max_float_count)
     {
         log_thread(thread_state,
@@ -810,9 +816,7 @@ bool SearchBase<S, M, R>::check_abort_expensive(ThreadState& thread_state) const
         simulations_per_sec = expected_sim_per_sec();
     else
     {
-        size_t nu_simulations =
-            LIBBOARDGAME_MCTS_ATOMIC_LOAD(m_nu_simulations,
-                                          memory_order_relaxed);
+        size_t nu_simulations = m_nu_simulations.load(memory_order_relaxed);
         simulations_per_sec = double(nu_simulations) / time;
     }
     double remaining_time;
@@ -838,10 +842,48 @@ bool SearchBase<S, M, R>::check_abort_expensive(ThreadState& thread_state) const
         m_callback(time, remaining_time);
     if (count + remaining_simulations > m_max_float_count)
         remaining_simulations = m_max_float_count - count;
-    if (check_move_cannot_change(count, remaining_simulations))
-    {
-        log_thread(thread_state, "Move cannot change anymore");
+    if (check_cannot_change(thread_state, remaining_simulations))
         return true;
+    return false;
+}
+
+template<class S, class M, class R>
+bool SearchBase<S, M, R>::check_cannot_change(ThreadState& thread_state,
+                                              Float remaining) const
+{
+    // select_final() selects move with highest number of wins.
+    Float max_wins = 0;
+    Float second_max = 0;
+    for (auto& i : m_tree.get_root_children())
+    {
+        Float wins = i.get_value() * i.get_value_count();
+        if (wins > max_wins)
+        {
+            second_max = max_wins;
+            max_wins = wins;
+        }
+    }
+    Float diff = max_wins - second_max;
+    if (diff >= remaining)
+    {
+        log_thread(thread_state, "Move cannot change");
+        return true;
+    }
+    if (SearchParamConst::use_unlikely_change)
+    {
+        // Weight remaining number of simulations with current global win rate,
+        // but not less than 10%
+        auto& root_val = m_root_val[m_player];
+        if (root_val.get_count() < 100)
+            return false; // Not enough statistics
+        auto win_rate = root_val.get_mean();
+        if (win_rate < 0.1f)
+            win_rate = 0.1f;
+        if (diff >= win_rate * remaining)
+        {
+            log_thread(thread_state, "Move change unlikely");
+            return true;
+        }
     }
     return false;
 }
@@ -860,34 +902,12 @@ bool SearchBase<S, M, R>::check_followup(ArrayList<Move, max_moves>& sequence)
     return false;
 }
 
-template<class S, class M, class R>
-bool SearchBase<S, M, R>::check_move_cannot_change(Float count,
-                                               Float remaining) const
-{
-    if (remaining > count)
-        return false;
-    Float max_count = 0;
-    Float second_max_count = 0;
-    for (ChildIterator i(m_tree, m_tree.get_root()); i; ++i)
-    {
-        Float count = i->get_visit_count();
-        if (count > max_count)
-        {
-            second_max_count = max_count;
-            max_count = count;
-        }
-    }
-    return (max_count > second_max_count + remaining);
-}
-
 template<class S, class M, class R>
 void SearchBase<S, M, R>::create_threads()
 {
-#ifdef LIBBOARDGAME_MCTS_SINGLE_THREAD
-    if (m_nu_threads > 1)
-        throw Exception("libboardgame_mcts::Search was compiled"
-                        " without support for multithreading");
-#endif
+    if (! multithread && m_nu_threads > 1)
+        throw runtime_error("libboardgame_mcts::Search was compiled"
+                            " without support for multithreading");
     log("Creating ", m_nu_threads, " threads");
     m_threads.clear();
     m_threads.reserve(m_nu_threads);
@@ -928,11 +948,15 @@ bool SearchBase<S, M, R>::expand_node(ThreadState& thread_state,
                                       const Node*& best_child)
 {
     auto& state = *thread_state.state;
-    typename Tree::NodeExpander expander(thread_state.thread_id, m_tree, node);
-    state.gen_children(expander, m_init_val[state.get_to_play()].get_mean());
+    // SearchParamConst::child_min_count is currently declared as unsigned int
+    // because MSVC does not support constexpr Float yet
+    Float min_count = static_cast<Float>(SearchParamConst::child_min_count);
+    auto thread_id = thread_state.thread_id;
+    typename Tree::NodeExpander expander(thread_id, m_tree, min_count);
+    state.gen_children(expander, m_root_val[state.get_player()].get_mean());
     if (! expander.is_tree_full())
     {
-        expander.link_children();
+        expander.link_children(m_tree, node);
         best_child = expander.get_best_child();
         return true;
     }
@@ -983,9 +1007,24 @@ inline size_t SearchBase<S, M, R>::get_nu_simulations() const
 }
 
 template<class S, class M, class R>
-inline auto SearchBase<S, M, R>::get_root_val() const -> const RootStat&
+inline auto SearchBase<S, M, R>::get_root_val(PlayerInt player) const
+-> const StatisticsDirtyLockFree<Float>&
 {
-    return m_root_val;
+    LIBBOARDGAME_ASSERT(player < m_nu_players);
+    return m_root_val[player];
+}
+
+template<class S, class M, class R>
+inline auto SearchBase<S, M, R>::get_root_val() const
+-> const StatisticsDirtyLockFree<Float>&
+{
+    return get_root_val(get_player());
+}
+
+template<class S, class M, class R>
+inline auto SearchBase<S, M, R>::get_root_visit_count() const -> Float
+{
+    return m_tree.get_root().get_visit_count();
 }
 
 template<class S, class M, class R>
@@ -994,12 +1033,6 @@ inline auto SearchBase<S, M, R>::get_prune_count_start() const -> Float
     return m_prune_count_start;
 }
 
-template<class S, class M, class R>
-inline bool SearchBase<S, M, R>::get_prune_full_tree() const
-{
-    return m_prune_full_tree;
-}
-
 template<class S, class M, class R>
 inline auto SearchBase<S, M, R>::get_rave_dist_final() const -> Float
 {
@@ -1098,31 +1131,21 @@ template<class S, class M, class R>
 void SearchBase<S, M, R>::playout(ThreadState& thread_state)
 {
     auto& state = *thread_state.state;
-    auto& simulation = thread_state.simulation;
     state.start_playout();
+    auto& simulation = thread_state.simulation;
+    auto& moves = simulation.moves;
+    auto nu_moves = moves.size();
+    Move last = nu_moves > 0 ? moves[nu_moves - 1].move : Move::null();
+    Move second_last = nu_moves > 1 ? moves[nu_moves - 2].move : Move::null();
     while (true)
     {
-        Move lgr1 = Move::null();
-        Move lgr2 = Move::null();
-        if (SearchParamConst::use_last_good_reply)
-        {
-            auto& moves = simulation.moves;
-            auto nu_moves = moves.size();
-            if (nu_moves > 0)
-            {
-                Move last_mv = moves[nu_moves - 1].move;
-                Move second_last_mv = Move::null();
-                if (nu_moves > 1)
-                    second_last_mv = moves[nu_moves - 2].move;
-                m_last_good_reply.get(state.get_to_play(), last_mv,
-                                      second_last_mv, lgr1, lgr2);
-            }
-        }
-        PlayerMove move;
-        if (! state.gen_playout_move(lgr1, lgr2, move))
+        PlayerMove mv;
+        if (! state.gen_playout_move(m_lgr, last, second_last, mv))
             break;
-        simulation.moves.push_back(move);
-        state.play_playout(move.move);
+        simulation.moves.push_back(mv);
+        state.play_playout(mv.move);
+        second_last = last;
+        last = mv.move;
     }
 }
 
@@ -1134,7 +1157,6 @@ void SearchBase<S, M, R>::play_in_tree(ThreadState& thread_state)
     auto& root = m_tree.get_root();
     auto node = &root;
     m_tree.inc_visit_count(*node);
-    simulation.nodes.push_back(node);
     Float expand_threshold = m_expand_threshold;
     if (thread_state.full_select_counter > 0)
     {
@@ -1145,36 +1167,36 @@ void SearchBase<S, M, R>::play_in_tree(ThreadState& thread_state)
         while (node->has_children())
         {
             if (node->get_visit_count() <= m_full_select_min
-                    || depth + 1 >= simulation.last_nodes.size())
+                    || depth + 1 >= simulation.nodes.size())
                 break;
-            node = simulation.last_nodes[depth + 1];
+            node = simulation.nodes[depth + 1];
             m_tree.inc_visit_count(*node);
-#ifndef LIBBOARDGAME_MCTS_SINGLE_THREAD
-            if (SearchParamConst::virtual_loss && m_nu_threads > 0)
+            if (multithread && SearchParamConst::virtual_loss
+                    && m_nu_threads > 0)
                 m_tree.add_value(*node, 0);
-#endif
-            simulation.nodes.push_back(node);
-            Move mv = node->get_move();
-            simulation.moves.push_back(PlayerMove(state.get_to_play(), mv));
-            state.play_in_tree(mv);
+            state.play_in_tree(node->get_move());
             ++depth;
             expand_threshold += m_expand_threshold_inc;
         }
+        simulation.nodes.resize(depth + 1);
+        simulation.moves.resize(depth);
     }
     else
+    {
+        simulation.nodes.assign(node);
+        simulation.moves.clear();
         thread_state.full_select_counter = m_full_select_interval;
+    }
     thread_state.stat_fs_len.add(double(simulation.moves.size()));
     while (node->has_children())
     {
         node = select_child(*node);
         m_tree.inc_visit_count(*node);
-#ifndef LIBBOARDGAME_MCTS_SINGLE_THREAD
-        if (SearchParamConst::virtual_loss && m_nu_threads > 0)
+        if (multithread && SearchParamConst::virtual_loss && m_nu_threads > 0)
             m_tree.add_value(*node, 0);
-#endif
         simulation.nodes.push_back(node);
         Move mv = node->get_move();
-        simulation.moves.push_back(PlayerMove(state.get_to_play(), mv));
+        simulation.moves.push_back(PlayerMove(state.get_player(), mv));
         state.play_in_tree(mv);
         expand_threshold += m_expand_threshold_inc;
     }
@@ -1183,11 +1205,11 @@ void SearchBase<S, M, R>::play_in_tree(ThreadState& thread_state)
     {
         if (! expand_node(thread_state, *node, node))
             thread_state.is_out_of_mem = true;
-        else if (node != nullptr)
+        else if (node)
         {
             simulation.nodes.push_back(node);
             Move mv = node->get_move();
-            simulation.moves.push_back(PlayerMove(state.get_to_play(), mv));
+            simulation.moves.push_back(PlayerMove(state.get_player(), mv));
             state.play_expanded_child(mv);
         }
     }
@@ -1207,12 +1229,12 @@ string SearchBase<S, M, R>::get_info() const
         s << "No simulations in thread 0\n";
         return s.str();
     }
-    s << fixed << setprecision(2) << "Val: " << root.get_value()
-      << setprecision(0) << ", ValCnt: " << root.get_value_count()
-      << ", VstCnt: " << root.get_visit_count()
+    s << fixed << setprecision(2) << "Val: " << get_root_val().get_mean()
+      << setprecision(0) << ", ValCnt: " << get_root_val().get_count()
+      << ", VstCnt: " << get_root_visit_count()
       << ", Sim: " << m_nu_simulations;
-    auto child = select_child_final(root);
-    if (child != nullptr && root.get_visit_count() > 0)
+    auto child = select_final();
+    if (child && root.get_visit_count() > 0)
         s << setprecision(1) << ", Chld: "
           << (100 * child->get_visit_count() / root.get_visit_count())
           << '%';
@@ -1268,28 +1290,28 @@ bool SearchBase<S, M, R>::prune(TimeSource& time_source, double time,
     }
 }
 
-/** Restore the value and count of a root node from its children.
-    The value of a root node has a different meaning than than values of inner
-    nodes (position vs. move value) so after reusing a subtree, we need to
-    restore it from the children. We use only the child with the highest count
-    to avoid backing up many values of unvisited children that have only a
-    value and count from prior knowledge initialization. */
+/** Estimate the value and count of a root node from its children.
+    After reusing a subtree, we don't know the value of the root because nodes
+    only store the value of moves. To estimate the root value, we use the child
+    with the highest visit count. */
 template<class S, class M, class R>
-void SearchBase<S, M, R>::restore_root_from_children(Tree& tree, const Node& root)
+bool SearchBase<S, M, R>::estimate_reused_root_val(Tree& tree,
+                                                   const Node& root,
+                                                   Float& value, Float& count)
 {
-    const Node* best_child = nullptr;
+    const Node* best = nullptr;
     Float max_count = 0;
-    for (ChildIterator i(tree, root); i; ++i)
-        if (i->get_visit_count() > max_count)
+    for (auto& i : tree.get_children(root))
+        if (i.get_visit_count() > max_count)
         {
-            best_child = &(*i);
-            max_count = i->get_visit_count();
+            best = &i;
+            max_count = i.get_visit_count();
         }
-    if (best_child == nullptr)
-        tree.init_root_value(get_tie_value(), 0);
-    else
-        tree.init_root_value(best_child->get_value(),
-                             best_child->get_value_count());
+    if (! best)
+        return false;
+    value = best->get_value();
+    count = best->get_value_count();
+    return count > 0;
 }
 
 template<class S, class M, class R>
@@ -1305,6 +1327,7 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
         // max_time is still used at some places in the code, so we set it to
         // infinity
         max_time = numeric_limits<double>::max();
+    m_player = get_player();
     m_nu_players = get_nu_players();
     bool clear_tree = true;
     bool is_same = false;
@@ -1313,15 +1336,16 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
         is_same = true;
         is_followup = false;
     }
-    for (PlayerInt i = 0; i < m_nu_players; ++i)
-    {
-        m_init_val[i].clear();
-        m_init_val[i].add(get_tie_value());
-    }
     if (is_same || (is_followup && m_followup_sequence.size() <= m_nu_players))
+    {
+        // Use root_val from last search but with a count of max. 100
         for (PlayerInt i = 0; i < m_nu_players; ++i)
-            if (m_root_val[i].get_count() > 0)
-                m_init_val[i] = m_root_val[i];
+            if (m_root_val[i].get_count() > 100)
+                m_root_val[i].init(m_root_val[i].get_mean(), 100);
+    }
+    else
+        for (PlayerInt i = 0; i < m_nu_players; ++i)
+            m_root_val[i].init(get_tie_value(), 1);
     if ((m_reuse_subtree && is_followup) || (m_reuse_tree && is_same))
     {
         size_t tree_nodes = m_tree.get_nu_nodes();
@@ -1336,7 +1360,7 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
             Timer timer(time_source);
             m_tmp_tree.clear(get_tie_value());
             auto node = find_node(m_tree, m_followup_sequence);
-            if (node != nullptr)
+            if (node)
             {
                 TimeIntervalChecker interval_checker(time_source, max_time);
                 if (m_deterministic)
@@ -1346,7 +1370,12 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
                                              &interval_checker);
                 auto& tmp_tree_root = m_tmp_tree.get_root();
                 if (! is_same)
-                    restore_root_from_children(m_tmp_tree, tmp_tree_root);
+                {
+                    Float value, count;
+                    if (estimate_reused_root_val(m_tmp_tree, tmp_tree_root,
+                                                 value, count))
+                        m_root_val[m_player].add(value, count);
+                }
                 if (aborted && ! always_search)
                     return false;
                 size_t tmp_tree_nodes = m_tmp_tree.get_nu_nodes();
@@ -1372,14 +1401,11 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
 
     m_timer.reset(time_source);
     m_time_source = &time_source;
-    m_player = get_player();
-    for (PlayerInt i = 0; i < m_nu_players; ++i)
-        m_root_val[i].clear();
-    if (SearchParamConst::use_last_good_reply && ! is_followup)
-        m_last_good_reply.init(m_nu_players);
-    for (unsigned i = 0; i < m_threads.size(); ++i)
+    if (SearchParamConst::use_lgr && ! is_followup)
+        m_lgr.init(m_nu_players);
+    for (auto& i : m_threads)
     {
-        auto& thread_state = m_threads[i]->thread_state;
+        auto& thread_state = i->thread_state;
         thread_state.nu_simulations = 0;
         thread_state.full_select_counter = 0;
         thread_state.stat_len.clear();
@@ -1390,7 +1416,7 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
     m_max_count = max_count;
     m_min_simulations = min_simulations;
     m_max_time = max_time;
-    m_nu_simulations = 0;
+    m_nu_simulations.store(0);
     Float prune_min_count = get_prune_count_start();
 
     // Don't use multi-threading for very short searches (less than 0.5s).
@@ -1421,7 +1447,7 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
 
     if (root.get_nu_children() == 0)
         log("No legal moves at root");
-    else if (root.get_nu_children() == 1)
+    else if (root.get_nu_children() == 1 && min_simulations == 0)
         log("Root has only one child");
     else
         while (true)
@@ -1440,11 +1466,6 @@ bool SearchBase<S, M, R>::search(Move& mv, Float max_count,
                 }
             if (! is_out_of_mem)
                 break;
-            if (! m_prune_full_tree)
-            {
-                log("Maximum tree size reached");
-                break;
-            }
             double time = m_timer();
             if (! prune(time_source, time, max_time - time, prune_min_count,
                         prune_min_count))
@@ -1466,13 +1487,15 @@ void SearchBase<S, M, R>::search_loop(ThreadState& thread_state)
 {
     auto& state = *thread_state.state;
     auto& simulation = thread_state.simulation;
-    simulation.last_nodes.clear();
+    simulation.nodes.clear();
+    simulation.moves.clear();
     double time_interval = 0.1;
     if (m_max_count == 0 && m_max_time < 1)
         time_interval = 0.1 * m_max_time;
-    IntervalChecker expensive_abort_checker(*m_time_source, time_interval,
-                                           bind(&SearchBase::check_abort_expensive,
-                                                this, ref(thread_state)));
+    IntervalChecker expensive_abort_checker(
+                *m_time_source, time_interval,
+                bind(&SearchBase::check_abort_expensive, this,
+                     ref(thread_state)));
     if (m_deterministic)
     {
         unsigned interval =
@@ -1483,17 +1506,14 @@ void SearchBase<S, M, R>::search_loop(ThreadState& thread_state)
     {
         thread_state.is_out_of_mem = false;
         auto root_count = m_tree.get_root().get_visit_count();
-        auto nu_simulations =
-            LIBBOARDGAME_MCTS_ATOMIC_FETCH_ADD(m_nu_simulations, 1);
+        auto nu_simulations = m_nu_simulations.fetch_add(1);
         if (root_count > 0 && nu_simulations > m_min_simulations
-            && (check_abort(thread_state) || expensive_abort_checker()))
+                && (check_abort(thread_state) || expensive_abort_checker()))
         {
-            LIBBOARDGAME_MCTS_ATOMIC_FETCH_ADD(m_nu_simulations, -1);
+            m_nu_simulations.fetch_add(-1);
             break;
         }
         ++thread_state.nu_simulations;
-        simulation.nodes.clear();
-        simulation.moves.clear();
         state.start_simulation(nu_simulations);
         play_in_tree(thread_state);
         if (thread_state.is_out_of_mem)
@@ -1504,67 +1524,71 @@ void SearchBase<S, M, R>::search_loop(ThreadState& thread_state)
         update_values(thread_state);
         if (SearchParamConst::rave)
             update_rave(thread_state);
-        if (SearchParamConst::use_last_good_reply)
-            update_last_good_reply(thread_state);
+        if (SearchParamConst::use_lgr)
+            update_lgr(thread_state);
         on_simulation_finished(nu_simulations, state, simulation);
-        simulation.last_nodes = simulation.nodes;
     }
 }
 
 template<class S, class M, class R>
-auto SearchBase<S, M, R>::select_child(const Node& node) -> const Node*
+inline auto SearchBase<S, M, R>::select_child(const Node& node) -> const Node*
 {
-    const Node* best_child = nullptr;
-    Float best_value = -numeric_limits<Float>::max();
-    ChildIterator i(m_tree, node);
-    LIBBOARDGAME_ASSERT(i);
+    auto children = m_tree.get_children(node);
+    LIBBOARDGAME_ASSERT(! children.empty());
     m_bias_term.start_iteration(node.get_visit_count());
-    auto bias_upper_limit = m_bias_term.get_upper_limit();
-    Float limit = -numeric_limits<Float>::max();
-    do
+    // SearchParamConst::child_min_count is currently declared as unsigned int
+    // because MSVC does not support constexpr Float yet
+    auto min_count = static_cast<Float>(SearchParamConst::child_min_count);
+    auto bias_upper_limit = m_bias_term.get(min_count);
+    auto i = children.begin();
+    auto value = i->get_value();
+    value += m_bias_term.get(i->get_value_count());
+    auto best_value = value;
+    auto best_child = i;
+    auto limit = best_value - bias_upper_limit;
+    while (++i != children.end())
     {
-        auto value = i->get_value();
+        value = i->get_value();
         if (value < limit)
             continue;
         value += m_bias_term.get(i->get_value_count());
         if (value > best_value)
         {
             best_value = value;
-            best_child = &(*i);
+            best_child = i;
             limit = best_value - bias_upper_limit;
         }
     }
-    while (++i);
     return best_child;
 }
 
 template<class S, class M, class R>
-auto SearchBase<S, M, R>::select_child_final(
-        const Node& node) const-> const Node*
+auto SearchBase<S, M, R>::select_final() const-> const Node*
 {
-    // Select the child with the highest visit count, use value as tie breaker
-    const Node* result = nullptr;
-    Float max_count = -1;
-    Float max_count_value = -numeric_limits<Float>::max();
-    for (ChildIterator i(m_tree, node); i; ++i)
+    // Select the child with the highest number of wins
+    auto children = m_tree.get_children(m_tree.get_root());
+    if (children.empty())
+        return nullptr;
+    auto i = children.begin();
+    auto best_child = i;
+    auto max_wins = i->get_value_count() * i->get_value();
+    while (++i != children.end())
     {
-        Float count = i->get_visit_count();
-        if (count > max_count
-            || (count == max_count && i->get_value() > max_count_value))
+        auto wins = i->get_value_count() * i->get_value();
+        if (wins > max_wins)
         {
-            max_count = count;
-            max_count_value = i->get_value();
-            result = &(*i);
+            max_wins = wins;
+            best_child = i;
         }
     }
-    return result;
+    return best_child;
 }
 
 template<class S, class M, class R>
 bool SearchBase<S, M, R>::select_move(Move& mv) const
 {
-    auto child = select_child_final(m_tree.get_root());
-    if (child != nullptr)
+    auto child = select_final();
+    if (child)
     {
         mv = child->get_move();
         return true;
@@ -1621,12 +1645,6 @@ void SearchBase<S, M, R>::set_prune_count_start(Float n)
     m_prune_count_start = n;
 }
 
-template<class S, class M, class R>
-void SearchBase<S, M, R>::set_prune_full_tree(bool enable)
-{
-    m_prune_full_tree = enable;
-}
-
 template<class S, class M, class R>
 void SearchBase<S, M, R>::set_rave_dist_final(Float v)
 {
@@ -1673,7 +1691,7 @@ void SearchBase<S, M, R>::set_tree_memory(size_t memory)
 }
 
 template<class S, class M, class R>
-void SearchBase<S, M, R>::update_last_good_reply(ThreadState& thread_state)
+void SearchBase<S, M, R>::update_lgr(ThreadState& thread_state)
 {
     const auto& simulation = thread_state.simulation;
     auto& eval = simulation.eval;
@@ -1685,26 +1703,26 @@ void SearchBase<S, M, R>::update_last_good_reply(ThreadState& thread_state)
         // Note: this handles a draw as a win. Without additional information
         // we cannot make a good decision how to handle draws and some
         // experiments in Blokus Duo showed (with low confidence) that treating
-        // them as a win for both players is slighly better than treating them
+        // them as a win for both players is slightly better than treating them
         // as a loss for both.
         is_winner[i] = (eval[i] == max_eval);
     auto& moves = simulation.moves;
     auto nu_moves = moves.size();
     if (nu_moves < 2)
         return;
-    Move last_mv = moves[0].move;
-    Move second_last_mv = Move::null();
+    Move last = moves[0].move;
+    Move second_last = Move::null();
     for (unsigned i = 1; i < nu_moves; ++i)
     {
         PlayerMove reply = moves[i];
         PlayerInt player = reply.player;
         Move mv = reply.move;
         if (is_winner[player])
-            m_last_good_reply.store(player, last_mv, second_last_mv, mv);
+            m_lgr.store(player, last, second_last, mv);
         else
-            m_last_good_reply.forget(player, last_mv, second_last_mv, mv);
-        second_last_mv = last_mv;
-        last_mv = mv;
+            m_lgr.forget(player, last, second_last, mv);
+        second_last = last;
+        last = mv;
     }
 }
 
@@ -1747,8 +1765,9 @@ void SearchBase<S, M, R>::update_rave(ThreadState& thread_state)
         Float dist_weight_factor;
         if (SearchParamConst::rave_dist_weighting)
             dist_weight_factor = (1 - m_rave_dist_final) / Float(nu_moves - i);
-        ChildIterator it(m_tree, *node);
-        LIBBOARDGAME_ASSERT(it);
+        auto children = m_tree.get_children(*node);
+        LIBBOARDGAME_ASSERT(! children.empty());
+        auto it = children.begin();
         do
         {
             auto mv = it->get_move();
@@ -1778,7 +1797,7 @@ void SearchBase<S, M, R>::update_rave(ThreadState& thread_state)
                 weight *= 1 - Float(first - i) * dist_weight_factor;
             m_tree.add_value(*it, thread_state.simulation.eval[player], weight);
         }
-        while (++it);
+        while (++it != children.end());
         if (i == 0)
             break;
         if (! state.skip_rave(mv.move))
@@ -1806,23 +1825,25 @@ void SearchBase<S, M, R>::update_values(ThreadState& thread_state)
     const auto& simulation = thread_state.simulation;
     auto& nodes = simulation.nodes;
     auto& eval = simulation.eval;
-    m_tree.add_value(m_tree.get_root(), eval[m_player]);
     unsigned nu_nodes = static_cast<unsigned>(nodes.size());
     for (unsigned i = 1; i < nu_nodes; ++i)
     {
         auto& node = *nodes[i];
         auto mv = simulation.moves[i - 1];
         m_tree.add_value(node, eval[mv.player]);
-#ifndef LIBBOARDGAME_MCTS_SINGLE_THREAD
-        if (SearchParamConst::virtual_loss && m_nu_threads > 0)
+        if (multithread && SearchParamConst::virtual_loss && m_nu_threads > 0)
+            // Note that this could become problematic if the number of threads
+            // is large. The lock-free algorithm intentionally ignores lost or
+            // partial updates to run faster. But the probability that adding
+            // a virtual loss is lost is not the same as the one that its
+            // removal is lost because the removal is done in this function
+            // with many calls to add_value() but the adding is done in
+            // play_in_tree(). This could introduce a systematic error unlike
+            // the effect of lost or partial add_value() updates.
             m_tree.remove_value(node, 0);
-#endif
     }
     for (PlayerInt i = 0; i < m_nu_players; ++i)
-    {
         m_root_val[i].add(eval[i]);
-        m_init_val[i].add(eval[i]);
-    }
 }
 
 //-----------------------------------------------------------------------------
