@@ -32,6 +32,8 @@ using libboardgame_util::IntervalChecker;
 template<typename N>
 class Tree
 {
+    struct ThreadStorage;
+
     friend class NodeExpander;
 
 public:
@@ -41,6 +43,40 @@ public:
 
     typedef typename Node::Float Float;
 
+    /** Range for iterating over the children of a node. */
+    class Children
+    {
+    public:
+        Children(const Tree& tree, const Node& node)
+        {
+            auto nu_children = node.get_nu_children();
+            m_begin = (nu_children != 0 ?
+                        &tree.get_node(node.get_first_child()) : nullptr);
+            m_end = m_begin + nu_children;
+        }
+
+        const Node* begin() const
+        {
+            return m_begin;
+        }
+
+        const Node* end() const
+        {
+            return m_end;
+        }
+
+        bool empty() const
+        {
+            return m_begin == nullptr;
+        }
+
+    private:
+        const Node* m_begin;
+
+        const Node* m_end;
+    };
+
+
     /** Helper class that is passed to the search state during node expansion.
         This class allows the search state to directly create children of a
         node at the node expansion, so that copying to a temporary move list
@@ -49,14 +85,21 @@ public:
     class NodeExpander
     {
     public:
-        NodeExpander(unsigned thread_id, Tree& tree, const Node& node);
+        /** Constructor.
+            @param thread_id
+            @param tree
+            @param child_min_count The minimum count used for initializing
+            children. Used only in debug mode to assert that the children
+            are really initialized with a minimum count as declared with
+            SearchParamConst::child_min_count. */
+        NodeExpander(unsigned thread_id, Tree& tree, Float child_min_count);
 
         /** Add new child.
             The child will only be added if the tree is not full. */
         void add_child(const Move& mv, Float value, Float count);
 
         /** Link the children to the parent node. */
-        void link_children();
+        void link_children(Tree& tree, const Node& node);
 
         /** Return if the tree capacity was reached during an add_child(). */
         bool is_tree_full() const;
@@ -69,21 +112,19 @@ public:
         const Node* get_best_child() const;
 
     private:
-        unsigned m_thread_id;
+        ThreadStorage& m_thread_storage;
 
-        bool m_is_tree_full;
+        unsigned short m_nu_children = 0;
 
-        unsigned short m_nu_children;
-
-        Float m_best_value;
-
-        Tree& m_tree;
-
-        const Node& m_node;
+        Float m_best_value = 0;
 
         const Node* m_first_child;
 
         const Node* m_best_child;
+
+#if LIBBOARDGAME_DEBUG
+        Float m_child_min_count;
+#endif
     };
 
     Tree(size_t max_nodes, unsigned nu_threads);
@@ -96,6 +137,16 @@ public:
 
     const Node& get_root() const;
 
+    Children get_children(const Node& node) const
+    {
+        return Children(*this, node);
+    }
+
+    Children get_root_children() const
+    {
+        return get_children(get_root());
+    }
+
     size_t get_nu_nodes() const;
 
     const Node& get_node(NodeIdx i) const;
@@ -107,9 +158,6 @@ public:
 
     size_t get_max_nodes() const;
 
-    bool create_node(unsigned thread_id, const Move& mv, Float value,
-                     Float count);
-
     void add_value(const Node& node, Float v);
 
     void add_value(const Node& node, Float v, Float weight);
@@ -141,7 +189,7 @@ public:
         @return @c false if the extraction was aborted. */
     bool extract_subtree(Tree& target, const Node& node,
                          bool check_abort = false,
-                         IntervalChecker* interval_checker = 0) const;
+                         IntervalChecker* interval_checker = nullptr) const;
 
     /** Copy a subtree.
         This operation can be lengthy and can be aborted by providing an abort
@@ -158,7 +206,7 @@ public:
         @return @c false if the copying was aborted. */
     bool copy_subtree(Tree& target, const Node& target_node, const Node& node,
                       Float min_count = 0, bool check_abort = false,
-                      IntervalChecker* interval_checker = 0) const;
+                      IntervalChecker* interval_checker = nullptr) const;
 
 private:
     struct ThreadStorage
@@ -189,34 +237,36 @@ private:
 
 template<typename N>
 inline Tree<N>::NodeExpander::NodeExpander(unsigned thread_id, Tree& tree,
-                                           const Node& node)
-    : m_thread_id(thread_id),
-      m_is_tree_full(false),
-      m_nu_children(0),
-      m_best_value(0),
-      m_tree(tree),
-      m_node(node),
-      m_first_child(m_tree.m_thread_storage[thread_id].next),
+                                           Float child_min_count)
+    : m_thread_storage(tree.m_thread_storage[thread_id]),
+      m_first_child(m_thread_storage.next),
       m_best_child(m_first_child)
 {
+    LIBBOARDGAME_ASSERT(thread_id < tree.m_nu_threads);
+#if LIBBOARDGAME_DEBUG
+    m_child_min_count = child_min_count;
+#else
+    LIBBOARDGAME_UNUSED(child_min_count);
+#endif
 }
 
 template<typename N>
 inline void Tree<N>::NodeExpander::add_child(const Move& mv, Float value,
                                              Float count)
 {
-    if (! (m_is_tree_full |= ! m_tree.create_node(m_thread_id, mv, value,
-                                                  count)))
+    LIBBOARDGAME_ASSERT(count >= m_child_min_count);
+    if (m_thread_storage.next >= m_thread_storage.end)
+        return;
+    m_thread_storage.next->init(mv, value, count);
+    ++m_thread_storage.next;
+    if (m_nu_children == 0)
+        m_best_value = value;
+    else if (value > m_best_value)
     {
-        if (m_nu_children == 0)
-            m_best_value = value;
-        else if (value > m_best_value)
-        {
-            m_best_child = m_first_child + m_nu_children;
-            m_best_value = value;
-        }
-        ++m_nu_children;
+        m_best_child = m_first_child + m_nu_children;
+        m_best_value = value;
     }
+    ++m_nu_children;
 }
 
 template<typename N>
@@ -237,14 +287,14 @@ inline auto Tree<N>::get_node(NodeIdx i) const -> const Node&
 template<typename N>
 inline bool Tree<N>::NodeExpander::is_tree_full() const
 {
-    return m_is_tree_full;
+    return m_thread_storage.next >= m_thread_storage.end;
 }
 
 template<typename N>
-inline void Tree<N>::NodeExpander::link_children()
+inline void Tree<N>::NodeExpander::link_children(Tree& tree, const Node& node)
 {
     if (m_nu_children > 0)
-        m_tree.link_children(m_node, m_first_child, m_nu_children);
+        tree.link_children(node, m_first_child, m_nu_children);
 }
 
 
@@ -284,7 +334,7 @@ void Tree<N>::clear(Float root_value)
 template<typename N>
 bool Tree<N>::contains(const Node& node) const
 {
-    return (&node >= m_nodes.get() && &node < m_nodes.get() + m_max_nodes);
+    return &node >= m_nodes.get() && &node < m_nodes.get() + m_max_nodes;
 }
 
 template<typename N>
@@ -300,7 +350,7 @@ bool Tree<N>::copy_subtree(Tree& target, const Node& target_node,
     target_node_non_const.copy_data_from(node);
     bool abort =
         (check_abort && get_abort())
-        || (interval_checker != nullptr && (*interval_checker)());
+        || (interval_checker && (*interval_checker)());
     if (! node.has_children()
             || (node.get_visit_count() < min_count && &node != &get_root())
             || abort)
@@ -334,21 +384,6 @@ bool Tree<N>::copy_subtree(Tree& target, const Node& target_node,
     return ! abort;
 }
 
-template<typename N>
-bool Tree<N>::create_node(unsigned thread_id, const Move& mv,
-                          Float value, Float count)
-{
-    LIBBOARDGAME_ASSERT(thread_id < m_nu_threads);
-    auto& thread_storage = m_thread_storage[thread_id];
-    if (thread_storage.next != thread_storage.end)
-    {
-        thread_storage.next->init(mv, value, count);
-        ++thread_storage.next;
-        return true;
-    }
-    return false;
-}
-
 template<typename N>
 bool Tree<N>::extract_subtree(Tree& target, const Node& node,
                               bool check_abort,
