@@ -7,6 +7,7 @@
 #ifndef LIBBOARDGAME_SGF_SGF_NODE_H
 #define LIBBOARDGAME_SGF_SGF_NODE_H
 
+#include <forward_list>
 #include <memory>
 #include <string>
 #include <vector>
@@ -30,7 +31,20 @@ struct Property
 
     unique_ptr<Property> next;
 
-    Property(const string& id, const vector<string>& values);
+    Property(const Property& p)
+        : id(p.id),
+          values(p.values)
+    { }
+
+    Property(const string& id, const vector<string>& values)
+        : id(id),
+          values(values)
+    {
+        LIBBOARDGAME_ASSERT(! id.empty());
+        LIBBOARDGAME_ASSERT(! values.empty());
+    }
+
+    ~Property();
 };
 
 //-----------------------------------------------------------------------------
@@ -38,6 +52,77 @@ struct Property
 class SgfNode
 {
 public:
+    /** Iterates over siblings. */
+    class Iterator
+    {
+    public:
+        Iterator(const SgfNode* node)
+        {
+            m_node = node;
+        }
+
+        bool operator==(Iterator it) const
+        {
+            return m_node == it.m_node;
+        }
+
+        bool operator!=(Iterator it) const
+        {
+            return m_node != it.m_node;
+        }
+
+        Iterator& operator++()
+        {
+            m_node = m_node->get_sibling();
+            return *this;
+        }
+
+        const SgfNode& operator*() const
+        {
+            return *m_node;
+        }
+
+        const SgfNode* operator->() const
+        {
+            return m_node;
+        }
+
+        bool is_null() const
+        {
+            return m_node == nullptr;
+        }
+
+    private:
+        const SgfNode* m_node;
+    };
+
+    /** Range for iterating over the children of a node. */
+    class Children
+    {
+    public:
+        Children(const SgfNode& node)
+            : m_begin(node.get_first_child_or_null())
+        { }
+
+        Iterator begin() const
+        {
+            return m_begin;
+        }
+
+        Iterator end() const
+        {
+            return nullptr;
+        }
+
+        bool empty() const
+        {
+            return m_begin.is_null();
+        }
+
+    private:
+        Iterator m_begin;
+    };
+
     SgfNode();
 
     ~SgfNode();
@@ -81,9 +166,19 @@ public:
     /** @return true, if node contained the property. */
     bool remove_property(const string& id);
 
+    /** @return true, if the property was found and not already at the
+        front. */
     bool move_property_to_front(const string& id);
 
-    const Property* get_first_property() const;
+    const forward_list<Property>& get_properties() const
+    {
+        return m_properties;
+    }
+
+    Children get_children() const
+    {
+        return Children(*this);
+    }
 
     SgfNode* get_sibling();
 
@@ -153,15 +248,19 @@ public:
     void delete_variations();
 
 private:
-    SgfNode* m_parent;
+    SgfNode* m_parent = nullptr;
 
     unique_ptr<SgfNode> m_first_child;
 
     unique_ptr<SgfNode> m_sibling;
 
-    unique_ptr<Property> m_first_property;
+    /** The properties.
+        Often a node has only one property (the move), so it saves memory
+        to use a forward_list instead of a vector. */
+    forward_list<Property> m_properties;
 
-    Property* find_property(const string& id) const;
+    forward_list<Property>::const_iterator find_property(
+            const string& id) const;
 
     SgfNode* get_last_child() const;
 };
@@ -211,11 +310,6 @@ inline const SgfNode* SgfNode::get_first_child_or_null() const
     return m_first_child.get();
 }
 
-inline const Property* SgfNode::get_first_property() const
-{
-    return m_first_property.get();
-}
-
 inline SgfNode* SgfNode::get_sibling()
 {
     return m_sibling.get();
@@ -238,7 +332,7 @@ inline bool SgfNode::has_parent() const
 
 inline bool SgfNode::has_single_child() const
 {
-    return (m_first_child && ! m_first_child->m_sibling);
+    return m_first_child && ! m_first_child->m_sibling;
 }
 
 template<typename T>
@@ -284,130 +378,21 @@ bool SgfNode::set_property(const string& id, const vector<T>& values)
     vector<string> values_to_string;
     for (const T& v : values)
         values_to_string.push_back(to_string(v));
-    auto property = m_first_property.get();
-    if (property == nullptr)
-    {
-        m_first_property.reset(new Property(id, values_to_string));
-        return true;
-    }
-    while (true)
-    {
-        if (property->id == id)
+    forward_list<Property>::const_iterator last = m_properties.end();
+    for (auto i = m_properties.begin(); i != m_properties.end(); ++i)
+        if (i->id == id)
         {
-            bool was_changed = (property->values != values_to_string);
-            property->values = values_to_string;
+            bool was_changed = (i->values != values_to_string);
+            i->values = values_to_string;
             return was_changed;
         }
-        if (! property->next)
-        {
-            property->next.reset(new Property(id, values_to_string));
-            return true;
-        }
-        property = property->next.get();
-    }
-}
-
-//-----------------------------------------------------------------------------
-
-class PropertyIterator
-{
-public:
-    PropertyIterator(const SgfNode& node);
-
-    operator bool() const;
-
-    void operator++();
-
-    const Property& operator*() const;
-
-    const Property* operator->() const;
-
-private:
-    const Property* m_current;
-};
-
-inline PropertyIterator::PropertyIterator(const SgfNode& node)
-{
-    m_current = node.get_first_property();
-}
-
-inline PropertyIterator::operator bool() const
-{
-    return m_current != nullptr;
-}
-
-inline void PropertyIterator::operator++()
-{
-    LIBBOARDGAME_ASSERT(*this);
-    m_current = m_current->next.get();
-}
-
-inline const Property& PropertyIterator::operator*() const
-{
-    LIBBOARDGAME_ASSERT(*this);
-    return *m_current;
-}
-
-inline const Property* PropertyIterator::operator->() const
-{
-    LIBBOARDGAME_ASSERT(*this);
-    return m_current;
-}
-
-//-----------------------------------------------------------------------------
-
-class ChildIterator
-{
-public:
-    ChildIterator(const SgfNode& node);
-
-    operator bool() const;
-
-    void operator++();
-
-    void operator--();
-
-    const SgfNode& operator*() const;
-
-    const SgfNode* operator->() const;
-
-private:
-    const SgfNode* m_current;
-};
-
-inline ChildIterator::ChildIterator(const SgfNode& node)
-{
-    m_current = node.get_first_child_or_null();
-}
-
-inline ChildIterator::operator bool() const
-{
-    return m_current != nullptr;
-}
-
-inline void ChildIterator::operator++()
-{
-    LIBBOARDGAME_ASSERT(*this);
-    m_current = m_current->get_sibling();
-}
-
-inline void ChildIterator::operator--()
-{
-    LIBBOARDGAME_ASSERT(*this);
-    m_current = m_current->get_previous_sibling();
-    LIBBOARDGAME_ASSERT(*this);
-}
-
-inline const SgfNode& ChildIterator::operator*() const
-{
-    LIBBOARDGAME_ASSERT(*this);
-    return *m_current;
-}
-
-inline const SgfNode* ChildIterator::operator->() const
-{
-    LIBBOARDGAME_ASSERT(*this);
-    return m_current;
+        else
+            last = i;
+    if (last == m_properties.end())
+        m_properties.emplace_front(id, values_to_string);
+    else
+        m_properties.emplace_after(last, id, values_to_string);
+    return true;
 }
 
 //-----------------------------------------------------------------------------
