@@ -33,7 +33,7 @@ public:
     virtual void init();
 
     /** Initialize from an existing SGF tree.
-        @param root The root node of the SGF tree; the ownership is transfered
+        @param root The root node of the SGF tree; the ownership is transferred
         to this class. */
     virtual void init(unique_ptr<SgfNode>& root);
 
@@ -101,7 +101,7 @@ public:
 
     bool remove_property(const SgfNode& node, const string& id);
 
-    bool move_property_to_front(const SgfNode& node, const string& id);
+    void move_property_to_front(const SgfNode& node, const string& id);
 
     /** See Node::remove_children() */
     unique_ptr<SgfNode> remove_children(const SgfNode& node);
