@@ -19,8 +19,6 @@ using libboardgame_util::log;
 
 namespace libboardgame_test {
 
-using namespace std;
-
 //-----------------------------------------------------------------------------
 
 namespace {
@@ -43,7 +41,7 @@ string get_fail_msg(const char* file, int line, const string& s)
 //-----------------------------------------------------------------------------
 
 TestFail::TestFail(const char* file, int line, const string& s)
-    : Exception(get_fail_msg(file, line, s))
+    : logic_error(get_fail_msg(file, line, s))
 {
 }
 
@@ -60,12 +58,11 @@ bool run_all_tests()
 {
     unsigned nu_fail = 0;
     log("Running ", get_all_tests().size(), " tests...");
-    auto all_tests = get_all_tests();
-    for (auto i = all_tests.begin(); i != all_tests.end(); ++i)
+    for (auto& i : get_all_tests())
     {
         try
         {
-            (i->second)();
+            (i.second)();
         }
         catch (const TestFail& e)
         {
@@ -87,14 +84,13 @@ bool run_all_tests()
 
 bool run_test(const string& name)
 {
-    auto all_tests = get_all_tests();
-    for (auto i = all_tests.begin(); i != all_tests.end(); ++i)
-        if (i->first == name)
+    for (auto& i : get_all_tests())
+        if (i.first == name)
         {
             log("Running ", name, "...");
             try
             {
-                (i->second)();
+                (i.second)();
                 log("OK");
                 return true;
             }
