@@ -8,10 +8,9 @@
 #define LIBBOARDGAME_UTIL_ARRAY_LIST_H
 
 #include <algorithm>
-#include <cstddef>
+#include <array>
 #include <initializer_list>
-#include <iosfwd>
-#include <limits>
+#include <iostream>
 #include "Assert.h"
 
 namespace libboardgame_util {
@@ -23,7 +22,8 @@ using namespace std;
 /** Array-based list with maximum number of elements.
     The user is responsible for not inserting more than the maximum number of
     elements. The elements must be default-constructible. If the size of the
-    list shrinks, the destructor of elements will be not be called immediately.
+    list shrinks, the destructor of elements will be not be called and the
+    elements above the current size are still usable with get_unchecked().
     The list contains iterator definitions that are compatible with STL
     containers.
     @tparam T The type of the elements
@@ -33,16 +33,18 @@ template<typename T, unsigned M, typename I = unsigned>
 class ArrayList
 {
 public:
-    static_assert(numeric_limits<I>::is_integer, "");
+    typedef I IntType;
 
-    typedef T* iterator;
+    static_assert(numeric_limits<IntType>::is_integer, "");
 
-    typedef const T* const_iterator;
+    static const IntType max_size = M;
+
+    typedef typename array<T, max_size>::iterator iterator;
+
+    typedef typename array<T, max_size>::const_iterator const_iterator;
 
     typedef T value_type;
 
-    static const I max_size = M;
-
     ArrayList();
 
     /** Construct list with a single element. */
@@ -50,6 +52,10 @@ public:
 
     ArrayList(const initializer_list<T>& l);
 
+    /** Assignment operator.
+        Copies only elements with index below the current size. */
+    ArrayList& operator=(const ArrayList& l);
+
     T& operator[](I i);
 
     const T& operator[](I i) const;
@@ -117,19 +123,17 @@ public:
     bool remove_fast(const T& t);
 
 private:
-    I m_size;
+    array<T, max_size> m_a;
 
-    T m_a[max_size];
+    I m_size = 0;
 };
 
 template<typename T, unsigned M, typename I>
-inline ArrayList<T,M,I>::ArrayList()
-    : m_size(0)
-{
-}
+inline ArrayList<T, M, I>::ArrayList()
+{ }
 
 template<typename T, unsigned M, typename I>
-inline ArrayList<T,M,I>::ArrayList(const T& t)
+inline ArrayList<T, M, I>::ArrayList(const T& t)
 {
     assign(t);
 }
@@ -143,100 +147,100 @@ ArrayList<T, M, I>::ArrayList(const initializer_list<T>& l)
 }
 
 template<typename T, unsigned M, typename I>
-inline T& ArrayList<T,M,I>::operator[](I i)
+auto ArrayList<T, M, I>::operator=(const ArrayList& l) -> ArrayList&
+{
+    m_size = l.size();
+    copy(l.begin(), l.end(), begin());
+    return *this;
+}
+
+template<typename T, unsigned M, typename I>
+inline T& ArrayList<T, M, I>::operator[](I i)
 {
     LIBBOARDGAME_ASSERT(i < m_size);
     return m_a[i];
 }
 
 template<typename T, unsigned M, typename I>
-inline const T& ArrayList<T,M,I>::operator[](I i) const
+inline const T& ArrayList<T, M, I>::operator[](I i) const
 {
     LIBBOARDGAME_ASSERT(i < m_size);
     return m_a[i];
 }
 
 template<typename T, unsigned M, typename I>
-bool ArrayList<T,M,I>::operator==(const ArrayList& array_list) const
+bool ArrayList<T, M, I>::operator==(const ArrayList& array_list) const
 {
     if (m_size != array_list.m_size)
         return false;
-    const T* elem_this = m_a;
-    const T* elem_other = array_list.m_a;
-    for (I i = 0; i < m_size; ++i, ++elem_this, ++elem_other)
-        if (*elem_this != *elem_other)
-            return false;
-    return true;
+    return equal(begin(), end(), array_list.begin());
 }
 
 template<typename T, unsigned M, typename I>
-bool ArrayList<T,M,I>::operator!=(const ArrayList& array_list) const
+bool ArrayList<T, M, I>::operator!=(const ArrayList& array_list) const
 {
     return ! operator==(array_list);
 }
 
 template<typename T, unsigned M, typename I>
-inline void ArrayList<T,M,I>::assign(const T& t)
+inline void ArrayList<T, M, I>::assign(const T& t)
 {
     m_size = 1;
     m_a[0] = t;
 }
 
 template<typename T, unsigned M, typename I>
-inline T& ArrayList<T,M,I>::back()
+inline T& ArrayList<T, M, I>::back()
 {
     LIBBOARDGAME_ASSERT(m_size > 0);
     return m_a[m_size - 1];
 }
 
 template<typename T, unsigned M, typename I>
-inline const T& ArrayList<T,M,I>::back() const
+inline const T& ArrayList<T, M, I>::back() const
 {
     LIBBOARDGAME_ASSERT(m_size > 0);
     return m_a[m_size - 1];
 }
 
 template<typename T, unsigned M, typename I>
-inline auto ArrayList<T,M,I>::begin() -> iterator
+inline auto ArrayList<T, M, I>::begin() -> iterator
 {
-    return m_a;
+    return m_a.begin();
 }
 
 template<typename T, unsigned M, typename I>
-inline auto ArrayList<T,M,I>::begin() const -> const_iterator
+inline auto ArrayList<T, M, I>::begin() const -> const_iterator
 {
-    return m_a;
+    return m_a.begin();
 }
 
 template<typename T, unsigned M, typename I>
-inline void ArrayList<T,M,I>::clear()
+inline void ArrayList<T, M, I>::clear()
 {
     m_size = 0;
 }
 
 template<typename T, unsigned M, typename I>
-bool ArrayList<T,M,I>::contains(const T& t) const
+bool ArrayList<T, M, I>::contains(const T& t) const
 {
-    for (const_iterator i = begin(); i != end(); ++i)
-        if (*i == t)
-            return true;
-    return false;
+    return find(begin(), end(), t) != end();
 }
 
 template<typename T, unsigned M, typename I>
-inline bool ArrayList<T,M,I>::empty() const
+inline bool ArrayList<T, M, I>::empty() const
 {
     return m_size == 0;
 }
 
 template<typename T, unsigned M, typename I>
-inline auto ArrayList<T,M,I>::end() -> iterator
+inline auto ArrayList<T, M, I>::end() -> iterator
 {
     return begin() + m_size;
 }
 
 template<typename T, unsigned M, typename I>
-inline auto ArrayList<T,M,I>::end() const -> const_iterator
+inline auto ArrayList<T, M, I>::end() const -> const_iterator
 {
     return begin() + m_size;
 }
@@ -256,37 +260,33 @@ inline const T& ArrayList<T, M, I>::get_unchecked(I i) const
 }
 
 template<typename T, unsigned M, typename I>
-bool ArrayList<T,M,I>::include(const T& t)
+bool ArrayList<T, M, I>::include(const T& t)
 {
-    iterator i;
-    for (i = begin(); i != end(); ++i)
-        if (*i == t)
-            return false;
-    LIBBOARDGAME_ASSERT(m_size < max_size);
-    *i = t;
-    ++m_size;
+    if (contains(t))
+        return false;
+    push_back(t);
     return true;
 }
 
 template<typename T, unsigned M, typename I>
-inline const T& ArrayList<T,M,I>::pop_back()
+inline const T& ArrayList<T, M, I>::pop_back()
 {
     LIBBOARDGAME_ASSERT(m_size > 0);
     return m_a[--m_size];
 }
 
 template<typename T, unsigned M, typename I>
-inline void ArrayList<T,M,I>::push_back(const T& t)
+inline void ArrayList<T, M, I>::push_back(const T& t)
 {
     LIBBOARDGAME_ASSERT(m_size < max_size);
     m_a[m_size++] = t;
 }
 
 template<typename T, unsigned M, typename I>
-inline bool ArrayList<T,M,I>::remove(const T& t)
+inline bool ArrayList<T, M, I>::remove(const T& t)
 {
-    T* end = this->end();
-    for (T* i = m_a; i != end; ++i)
+    auto end = this->end();
+    for (auto i = begin(); i != end; ++i)
         if (*i == t)
         {
             --end;
@@ -299,10 +299,10 @@ inline bool ArrayList<T,M,I>::remove(const T& t)
 }
 
 template<typename T, unsigned M, typename I>
-inline bool ArrayList<T,M,I>::remove_fast(const T& t)
+inline bool ArrayList<T, M, I>::remove_fast(const T& t)
 {
-    T* end = this->end();
-    for (T* i = m_a; i != end; ++i)
+    auto end = this->end();
+    for (auto i = this->begin(); i != end; ++i)
         if (*i == t)
         {
             remove_fast(i);
@@ -312,23 +312,23 @@ inline bool ArrayList<T,M,I>::remove_fast(const T& t)
 }
 
 template<typename T, unsigned M, typename I>
-inline void ArrayList<T,M,I>::remove_fast(iterator i)
+inline void ArrayList<T, M, I>::remove_fast(iterator i)
 {
     LIBBOARDGAME_ASSERT(i >= begin());
     LIBBOARDGAME_ASSERT(i < end());
     --m_size;
-    *i = *(m_a + m_size);
+    *i = *(begin() + m_size);
 }
 
 template<typename T, unsigned M, typename I>
-inline void ArrayList<T,M,I>::resize(I size)
+inline void ArrayList<T, M, I>::resize(I size)
 {
     LIBBOARDGAME_ASSERT(size <= max_size);
     m_size = size;
 }
 
 template<typename T, unsigned M, typename I>
-inline I ArrayList<T,M,I>::size() const
+inline I ArrayList<T, M, I>::size() const
 {
     return m_size;
 }
@@ -336,7 +336,7 @@ inline I ArrayList<T,M,I>::size() const
 //-----------------------------------------------------------------------------
 
 template<typename T, unsigned M, typename I>
-ostream& operator<<(ostream& out, const ArrayList<T,M,I>& l)
+ostream& operator<<(ostream& out, const ArrayList<T, M, I>& l)
 {
     auto begin = l.begin();
     auto end = l.end();
