@@ -168,11 +168,11 @@ template<typename FLOAT>
 inline FLOAT Statistics<FLOAT>::get_deviation() const
 {
     // m_variance can become negative (due to rounding errors?)
-    return (m_variance < 0 ? 0 : sqrt(m_variance));
+    return m_variance < 0 ? 0 : sqrt(m_variance);
 }
 
 template<typename FLOAT>
-inline FLOAT Statistics<FLOAT>::get_error() const
+FLOAT Statistics<FLOAT>::get_error() const
 {
     auto count = get_count();
     return count == 0 ? 0 : get_deviation() / sqrt(count);
@@ -353,10 +353,12 @@ public:
 
     StatisticsDirtyLockFree& operator=(const StatisticsDirtyLockFree& s);
 
-    void add(FLOAT val);
+    void add(FLOAT val, FLOAT weight = 1);
 
     void clear(FLOAT init_val = 0);
 
+    void init(FLOAT mean, FLOAT count);
+
     FLOAT get_count() const;
 
     FLOAT get_mean() const;
@@ -386,12 +388,12 @@ StatisticsDirtyLockFree<FLOAT>::operator=(const StatisticsDirtyLockFree& s)
 }
 
 template<typename FLOAT>
-void StatisticsDirtyLockFree<FLOAT>::add(FLOAT val)
+void StatisticsDirtyLockFree<FLOAT>::add(FLOAT val, FLOAT weight)
 {
     FLOAT count = m_count.load(memory_order_relaxed);
     FLOAT mean = m_mean.load(memory_order_relaxed);
-    ++count;
-    mean +=  (val - mean) / count;
+    count += weight;
+    mean +=  weight * (val - mean) / count;
     m_mean.store(mean, memory_order_relaxed);
     m_count.store(count, memory_order_relaxed);
 }
@@ -399,8 +401,7 @@ void StatisticsDirtyLockFree<FLOAT>::add(FLOAT val)
 template<typename FLOAT>
 inline void StatisticsDirtyLockFree<FLOAT>::clear(FLOAT init_val)
 {
-    m_count = 0;
-    m_mean = init_val;
+    init(init_val, 0);
 }
 
 template<typename FLOAT>
@@ -415,6 +416,13 @@ inline FLOAT StatisticsDirtyLockFree<FLOAT>::get_mean() const
     return m_mean.load(memory_order_relaxed);
 }
 
+template<typename FLOAT>
+inline void StatisticsDirtyLockFree<FLOAT>::init(FLOAT mean, FLOAT count)
+{
+    m_count = count;
+    m_mean = mean;
+}
+
 template<typename FLOAT>
 void StatisticsDirtyLockFree<FLOAT>::write(ostream& out, bool fixed,
                                            unsigned precision) const
