@@ -33,12 +33,12 @@ public:
 private:
     double m_start;
 
-    TimeSource* m_time_source;
+    TimeSource* m_time_source = nullptr;
 };
 
 inline double Timer::operator()() const
 {
-    LIBBOARDGAME_ASSERT(m_time_source != nullptr);
+    LIBBOARDGAME_ASSERT(m_time_source);
     return (*m_time_source)() - m_start;
 }
 
