@@ -15,8 +15,6 @@
 
 namespace libpentobi_base {
 
-using namespace std;
-
 //-----------------------------------------------------------------------------
 
 namespace {
@@ -59,6 +57,10 @@ Board::Board(Variant variant)
     m_color_char[Color(3)] = '@';
     init_variant(variant);
     init();
+#if LIBBOARDGAME_DEBUG
+    m_snapshot.moves_size =
+            numeric_limits<decltype(m_snapshot.moves_size)>::max();
+#endif
 }
 
 void Board::copy_from(const Board& bd)
@@ -68,62 +70,36 @@ void Board::copy_from(const Board& bd)
     m_moves = bd.m_moves;
     m_setup.to_play = bd.m_setup.to_play;
     m_state_base = bd.m_state_base;
-    for (ColorIterator i(m_nu_colors); i; ++i)
+    for (Color c : get_colors())
     {
-        m_state_color[*i] = bd.m_state_color[*i];
-        m_setup.placements[*i] = bd.m_setup.placements[*i];
-        m_attach_points[*i] = bd.m_attach_points[*i];
+        m_state_color[c] = bd.m_state_color[c];
+        m_setup.placements[c] = bd.m_setup.placements[c];
+        m_attach_points[c] = bd.m_attach_points[c];
     }
 }
 
-void Board::gen_moves(Color c, ArrayList<Move, Move::range>& moves) const
+void Board::gen_moves(Color c, MoveMarker& marker, MoveList& moves) const
 {
     moves.clear();
     if (is_first_piece(c))
     {
         for (Point p : get_starting_points(c))
             if (! m_state_color[c].forbidden[p])
-                gen_moves(c, p, m_marker, moves);
+                gen_moves(c, p, get_adj_status(p, c), marker, moves);
     }
     else
     {
-        for (Iterator i(*this); i; ++i)
-            if (is_attach_point(*i, c) && ! m_state_color[c].forbidden[*i])
-                gen_moves(c, *i, get_adj_status(*i, c), m_marker, moves);
+        for (Point p : get_attach_points(c))
+            if (! m_state_color[c].forbidden[p])
+                gen_moves(c, p, get_adj_status(p, c), marker, moves);
     }
-    m_marker.clear(moves);
-}
-
-void Board::gen_moves(Color c, Point p,
-                      ArrayList<Move, Move::range>& moves) const
-{
-    moves.clear();
-    gen_moves(c, p, m_marker, moves);
-    m_marker.clear(moves);
-}
-
-void Board::gen_moves(Color c, Point p, MoveMarker& marker,
-                      ArrayList<Move, Move::range>& moves) const
-{
-    for (Piece piece : m_state_color[c].pieces_left)
-        for (Move mv : m_board_const->get_moves(piece, p))
-        {
-            if (marker[mv])
-                continue;
-            if (! is_forbidden(c, mv))
-            {
-                moves.push_back(mv);
-                marker.set(mv);
-            }
-        }
 }
 
 void Board::gen_moves(Color c, Point p, unsigned adj_status,
-                      MoveMarker& marker,
-                      ArrayList<Move,Move::range>& moves) const
+                      MoveMarker& marker, MoveList& moves) const
 {
     for (Piece piece : m_state_color[c].pieces_left)
-        for (Move mv : m_board_const->get_moves(piece, p, adj_status))
+        for (Move mv : m_bc->get_moves(piece, p, adj_status))
         {
             if (marker[mv])
                 continue;
@@ -183,7 +159,7 @@ Move Board::get_move_at(Point p) const
             if (color_mv.color == c)
             {
                 Move mv = color_mv.move;
-                if (! mv.is_pass() && get_move_info(mv).contains(p))
+                if (get_move_info(mv).contains(p))
                     return mv;
             }
     }
@@ -192,20 +168,28 @@ Move Board::get_move_at(Point p) const
 
 bool Board::has_moves(Color c) const
 {
-    bool is_first = is_first_piece(c);
-    for (Iterator i(*this); i; ++i)
-        if (! m_state_color[c].forbidden[*i]
-            && (is_attach_point(*i, c)
-                || (is_first && get_starting_points(c).contains(*i))))
-            if (has_moves(c, *i))
+    if (is_first_piece(c))
+    {
+        for (auto p : get_starting_points(c))
+            if (has_moves(c, p))
                 return true;
+    }
+    else
+    {
+        for (auto p : get_attach_points(c))
+            if (has_moves(c, p))
+                return true;
+    }
     return false;
 }
 
 bool Board::has_moves(Color c, Point p) const
 {
-    for (Piece piece : m_state_color[c].pieces_left)
-        for (Move mv : m_board_const->get_moves(piece, p))
+    if (is_forbidden(p, c))
+        return false;
+    auto adj_status = get_adj_status(p, c);
+    for (auto piece : m_state_color[c].pieces_left)
+        for (auto mv : m_bc->get_moves(piece, p, adj_status))
             if (! is_forbidden(c, mv))
                 return true;
     return false;
@@ -213,8 +197,8 @@ bool Board::has_moves(Color c, Point p) const
 
 bool Board::has_setup() const
 {
-    for (ColorIterator i(m_nu_colors); i; ++i)
-        if (! m_setup.placements[*i].empty())
+    for (Color c : get_colors())
+        if (! m_setup.placements[c].empty())
             return true;
     return false;
 }
@@ -231,20 +215,20 @@ void Board::init(Variant variant, const Setup* setup)
         m_nu_piece_instances = 2;
     else
         m_nu_piece_instances = 1;
-    for (ColorIterator i(m_nu_colors); i; ++i)
+    for (Color c : get_colors())
     {
-        m_state_color[*i].forbidden.fill(false, *m_geo);
-        m_state_color[*i].is_attach_point.fill(false, *m_geo);
-        m_attach_points[*i].clear();
-        m_state_color[*i].pieces_left.clear();
-        m_state_color[*i].nu_onboard_pieces = 0;
-        m_state_color[*i].points = 0;
+        m_state_color[c].forbidden.fill(false, *m_geo);
+        m_state_color[c].is_attach_point.fill(false, *m_geo);
+        m_attach_points[c].clear();
+        m_state_color[c].pieces_left.clear();
+        m_state_color[c].nu_onboard_pieces = 0;
+        m_state_color[c].points = 0;
         for (Piece::IntType j = 0; j < get_nu_uniq_pieces(); ++j)
-            m_state_color[*i].pieces_left.push_back(Piece(j));
-        m_state_color[*i].nu_left_piece.fill(m_nu_piece_instances);
+            m_state_color[c].pieces_left.push_back(Piece(j));
+        m_state_color[c].nu_left_piece.fill(m_nu_piece_instances);
     }
     m_state_base.nu_onboard_pieces_all = 0;
-    if (setup == nullptr)
+    if (! setup)
     {
         m_setup.clear();
         m_state_base.to_play = Color(0);
@@ -255,9 +239,9 @@ void Board::init(Variant variant, const Setup* setup)
         place_setup(m_setup);
         m_state_base.to_play = setup->to_play;
         optimize_attach_point_lists();
-        for (ColorIterator i(m_nu_colors); i; ++i)
-            if (m_state_color[*i].pieces_left.empty())
-                m_state_color[*i].points += m_bonus_all_pieces;
+        for (Color c : get_colors())
+            if (m_state_color[c].pieces_left.empty())
+                m_state_color[c].points += m_bonus_all_pieces;
     }
     m_moves.clear();
 }
@@ -291,7 +275,7 @@ void Board::init_variant(Variant variant)
         m_color_esc_sequence_text[Color(3)] = "\x1B[1;32m";
     }
     m_nu_players = libpentobi_base::get_nu_players(variant);
-    m_board_const = &BoardConst::get(variant);
+    m_bc = &BoardConst::get(variant);
     if (m_variant == Variant::junior)
     {
         m_bonus_all_pieces = 0;
@@ -302,25 +286,25 @@ void Board::init_variant(Variant variant)
         m_bonus_all_pieces = 15;
         m_bonus_one_piece = 5;
     }
-    m_geo = &m_board_const->get_geometry();
-    m_move_info_array = m_board_const->get_move_info_array();
-    m_move_info_ext_array = m_board_const->get_move_info_ext_array();
-    m_move_info_ext_2_array = m_board_const->get_move_info_ext_2_array();
+    m_geo = &m_bc->get_geometry();
+    m_move_info_array = m_bc->get_move_info_array();
+    m_move_info_ext_array = m_bc->get_move_info_ext_array();
+    m_move_info_ext_2_array = m_bc->get_move_info_ext_2_array();
     m_starting_points.init(variant, *m_geo);
-    for (ColorIterator i(m_nu_colors); i; ++i)
+    for (Color c : get_colors())
     {
         if (variant == Variant::classic_2
             || variant == Variant::trigon_2)
-            m_second_color[*i] = get_next(get_next(*i));
+            m_second_color[c] = get_next(get_next(c));
         else
-            m_second_color[*i] = *i;
+            m_second_color[c] = c;
     }
 }
 
 bool Board::is_game_over() const
 {
-    for (ColorIterator i(m_nu_colors); i; ++i)
-        if (has_moves(*i))
+    for (Color c : get_colors())
+        if (has_moves(c))
             return false;
     return true;
 }
@@ -328,21 +312,18 @@ bool Board::is_game_over() const
 /** Remove forbidden points from attach point lists.
     The attach point lists do not guarantee that they contain only
     non-forbidden attach points because that would be too expensive to
-    incrementally update but at certain points that are not performance
+    update incrementally but at certain times that are not performance
     critical (e.g. before taking a snapshot), we can remove them. */
 void Board::optimize_attach_point_lists()
 {
-    for (ColorIterator i(m_nu_colors); i; ++i)
+    PointList l;
+    for (Color c : get_colors())
     {
-        auto& attach_points = m_attach_points[*i];
-        for (auto j = attach_points.begin(); j != attach_points.end(); ++j)
-            if (is_forbidden(*j, *i))
-            {
-                m_state_color[*i].is_attach_point[*j] = false;
-                attach_points.remove_fast(j);
-                --j;
-                continue;
-            }
+        l.clear();
+        for (Point p : m_attach_points[c])
+            if (! is_forbidden(p, c))
+                l.push_back(p);
+        m_attach_points[c] = l;
     }
 }
 
@@ -352,34 +333,32 @@ void Board::optimize_attach_point_lists()
     with -Winline with GCC 4.8.2. */
 void Board::place_setup(const Setup& setup)
 {
-    for (ColorIterator i(m_nu_colors); i; ++i)
-        for (Move mv : setup.placements[*i])
-            place(*i, mv);
+    for (Color c : get_colors())
+        for (Move mv : setup.placements[c])
+            place(c, mv);
 }
 
 void Board::take_snapshot()
 {
-    if (! m_snapshot)
-        m_snapshot.reset(new Snapshot());
     optimize_attach_point_lists();
-    m_snapshot->moves_size = m_moves.size();
-    m_snapshot->state_base.to_play = m_state_base.to_play;
-    m_snapshot->state_base.nu_onboard_pieces_all =
+    m_snapshot.moves_size = m_moves.size();
+    m_snapshot.state_base.to_play = m_state_base.to_play;
+    m_snapshot.state_base.nu_onboard_pieces_all =
         m_state_base.nu_onboard_pieces_all;
-    m_snapshot->state_base.point_state.memcpy_from(m_state_base.point_state,
-                                                   *m_geo);
-    for (ColorIterator i(m_nu_colors); i; ++i)
+    m_snapshot.state_base.point_state.copy_from(m_state_base.point_state,
+                                                *m_geo);
+    for (Color c : get_colors())
     {
-        m_snapshot->attach_points_size[*i] = m_attach_points[*i].size();
-        const auto& state = m_state_color[*i];
-        auto& snapshot_state = m_snapshot->state_color[*i];
-        snapshot_state.forbidden.memcpy_from(state.forbidden, *m_geo);
-        snapshot_state.is_attach_point.memcpy_from(state.is_attach_point,
-                                                   *m_geo);
-        // Uncomment once is_trivially_copyable is implemented in GCC and MSVC
-        //static_assert(is_trivially_copyable<StateColor>::value, "")
-        memcpy(&snapshot_state.pieces_left, &state.pieces_left,
-               sizeof(StateColor) - offsetof(StateColor, pieces_left));
+        m_snapshot.attach_points_size[c] = m_attach_points[c].size();
+        const auto& state = m_state_color[c];
+        auto& snapshot_state = m_snapshot.state_color[c];
+        snapshot_state.forbidden.copy_from(state.forbidden, *m_geo);
+        snapshot_state.is_attach_point.copy_from(state.is_attach_point,
+                                                 *m_geo);
+        snapshot_state.pieces_left = state.pieces_left;
+        snapshot_state.nu_left_piece = state.nu_left_piece;
+        snapshot_state.nu_onboard_pieces = state.nu_onboard_pieces;
+        snapshot_state.points = state.points;
     }
 }
 
@@ -389,18 +368,18 @@ void Board::undo()
     ArrayList<ColorMove, max_game_moves> moves = m_moves;
     moves.pop_back();
     init();
-    for (unsigned i = 0; i < moves.size(); ++i)
-        play(moves[i]);
+    for (auto& mv : moves)
+        play(mv);
 }
 
 void Board::write(ostream& out, bool mark_last_move) const
 {
     // Sort lists of left pieces by name
     ColorMap<PiecesLeftList> pieces_left;
-    for (ColorIterator i(m_nu_colors); i; ++i)
+    for (Color c : get_colors())
     {
-        pieces_left[*i] = m_state_color[*i].pieces_left;
-        sort(pieces_left[*i].begin(), pieces_left[*i].end(),
+        pieces_left[c] = m_state_color[c].pieces_left;
+        sort(pieces_left[c].begin(), pieces_left[c].end(),
              [&](Piece p1, Piece p2)
              {
                  return
@@ -413,13 +392,8 @@ void Board::write(ostream& out, bool mark_last_move) const
     if (mark_last_move)
     {
         unsigned n = get_nu_moves();
-        while (n > 0)
-        {
-            --n;
-            last_mv = get_move(n);
-            if (! last_mv.move.is_pass())
-                break;
-        }
+        if (n > 0)
+            last_mv = get_move(n - 1);
     }
     unsigned width = m_geo->get_width();
     unsigned height = m_geo->get_height();
@@ -434,10 +408,10 @@ void Board::write(ostream& out, bool mark_last_move) const
         out << (height - y) << ' ';
         for (unsigned x = 0; x < width; ++x)
         {
-            Point p(x, y, width);
-            bool is_offboard = ! is_onboard(p);
-            if ((x > 0 || (is_trigon && x == 0 && is_onboard(p.get_right())))
-                && ! is_offboard)
+            Point p = m_geo->get_point(x, y);
+            bool is_offboard = p.is_null();
+            if ((x > 0 || (is_trigon && x == 0 && m_geo->is_onboard(x + 1, y)))
+                    && ! is_offboard)
             {
                 // Print a space horizontally between fields on the board. On a
                 // Trigon board, a slash or backslash is used instead of the
@@ -446,19 +420,20 @@ void Board::write(ostream& out, bool mark_last_move) const
                 // space to mark the last piece played (the mark is not placed
                 // within the piece or off-board).
                 if (! last_mv.is_null()
-                    && get_move_info(last_mv.move).contains(p)
-                    && (x == 0 || ! is_onboard(p.get_left())
-                        || get_point_state(p.get_left()) != last_mv.color))
+                        && get_move_info(last_mv.move).contains(p)
+                        && (x == 0 || ! m_geo->is_onboard(x - 1, y)
+                            || get_point_state(m_geo->get_point(x - 1, y))
+                               != last_mv.color))
                 {
                     set_color(out, "\x1B[1;37;47m");
                     out << '>';
                     last_mv = ColorMove::null();
                 }
                 else if (! last_mv.is_null()
-                         && x > 0 && is_onboard(p.get_left())
-                         && get_move_info(last_mv.move).contains(p.get_left())
+                         && x > 0 && m_geo->is_onboard(x - 1, y)
+                         && get_move_info(last_mv.move).contains(m_geo->get_point(x - 1, y))
                          && get_point_state(p) != last_mv.color
-                         && get_point_state(p.get_left()) == last_mv.color)
+                         && get_point_state(m_geo->get_point(x - 1, y)) == last_mv.color)
                 {
                     set_color(out, "\x1B[1;37;47m");
                     out << '<';
@@ -477,10 +452,10 @@ void Board::write(ostream& out, bool mark_last_move) const
             }
             if (is_offboard)
             {
-                if (is_trigon && x > 0 && is_onboard(p.get_left()))
+                if (is_trigon && x > 0 && m_geo->is_onboard(x - 1, y))
                 {
                     set_color(out, "\x1B[1;30;47m");
-                    out << (m_geo->get_point_type(p) == 1 ? '\\' : '/');
+                    out << (m_geo->get_point_type(x, y) == 1 ? '\\' : '/');
                 }
                 else
                 {
@@ -520,7 +495,7 @@ void Board::write(ostream& out, bool mark_last_move) const
         }
         if (is_trigon)
         {
-            if (is_onboard(Point(width - 1, y, width)))
+            if (m_geo->is_onboard(width - 1, y))
             {
                 set_color(out, "\x1B[1;30;47m");
                 out << (m_geo->get_point_type(width - 1, y) != 1 ? '\\' : '/');
@@ -538,13 +513,13 @@ void Board::write(ostream& out, bool mark_last_move) const
     }
     write_x_coord(out, width, is_trigon ? 3 : 2);
     if (! is_info_location_right)
-        for (ColorIterator i(m_nu_colors); i; ++i)
+        for (Color c : get_colors())
         {
-            write_color_info_line1(out, *i);
+            write_color_info_line1(out, c);
             out << "  ";
-            write_color_info_line2(out, *i, pieces_left[*i]);
+            write_color_info_line2(out, c, pieces_left[c]);
             out << ' ';
-            write_color_info_line3(out, *i, pieces_left[*i]);
+            write_color_info_line3(out, c, pieces_left[c]);
             out << '\n';
         }
 }
