@@ -7,25 +7,21 @@
 #ifndef LIBPENTOBI_BASE_BOARD_H
 #define LIBPENTOBI_BASE_BOARD_H
 
-#include <cstddef>
-#include <memory>
 #include "BoardConst.h"
 #include "ColorMap.h"
 #include "ColorMove.h"
 #include "Variant.h"
 #include "Geometry.h"
 #include "Grid.h"
+#include "MoveList.h"
 #include "MoveMarker.h"
 #include "PointList.h"
 #include "PointState.h"
 #include "Setup.h"
 #include "StartingPoints.h"
-#include "libboardgame_util/Unused.h"
 
 namespace libpentobi_base {
 
-using libboardgame_util::NullTermList;
-
 //-----------------------------------------------------------------------------
 
 /** Blokus board.
@@ -35,41 +31,16 @@ class Board
 public:
     typedef Grid<PointState> PointStateGrid;
 
-    typedef PrecompMoves::LocalMovesListRange LocalMovesListRange;
-
-    /** Iterator over all points on the board. */
-    class Iterator
-        : public GeometryIterator
-    {
-        friend class Board;
-
-    public:
-        Iterator(const Board& bd);
-    };
-
     /** Maximum number of pieces per player in any game variant. */
     static const unsigned max_pieces = Setup::max_pieces;
 
-    typedef ArrayList<Piece,BoardConst::max_pieces> PiecesLeftList;
+    typedef ArrayList<Piece, BoardConst::max_pieces> PiecesLeftList;
 
-    /** Maximum number of moves for a player in a game.
-        Non-alternating moves and alternating pass moves are allowed but the
-        game must end after all colors have passed in a row. Therefore, the
-        maximum number of moves is reached in case that a piece move is
-        followed by (Color::range-1) pass moves and an extra Color::range pass
-        moves at the end. */
-    static const unsigned max_player_moves = Color::range * max_pieces + 1;
+    static const unsigned max_player_moves = max_pieces;
 
-    static const unsigned max_nonpass_player_moves = max_pieces;
-
-    /** Maximum number of moves in any game variant.
-        Includes an extra pass move per color at the end of the game. */
+    /** Maximum number of moves in any game variant. */
     static const unsigned max_game_moves = Color::range * max_player_moves;
 
-    /** Maximum number of real (=non-pass) moves in any game variant. */
-    static const unsigned max_nonpass_game_moves =
-        Color::range * max_nonpass_player_moves;
-
     /** Use ANSI escape sequences for colored text output in operator>> */
     static bool color_output;
 
@@ -83,10 +54,25 @@ public:
         Use copy_from() to copy a board state. */
     Board& operator=(const Board&) = delete;
 
+    Geometry::Iterator begin() const
+    {
+        return m_geo->begin();
+    }
+
+    Geometry::Iterator end() const
+    {
+        return m_geo->end();
+    }
+
     Variant get_variant() const;
 
     Color::IntType get_nu_colors() const;
 
+    Color::Range get_colors() const
+    {
+        return Color::Range(m_nu_colors);
+    }
+
     /** Number of colors that are not played alternately.
         This is equal to get_nu_colors() apart from Variant::classic_3. */
     Color::IntType get_nu_nonalt_colors() const;
@@ -102,10 +88,10 @@ public:
     /** Number of instances of each unique piece per color. */
     unsigned get_nu_piece_instances() const;
 
-    unsigned get_max_nonpass_player_moves() const;
+    unsigned get_max_player_moves() const;
 
-    /** Maximum number of non-pass moves in the current game variant. */
-    unsigned get_max_nonpass_game_moves() const;
+    /** Maximum number of moves in the current game variant. */
+    unsigned get_max_game_moves() const;
 
     Color get_next(Color c) const;
 
@@ -116,16 +102,11 @@ public:
     /** Get the state of an on-board point. */
     PointState get_point_state(Point p) const;
 
-    bool is_empty(Point p) const;
-
-    bool is_onboard(Point p) const;
-
-    const PointStateGrid& get_grid() const;
+    const PointStateGrid& get_point_state() const;
 
     /** Get next color to play.
         The next color to play is the next color of the color of the last move
-        played even if it has no more moves to play (apart from a pass
-        move). */
+        played even if it has no more moves to play. */
     Color get_to_play() const;
 
     /** Get the player who plays the next move for the 4th color in
@@ -178,34 +159,20 @@ public:
     void copy_from(const Board& bd);
 
     /** Play a move.
+        @pre mv.is_regular()
         @pre get_nu_moves() < max_game_moves */
-    void play(Color c, Move move);
+    void play(Color c, Move mv);
 
-    /** Play a pass move.
+    /** Play a move for the current color to play.
+        @pre mv.is_regular()
         @pre get_nu_moves() < max_game_moves */
-    void play_pass(Color c);
-
-    /** Play a pass move for the current color to play.
-        @pre get_nu_moves() < max_game_moves */
-    void play_pass();
-
-    /** Play a move that is known to be not a pass move.
-        Slightly faster than play(Color,Move)
-        @pre get_nu_moves() < max_game_moves */
-    void play_nonpass(Color c, Move move);
-
-    /** Play a move that is known to be not a pass move for the current color
-        to play.
-        Slightly faster than play(Move)
-        @pre get_nu_moves() < max_game_moves */
-    void play_nonpass(Move move);
-
-    /** See play(Color,Move) */
-    void play(ColorMove move);
-
-    /** Calls play(Color,Move) with the current color to play. */
     void play(Move mv);
 
+    /** Play a move.
+        @pre mv.move.is_regular()
+        @pre get_nu_moves() < max_game_moves */
+    void play(ColorMove mv);
+
     void undo();
 
     void set_to_play(Color c);
@@ -234,29 +201,27 @@ public:
 
     ColorMove get_move(unsigned n) const;
 
-    /** Generate all moves for one player.
-        The generated moves do not include the pass move. */
-    void gen_moves(Color c, ArrayList<Move, Move::range>& moves) const;
+    const ArrayList<ColorMove, max_game_moves>& get_moves() const;
 
-    void gen_moves(Color c, Point p,
-                   ArrayList<Move, Move::range>& moves) const;
-
-    void gen_moves(Color c, Point p, MoveMarker& marker,
-                   ArrayList<Move, Move::range>& moves) const;
+    /** Generate all legal moves for a color.
+        @param c The color
+        @param marker A move marker reused for efficiency (needs to be clear)
+        @param[out] moves The list of moves. */
+    void gen_moves(Color c, MoveMarker& marker, MoveList& moves) const;
 
     bool has_moves(Color c) const;
 
     /** Check that no color has any moves left. */
     bool is_game_over() const;
 
+    /** Check if a move is legal.
+        @pre mv.is_regular() */
     bool is_legal(Color c, Move mv) const;
 
+    /** Check if a move is legal for the current color to play.
+        @pre mv.is_regular() */
     bool is_legal(Move mv) const;
 
-    bool is_legal_nonpass(Color c, Move mv) const;
-
-    bool is_legal_nonpass(Move mv) const;
-
     /** Check that point is not already occupied or adjacent to own color.
         @param c
         @param p The point. Off-board points are allowed and return true. */
@@ -276,7 +241,7 @@ public:
 
     unsigned get_adj_status(Point p, Color c) const;
 
-    LocalMovesListRange get_moves(Piece piece, Point p,
+    PrecompMoves::Range get_moves(Piece piece, Point p,
                                   unsigned adj_status) const;
 
     /** Get score.
@@ -297,7 +262,7 @@ public:
     const Geometry& get_geometry() const;
 
     /** See BoardConst::to_string() */
-    string to_string(Move mv, bool with_piece_name = true) const;
+    string to_string(Move mv, bool with_piece_name = false) const;
 
     /** See BoardConst::from_string() */
     Move from_string(const string& s) const;
@@ -342,13 +307,8 @@ public:
     /** See take_snapshot() */
     void restore_snapshot();
 
-    WritePoint write(Point p) const;
-
 private:
-    /** Color-independent part of the board state for fast snapshot
-        restoration.
-        Must have only POD-like members such that it can quickly be copied
-        with memcpy in copy_from(). */
+    /** Color-independent part of the board state. */
     struct StateBase
     {
         Color to_play;
@@ -358,18 +318,13 @@ private:
         PointStateGrid point_state;
     };
 
-    /** Color-dependent part of the board state for fast snapshot restoration.
-        Must have only POD-like members such that it can quickly be copied
-        with memcpy in copy_from(). */
+    /** Color-dependent part of the board state. */
     struct StateColor
     {
         Grid<bool> forbidden;
 
         Grid<bool> is_attach_point;
 
-        /** @note Order dependency: This must be the first of all remaining
-            members because they are copied with memcpy in take_snapshot()
-            and restore_snapshot() */
         PiecesLeftList pieces_left;
 
         PieceMap<uint_fast8_t> nu_left_piece;
@@ -379,9 +334,7 @@ private:
         unsigned points;
     };
 
-    /** Snapshot for fast restoration of a previous position.
-        @note state_base and state_color are restored with a single memcpy,
-        they must be subsequent members and their order must not be changed. */
+    /** Snapshot for fast restoration of a previous position. */
     struct Snapshot
     {
         StateBase state_base;
@@ -403,8 +356,6 @@ private:
 
     unsigned m_nu_players;
 
-    const BoardConst* m_board_const;
-
     /** Bonus for playing all pieces. */
     unsigned m_bonus_all_pieces;
 
@@ -414,13 +365,15 @@ private:
     /** See get_nu_piece_instances() */
     uint_fast8_t m_nu_piece_instances;
 
-    /** Same as m_board_const->get_move_info_array() */
+    const BoardConst* m_bc;
+
+    /** Same as m_bc->get_move_info_array() */
     const MoveInfo* m_move_info_array;
 
-    /** Same as m_board_const->get_move_info_ext_array() */
+    /** Same as m_bc->get_move_info_ext_array() */
     const MoveInfoExt* m_move_info_ext_array;
 
-    /** Same as m_board_const->get_move_info_ext_2_array() */
+    /** Same as m_bc->get_move_info_ext_2_array() */
     const MoveInfoExt2* m_move_info_ext_2_array;
 
     const Geometry* m_geo;
@@ -440,18 +393,14 @@ private:
 
     ArrayList<ColorMove, max_game_moves> m_moves;
 
-    unique_ptr<Snapshot> m_snapshot;
+    Snapshot m_snapshot;
 
     Setup m_setup;
 
     StartingPoints m_starting_points;
 
-    /** Local variable during move generation.
-        Reused for efficiency. */
-    mutable MoveMarker m_marker;
-
     void gen_moves(Color c, Point p, unsigned adj_status, MoveMarker& marker,
-                   ArrayList<Move,Move::range>& moves) const;
+                   MoveList& moves) const;
 
     bool has_moves(Color c, Point p) const;
 
@@ -479,34 +428,33 @@ private:
                          const ColorMap<PiecesLeftList>& pieces_left) const;
 };
 
-typedef Board::Iterator BoardIterator;
-
-inline Board::Iterator::Iterator(const Board& bd)
-    : GeometryIterator(*bd.m_geo)
-{
-}
 
 inline bool Board::find_move(const MovePoints& points, Move& move) const
 {
-    return m_board_const->find_move(points, move);
+    return m_bc->find_move(points, move);
 }
 
 inline Move Board::from_string(const string& s) const
 {
-    return m_board_const->from_string(s);
+    return m_bc->from_string(s);
 }
 
 inline unsigned Board::get_adj_status(Point p, Color c) const
 {
-    unsigned result = 0;
+    auto& adj_status_list = m_bc->get_adj_status_list(p);
+    auto i = adj_status_list.begin();
+    auto end = adj_status_list.end();
+    LIBBOARDGAME_ASSERT(i != end);
     unsigned val = 1;
-    NullTermList<Point, 12>::Iterator i(m_geo->get_adj_diag(p));
-    LIBBOARDGAME_ASSERT(i);
-    static_assert(PrecompMoves::adj_status_nu_adj > 0, "");
-    do
+    unsigned result = is_forbidden(*i, c) ? val : 0;
+    for (unsigned j = 1; j < PrecompMoves::adj_status_nu_adj; ++j)
+    {
+        if (++i == end)
+            break;
+        val <<= 1;
         if (is_forbidden(*i, c))
             result |= val;
-    while ((val <<= 1) < (1 << PrecompMoves::adj_status_nu_adj) && ++i);
+    }
     return result;
 }
 
@@ -523,12 +471,12 @@ inline const PointList&  Board::get_attach_points(Color c) const
 
 inline const BoardConst& Board::get_board_const() const
 {
-    return *m_board_const;
+    return *m_bc;
 }
 
 inline BoardType Board::get_board_type() const
 {
-    return m_board_const->get_board_type();
+    return m_bc->get_board_type();
 }
 
 inline const Geometry& Board::get_geometry() const
@@ -536,17 +484,12 @@ inline const Geometry& Board::get_geometry() const
     return *m_geo;
 }
 
-inline const Board::PointStateGrid& Board::get_grid() const
+inline unsigned Board::get_max_game_moves() const
 {
-    return m_state_base.point_state;
+    return m_nu_colors * get_max_player_moves();
 }
 
-inline unsigned Board::get_max_nonpass_game_moves() const
-{
-    return m_nu_colors * get_max_nonpass_player_moves();
-}
-
-inline unsigned Board::get_max_nonpass_player_moves() const
+inline unsigned Board::get_max_player_moves() const
 {
     return get_nu_pieces();
 }
@@ -559,28 +502,34 @@ inline ColorMove Board::get_move(unsigned n) const
 inline const MoveInfo& Board::get_move_info(Move mv) const
 {
     LIBBOARDGAME_ASSERT(! mv.is_null());
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_board_const->get_nu_all_moves());
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
     return *(m_move_info_array + mv.to_int());
 }
 
 inline const MoveInfoExt& Board::get_move_info_ext(Move mv) const
 {
     LIBBOARDGAME_ASSERT(! mv.is_null());
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_board_const->get_nu_all_moves());
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
     return *(m_move_info_ext_array + mv.to_int());
 }
 
 inline const MoveInfoExt2& Board::get_move_info_ext_2(Move mv) const
 {
     LIBBOARDGAME_ASSERT(! mv.is_null());
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_board_const->get_nu_all_moves());
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
     return *(m_move_info_ext_2_array + mv.to_int());
 }
 
-inline Board::LocalMovesListRange Board::get_moves(Piece piece, Point p,
-                                                   unsigned adj_status) const
+inline auto Board::get_moves() const
+-> const ArrayList<ColorMove, max_game_moves>&
 {
-    return m_board_const->get_moves(piece, p, adj_status);
+    return m_moves;
+}
+
+inline PrecompMoves::Range Board::get_moves(Piece piece, Point p,
+                                            unsigned adj_status) const
+{
+    return m_bc->get_moves(piece, p, adj_status);
 }
 
 inline Color Board::get_next(Color c) const
@@ -631,22 +580,22 @@ inline unsigned Board::get_nu_piece_instances() const
 
 inline unsigned Board::get_nu_pieces() const
 {
-    return m_nu_piece_instances * m_board_const->get_nu_pieces();
+    return m_nu_piece_instances * m_bc->get_nu_pieces();
 }
 
 inline Piece::IntType Board::get_nu_uniq_pieces() const
 {
-    return m_board_const->get_nu_pieces();
+    return m_bc->get_nu_pieces();
 }
 
 inline const PieceInfo& Board::get_piece_info(Piece piece) const
 {
-    return m_board_const->get_piece_info(piece);
+    return m_bc->get_piece_info(piece);
 }
 
 inline bool Board::get_piece_by_name(const string& name, Piece& piece) const
 {
-    return m_board_const->get_piece_by_name(name, piece);
+    return m_bc->get_piece_by_name(name, piece);
 }
 
 inline const Board::PiecesLeftList& Board::get_pieces_left(Color c) const
@@ -656,10 +605,14 @@ inline const Board::PiecesLeftList& Board::get_pieces_left(Color c) const
 
 inline PointState Board::get_point_state(Point p) const
 {
-    LIBBOARDGAME_ASSERT(is_onboard(p));
     return PointState(m_state_base.point_state[p].to_int());
 }
 
+inline const Board::PointStateGrid& Board::get_point_state() const
+{
+    return m_state_base.point_state;
+}
+
 inline unsigned Board::get_points(Color c) const
 {
     return m_state_color[c].points;
@@ -696,9 +649,9 @@ inline int Board::get_score(Color c) const
     {
         int score = 0;
         auto nu_players = static_cast<Color::IntType>(m_nu_players);
-        for (ColorIterator i(nu_players); i; ++i)
-            if (*i != c)
-                score -= get_points(*i);
+        for (Color i : get_colors())
+            if (i != c)
+                score -= get_points(i);
         score = get_points(c) + score / (static_cast<int>(nu_players) - 1);
         return score;
     }
@@ -732,7 +685,7 @@ inline Color Board::get_to_play() const
 
 inline const PieceTransforms& Board::get_transforms() const
 {
-    return m_board_const->get_transforms();
+    return m_bc->get_transforms();
 }
 
 inline Variant Board::get_variant() const
@@ -760,11 +713,6 @@ inline bool Board::is_colorless_starting_point(Point p) const
     return m_starting_points.is_colorless_starting_point(p);
 }
 
-inline bool Board::is_empty(Point p) const
-{
-    return get_point_state(p).is_empty();
-}
-
 inline bool Board::is_first_piece(Color c) const
 {
     return m_state_color[c].nu_onboard_pieces == 0;
@@ -799,17 +747,6 @@ inline bool Board::is_legal(Move mv) const
 
 inline bool Board::is_legal(Color c, Move mv) const
 {
-    return (mv.is_pass() || is_legal_nonpass(c, mv));
-}
-
-inline bool Board::is_legal_nonpass(Move mv) const
-{
-    return is_legal_nonpass(m_state_base.to_play, mv);
-}
-
-inline bool Board::is_legal_nonpass(Color c, Move mv) const
-{
-    LIBBOARDGAME_ASSERT(mv.is_regular());
     auto& info = get_move_info(mv);
     if (! is_piece_left(c, info.get_piece()))
         return false;
@@ -838,11 +775,6 @@ inline bool Board::is_legal_nonpass(Color c, Move mv) const
     return false;
 }
 
-inline bool Board::is_onboard(Point p) const
-{
-    return m_geo->is_onboard(p);
-}
-
 inline bool Board::is_piece_left(Color c, Piece piece) const
 {
     LIBBOARDGAME_ASSERT(piece.to_int() < get_nu_uniq_pieces());
@@ -851,12 +783,11 @@ inline bool Board::is_piece_left(Color c, Piece piece) const
 
 inline bool Board::is_same_player(Color c1, Color c2) const
 {
-    return (c1 == c2 || c1 == m_second_color[c2]);
+    return c1 == c2 || c1 == m_second_color[c2];
 }
 
 inline void Board::place(Color c, Move mv)
 {
-    LIBBOARDGAME_ASSERT(mv.is_regular());
     auto& info = get_move_info(mv);
     auto& info_ext = get_move_info_ext(mv);
     auto piece = info.get_piece();
@@ -881,18 +812,24 @@ inline void Board::place(Color c, Move mv)
     do
     {
         m_state_base.point_state[*i] = c;
-        LIBPENTOBI_FOREACH_COLOR(c, m_state_color[c].forbidden[*i] = true);
+        for_each_color([&](Color c) {
+            m_state_color[c].forbidden[*i] = true;
+        });
     }
     while (++i != end);
     i = info_ext.begin_attach();
     end = info_ext.end_attach();
+    auto& attach_points = m_attach_points[c];
+    auto n = attach_points.size();
     do
-        if (! state_color.is_attach_point[*i] && ! state_color.forbidden[*i])
+        if (! state_color.forbidden[*i] && ! state_color.is_attach_point[*i])
         {
             state_color.is_attach_point[*i] = true;
-            m_attach_points[c].push_back(*i);
+            attach_points.get_unchecked(n) = *i;
+            ++n;
         }
     while (++i != end);
+    attach_points.resize(n);
     LIBBOARDGAME_ASSERT(i == info_ext.begin_adj());
     end += info_ext.size_adj_points;
     do
@@ -900,71 +837,43 @@ inline void Board::place(Color c, Move mv)
     while (++i != end);
 }
 
-inline void Board::play(ColorMove move)
-{
-    play(move.color, move.move);
-}
-
 inline void Board::play(Color c, Move mv)
-{
-    if (! mv.is_pass())
-        play_nonpass(c, mv);
-    else
-        play_pass(c);
-}
-
-inline void Board::play_nonpass(Color c, Move mv)
 {
     place(c, mv);
     m_moves.push_back(ColorMove(c, mv));
     m_state_base.to_play = get_next(c);
 }
 
-inline void Board::play_nonpass(Move mv)
+inline void Board::play(Move mv)
 {
-    play_nonpass(m_state_base.to_play, mv);
+    play(m_state_base.to_play, mv);
 }
 
-inline void Board::play_pass(Color c)
+inline void Board::play(ColorMove mv)
 {
-    m_moves.push_back(ColorMove(c, Move::pass()));
-    m_state_base.to_play = get_next(c);
-}
-
-inline void Board::play_pass()
-{
-    play_pass(m_state_base.to_play);
-}
-
-inline void Board::play(Move move)
-{
-    play(m_state_base.to_play, move);
+    play(mv.color, mv.move);
 }
 
 inline void Board::restore_snapshot()
 {
-    LIBBOARDGAME_ASSERT(m_snapshot);
-    LIBBOARDGAME_ASSERT(m_snapshot->moves_size <= m_moves.size());
-    m_moves.resize(m_snapshot->moves_size);
-    m_state_base.to_play = m_snapshot->state_base.to_play;
+    LIBBOARDGAME_ASSERT(m_snapshot.moves_size <= m_moves.size());
+    auto& geo = get_geometry();
+    m_moves.resize(m_snapshot.moves_size);
+    m_state_base.to_play = m_snapshot.state_base.to_play;
     m_state_base.nu_onboard_pieces_all =
-        m_snapshot->state_base.nu_onboard_pieces_all;
-    m_state_base.point_state.memcpy_from(m_snapshot->state_base.point_state,
-                                         *m_geo);
-    for (ColorIterator i(m_nu_colors); i; ++i)
+        m_snapshot.state_base.nu_onboard_pieces_all;
+    m_state_base.point_state.copy_from(m_snapshot.state_base.point_state, geo);
+    for (Color c : get_colors())
     {
-        const auto& snapshot_state = m_snapshot->state_color[*i];
-        auto& state = m_state_color[*i];
-        state.forbidden.memcpy_from(snapshot_state.forbidden, *m_geo);
-        state.is_attach_point.memcpy_from(snapshot_state.is_attach_point,
-                                          *m_geo);
-        // Uncomment once is_trivially_copyable is implemented in GCC and MSVC
-        //static_assert(is_trivially_copyable<StateColor>::value, "")
-        memcpy(&state.pieces_left, &snapshot_state.pieces_left,
-               sizeof(StateColor) - offsetof(StateColor, pieces_left));
-        LIBBOARDGAME_ASSERT(m_snapshot->attach_points_size[*i]
-                            <= m_attach_points[*i].size());
-        m_attach_points[*i].resize(m_snapshot->attach_points_size[*i]);
+        const auto& snapshot_state = m_snapshot.state_color[c];
+        auto& state = m_state_color[c];
+        state.forbidden.copy_from(snapshot_state.forbidden, geo);
+        state.is_attach_point.copy_from(snapshot_state.is_attach_point, geo);
+        state.pieces_left = snapshot_state.pieces_left;
+        state.nu_left_piece = snapshot_state.nu_left_piece;
+        state.nu_onboard_pieces = snapshot_state.nu_onboard_pieces;
+        state.points = snapshot_state.points;
+        m_attach_points[c].resize(m_snapshot.attach_points_size[c]);
     }
 }
 
@@ -975,12 +884,7 @@ inline void Board::set_to_play(Color c)
 
 inline string Board::to_string(Move mv, bool with_piece_name) const
 {
-    return m_board_const->to_string(mv, with_piece_name);
-}
-
-inline WritePoint Board::write(Point p) const
-{
-    return WritePoint(p, m_geo->get_width(), m_geo->get_height());
+    return m_bc->to_string(mv, with_piece_name);
 }
 
 //-----------------------------------------------------------------------------
