@@ -12,26 +12,21 @@
 
 #include <algorithm>
 #include <functional>
-#include "AdjIterator.h"
-#include "AdjDiagIterator.h"
-#include "DiagIterator.h"
 #include "Grid.h"
 #include "PieceTransformsClassic.h"
 #include "PieceTransformsTrigon.h"
 #include "SymmetricPoints.h"
+#include "TrigonGeometry.h"
 #include "libboardgame_base/RectGeometry.h"
 #include "libboardgame_base/Transform.h"
-#include "libboardgame_base/TrigonGeometry.h"
 #include "libboardgame_util/Log.h"
 #include "libboardgame_util/StringUtil.h"
 
 namespace libpentobi_base {
 
-using namespace std;
 using libboardgame_base::PointTransfRot180;
 using libboardgame_base::RectGeometry;
 using libboardgame_base::Transform;
-using libboardgame_base::TrigonGeometry;
 using libboardgame_util::get_log;
 using libboardgame_util::log;
 using libboardgame_util::split;
@@ -44,26 +39,67 @@ namespace {
 
 const bool log_move_creation = false;
 
-/** Compare two points using the ordering used in blksgf files.
-    As specified in doc/blksgf/Pentobi-SGF.html, the order should be
-    (a1, b1, ..., a2, b2, ...) with y going upwards whereas the convention
-    for Point is that y goes downwards. */
-bool compare_points(unsigned width, Point p1, Point p2)
+// Sort points using the ordering used in blksgf files (switches the direction
+// of the y axis!)
+inline void sort_piece_points(PiecePoints& points)
 {
-    auto y1 = p1.get_y(width);
-    auto y2 = p2.get_y(width);
-    if (y1 != y2)
-        return y1 > y2;
-    return p1.get_x(width) < p2.get_x(width);
-}
-
-/** Compare two coordinate points using the ordering used in blksgf files.
-    See compare_points() */
-bool compare_coord_points(const CoordPoint& p1, const CoordPoint& p2)
-{
-    if (p1.y != p2.y)
-        return p1.y > p2.y;
-    return p1.x < p2.x;
+    auto check = [&](unsigned short a, unsigned short b)
+    {
+        if ((points[a].y == points[b].y && points[a].x > points[b].x)
+                || points[a].y < points[b].y)
+            swap(points[a], points[b]);
+    };
+    // Minimal number of necessary comparisons with sorting networks
+    auto size = points.size();
+    if (size == 6)
+    {
+        check(1, 2);
+        check(4, 5);
+        check(0, 2);
+        check(3, 5);
+        check(0, 1);
+        check(3, 4);
+        check(2, 5);
+        check(0, 3);
+        check(1, 4);
+        check(2, 4);
+        check(1, 3);
+        check(2, 3);
+    }
+    else if (size == 5)
+    {
+        check(0, 1);
+        check(3, 4);
+        check(2, 4);
+        check(2, 3);
+        check(1, 4);
+        check(0, 3);
+        check(0, 2);
+        check(1, 3);
+        check(1, 2);
+    }
+    else if (size == 4)
+    {
+        check(0, 1);
+        check(2, 3);
+        check(0, 2);
+        check(1, 3);
+        check(1, 2);
+    }
+    else if (size == 3)
+    {
+        check(1, 2);
+        check(0, 2);
+        check(0, 1);
+    }
+    else if (size == 2)
+    {
+        check(0, 1);
+    }
+    else
+    {
+        LIBBOARDGAME_ASSERT(size == 1);
+    }
 }
 
 vector<PieceInfo> create_pieces_classic(const Geometry& geo,
@@ -345,74 +381,65 @@ vector<PieceInfo> create_pieces_trigon(const Geometry& geo,
     return pieces;
 }
 
-const Geometry& create_geometry(BoardType board_type)
-{
-    if (board_type == BoardType::classic)
-        return RectGeometry<Point>::get(20, 20);
-    else if (board_type == BoardType::duo)
-        return RectGeometry<Point>::get(14, 14);
-    else if (board_type == BoardType::trigon)
-        return TrigonGeometry<Point>::get(9);
-    else
-    {
-        LIBBOARDGAME_ASSERT(board_type == BoardType::trigon_3);
-        return TrigonGeometry<Point>::get(8);
-    }
-}
-
 } // namespace
 
 //-----------------------------------------------------------------------------
 
-BoardConst::BoardConst(BoardType board_type, Variant variant)
-    : m_geo(create_geometry(board_type)),
-      m_nu_attach_points(0)
+Marker BoardConst::s_marker;
+
+Grid<array<ArrayList<Point, PrecompMoves::adj_status_nu_adj>,
+                 PrecompMoves::nu_adj_status>>
+    BoardConst::s_adj_status;
+
+BoardConst::BoardConst(BoardType board_type, PieceSet piece_set)
+    : m_geo(libpentobi_base::get_geometry(board_type))
 {
     m_board_type = board_type;
-    if (board_type == BoardType::trigon)
+    if (board_type == BoardType::trigon && piece_set == PieceSet::trigon)
     {
-        m_transforms.reset(new PieceTransformsTrigon());
+        m_transforms.reset(new PieceTransformsTrigon);
         m_pieces = create_pieces_trigon(m_geo, *m_transforms);
-        reserve_info(Move::onboard_moves_trigon);
+        m_nu_moves = Move::onboard_moves_trigon + 1;
     }
-    else if (board_type == BoardType::trigon_3)
+    else if (board_type == BoardType::trigon_3
+             && piece_set == PieceSet::trigon)
     {
-        m_transforms.reset(new PieceTransformsTrigon());
+        m_transforms.reset(new PieceTransformsTrigon);
         m_pieces = create_pieces_trigon(m_geo, *m_transforms);
-        reserve_info(Move::onboard_moves_trigon_3);
+        m_nu_moves = Move::onboard_moves_trigon_3 + 1;
     }
-    else if (board_type == BoardType::classic)
+    else if (board_type == BoardType::classic
+             && piece_set == PieceSet::classic)
     {
-        m_transforms.reset(new PieceTransformsClassic());
+        m_transforms.reset(new PieceTransformsClassic);
         m_pieces = create_pieces_classic(m_geo, *m_transforms);
-        reserve_info(Move::onboard_moves_classic);
+        m_nu_moves = Move::onboard_moves_classic + 1;
     }
-    else if (variant == Variant::junior)
+    else if (board_type == BoardType::duo && piece_set == PieceSet::junior)
     {
-        m_transforms.reset(new PieceTransformsClassic());
+        m_transforms.reset(new PieceTransformsClassic);
         m_pieces = create_pieces_junior(m_geo, *m_transforms);
-        reserve_info(Move::onboard_moves_junior);
+        m_nu_moves = Move::onboard_moves_junior + 1;
     }
     else
     {
-        LIBBOARDGAME_ASSERT(variant == Variant::duo);
-        m_transforms.reset(new PieceTransformsClassic());
+        LIBBOARDGAME_ASSERT(board_type == BoardType::duo);
+        LIBBOARDGAME_ASSERT(piece_set == PieceSet::classic);
+        m_transforms.reset(new PieceTransformsClassic);
         m_pieces = create_pieces_classic(m_geo, *m_transforms);
-        reserve_info(Move::onboard_moves_duo);
+        m_nu_moves = Move::onboard_moves_duo + 1;
     }
+    m_move_info.reset(new MoveInfo[m_nu_moves]);
+    m_move_info_ext.reset(new MoveInfoExt[m_nu_moves]);
+    m_move_info_ext_2.reset(new MoveInfoExt2[m_nu_moves]);
     m_nu_pieces = static_cast<Piece::IntType>(m_pieces.size());
     init_adj_status();
+    auto width = m_geo.get_width();
+    auto height = m_geo.get_height();
+    for (Point p : m_geo)
+        m_compare_val[p] =
+                (height - m_geo.get_y(p) - 1) * width + m_geo.get_x(p);
     create_moves();
-    if (board_type == BoardType::classic)
-        LIBBOARDGAME_ASSERT(m_move_info.size() == Move::onboard_moves_classic);
-    else if (board_type == BoardType::trigon)
-        LIBBOARDGAME_ASSERT(m_move_info.size() == Move::onboard_moves_trigon);
-    else if (board_type == BoardType::trigon_3)
-        LIBBOARDGAME_ASSERT(m_move_info.size() == Move::onboard_moves_trigon_3);
-    else if (variant == Variant::duo)
-        LIBBOARDGAME_ASSERT(m_move_info.size() == Move::onboard_moves_duo);
-    else if (variant == Variant::junior)
-        LIBBOARDGAME_ASSERT(m_move_info.size() == Move::onboard_moves_junior);
     m_total_piece_points = 0;
     m_max_piece_size = 0;
     for (const PieceInfo& piece : m_pieces)
@@ -420,48 +447,44 @@ BoardConst::BoardConst(BoardType board_type, Variant variant)
         m_max_piece_size = max(m_max_piece_size, piece.get_size());
         m_total_piece_points += piece.get_size();
     }
-    if (board_type == BoardType::classic || variant == Variant::duo)
+    if (piece_set == PieceSet::classic)
     {
         LIBBOARDGAME_ASSERT(m_nu_pieces == 21);
         LIBBOARDGAME_ASSERT(m_total_piece_points == 89);
     }
-    else if (variant == Variant::junior)
+    else if (piece_set == PieceSet::junior)
     {
         LIBBOARDGAME_ASSERT(m_nu_pieces == 12);
         LIBBOARDGAME_ASSERT(m_total_piece_points == 44);
     }
-    else if (board_type == BoardType::trigon
-             || board_type == BoardType::trigon_3)
+    else if (piece_set == PieceSet::trigon)
     {
         LIBBOARDGAME_ASSERT(m_nu_pieces == 22);
         LIBBOARDGAME_ASSERT(m_total_piece_points == 110);
     }
-    if (variant == Variant::duo || variant == Variant::junior
-        || variant == Variant::trigon_2)
+    if (board_type == BoardType::duo || board_type == BoardType::trigon)
         init_symmetry_info();
 }
 
-void BoardConst::create_move(Piece piece, const PiecePoints& coord_points,
-                             Point label_pos)
+void BoardConst::create_move(unsigned& moves_created, Piece piece,
+                             const PiecePoints& coord_points, Point label_pos)
 {
     MovePoints points;
-    auto width = m_geo.get_width();
-    for (auto i = coord_points.begin(); i != coord_points.end(); ++i)
-        points.push_back(Point((*i).x, (*i).y, width));
-    MoveInfo info(piece, points);
-    MoveInfoExt info_ext;
+    for (auto& i : coord_points)
+        points.push_back(m_geo.get_point(i.x, i.y));
+    auto& info = m_move_info[moves_created];
+    auto& info_ext = m_move_info_ext[moves_created];
+    auto& info_ext_2 = m_move_info_ext_2[moves_created];
+    info = MoveInfo(piece, points);
     set_adj_and_attach_points(info, info_ext);
-    MoveInfoExt2 info_ext_2;
     info_ext_2.label_pos = label_pos;
     info_ext_2.breaks_symmetry = false;
     info_ext_2.symmetric_move = Move::null();
-    m_move_info.push_back(info);
-    m_move_info_ext.push_back(info_ext);
-    m_move_info_ext_2.push_back(info_ext_2);
     m_nu_attach_points[piece] =
         max(m_nu_attach_points[piece],
             static_cast<unsigned>(info_ext.size_attach_points));
-    Move mv(static_cast<Move::IntType>(m_move_info.size() - 1));
+    Move mv(static_cast<Move::IntType>(moves_created));
+    ++moves_created;
     if (log_move_creation)
     {
         Grid<char> grid;
@@ -482,56 +505,73 @@ void BoardConst::create_move(Piece piece, const PiecePoints& coord_points,
 
 void BoardConst::create_moves()
 {
+    unsigned moves_created = 1; // Unused move infos for Move::null()
     m_full_move_table.reset(new FullMoveTable);
     for (Piece::IntType i = 0; i < m_nu_pieces; ++i)
-        create_moves(Piece(i));
-    for (GeometryIterator i(m_geo); i; ++i)
-        for (unsigned j = 0; j < PrecompMoves::nu_adj_status; ++j)
-            for (Piece::IntType k = 0; k < m_nu_pieces; ++k)
+        create_moves(moves_created, Piece(i));
+    for (Point p : m_geo)
+        for (unsigned i = 0; i < PrecompMoves::nu_adj_status; ++i)
+            for (Piece::IntType j = 0; j < m_nu_pieces; ++j)
             {
-                Piece piece(k);
-                auto& list = (*m_full_move_table)[j][piece][*i];
+                Piece piece(j);
+                auto& list = (*m_full_move_table)[i][piece][p];
                 auto begin = m_precomp_moves.get_size();
-                for (unsigned l = 0; l < list.size(); ++l)
-                    m_precomp_moves.push_move(list[l]);
+                for (auto mv : list)
+                    m_precomp_moves.push_move(mv);
                 auto size = m_precomp_moves.get_size() - begin;
-                m_precomp_moves.set_list_range(*i, j, piece, begin, size);
+                m_precomp_moves.set_list_range(p, i, piece, begin, size);
             }
     if (log_move_creation)
-        log("Created moves: ", m_move_info.size(), ", precomputed: ",
+        log("Created moves: ", moves_created, ", precomputed: ",
             m_precomp_moves.get_size());
+    LIBBOARDGAME_ASSERT(moves_created == m_nu_moves);
     m_full_move_table.reset(nullptr); // Free space, no longer needed
 }
 
-void BoardConst::create_moves(Piece piece)
+void BoardConst::create_moves(unsigned& moves_created, Piece piece)
 {
     auto& piece_info = m_pieces[piece.to_int()];
     if (log_move_creation)
         log("Creating moves for piece ", piece_info.get_name());
+    auto& transforms = piece_info.get_transforms();
+    auto nu_transforms = transforms.size();
+    vector<PiecePoints> transformed_points(nu_transforms);
+    vector<CoordPoint> transformed_label_pos(nu_transforms);
+    for (size_t i = 0; i < nu_transforms; ++i)
+    {
+        auto transform = transforms[i];
+        transformed_points[i] = piece_info.get_points();
+        transform->transform(transformed_points[i].begin(),
+                             transformed_points[i].end());
+        sort_piece_points(transformed_points[i]);
+        transformed_label_pos[i] =
+                transform->get_transformed(piece_info.get_label_pos());
+    }
     PiecePoints points;
-    auto width = m_geo.get_width();
-    auto height = m_geo.get_height();
-    for (GeometryIterator i(m_geo); i; ++i)
+    // Make outer loop iterator over geometry for better memory locality
+    for (Point p : m_geo)
     {
         if (log_move_creation)
-            log("Creating moves at ", WritePoint(*i, width, height));
-        auto x = (*i).get_x(width);
-        auto y = (*i).get_y(width);
-        for (const Transform* transform : piece_info.get_transforms())
+            log("Creating moves at ", m_geo.to_string(p));
+        auto x = m_geo.get_x(p);
+        auto y = m_geo.get_y(p);
+        for (size_t i = 0; i < nu_transforms; ++i)
         {
             if (log_move_creation)
-                log("Transformation ", typeid(*transform).name());
-            auto point_type = m_geo.get_point_type(x, y);
-            if (transform->get_new_point_type() != point_type)
-                continue;
-            points = piece_info.get_points();
-            transform->transform(points.begin(), points.end());
-            bool is_onboard = true;
-            for (CoordPoint& p : points)
             {
-                p.x += x;
-                p.y += y;
-                if (! m_geo.is_onboard(p))
+                auto& transform = *transforms[i];
+                log("Transformation ", typeid(transform).name());
+            }
+            auto point_type = m_geo.get_point_type(x, y);
+            if (transforms[i]->get_new_point_type() != point_type)
+                continue;
+            points = transformed_points[i];
+            bool is_onboard = true;
+            for (auto& pp : points)
+            {
+                pp.x += x;
+                pp.y += y;
+                if (! m_geo.is_onboard(pp))
                 {
                     is_onboard = false;
                     break;
@@ -539,12 +579,11 @@ void BoardConst::create_moves(Piece piece)
             }
             if (! is_onboard)
                 continue;
-            sort(points.begin(), points.end(), compare_coord_points);
-            CoordPoint label_pos = piece_info.get_label_pos();
-            label_pos = transform->get_transformed(label_pos);
+            CoordPoint label_pos = transformed_label_pos[i];
             label_pos.x += x;
             label_pos.y += y;
-            create_move(piece, points, Point(label_pos.x, label_pos.y, width));
+            create_move(moves_created, piece, points,
+                        m_geo.get_point(label_pos.x, label_pos.y));
         }
     }
 }
@@ -552,76 +591,34 @@ void BoardConst::create_moves(Piece piece)
 Move BoardConst::from_string(const string& s) const
 {
     string trimmed = to_lower(trim(s));
-    if (trimmed == "pass")
-        return Move::pass();
-    else if (trimmed == "null")
+    if (trimmed == "null")
         return Move::null();
     vector<string> v = split(trimmed, ',');
     if (v.size() > PieceInfo::max_size)
-        throw Exception("illegal move (too many points)");
+        throw runtime_error("illegal move (too many points)");
     MovePoints points;
-    auto width = m_geo.get_width();
-    auto height = m_geo.get_height();
-    for (const string& p : v)
-        points.push_back(Point::from_string(p, width, height));
+    for (const auto& s : v)
+    {
+        Point p;
+        if (! m_geo.from_string(s, p))
+            throw runtime_error("illegal move (invalid point)");
+        points.push_back(p);
+    }
     Move mv;
     if (! find_move(points, mv))
-        throw Exception("illegal move");
+        throw runtime_error("illegal move");
     return mv;
 }
 
 const BoardConst& BoardConst::get(Variant variant)
 {
-    static unique_ptr<BoardConst> board_const_classic;
-    static unique_ptr<BoardConst> board_const_duo;
-    static unique_ptr<BoardConst> board_const_junior;
-    static unique_ptr<BoardConst> board_const_trigon;
-    static unique_ptr<BoardConst> board_const_trigon_2;
-    static unique_ptr<BoardConst> board_const_trigon_3;
-    if (variant == Variant::classic || variant == Variant::classic_2
-             || variant == Variant::classic_3)
-    {
-        if (! board_const_classic)
-            board_const_classic.reset(new BoardConst(BoardType::classic,
-                                                     Variant::classic));
-        return *board_const_classic;
-    }
-    else if (variant == Variant::duo)
-    {
-        if (! board_const_duo)
-            board_const_duo.reset(new BoardConst(BoardType::duo,
-                                                 Variant::duo));
-        return *board_const_duo;
-    }
-    else if (variant == Variant::junior)
-    {
-        if (! board_const_junior)
-            board_const_junior.reset(new BoardConst(BoardType::duo,
-                                                    Variant::junior));
-        return *board_const_junior;
-    }
-    else if (variant == Variant::trigon)
-    {
-        if (! board_const_trigon)
-            board_const_trigon.reset(new BoardConst(BoardType::trigon,
-                                                    Variant::trigon));
-        return *board_const_trigon;
-    }
-    else if (variant == Variant::trigon_2)
-    {
-        if (! board_const_trigon_2)
-            board_const_trigon_2.reset(new BoardConst(BoardType::trigon,
-                                                      Variant::trigon_2));
-        return *board_const_trigon_2;
-    }
-    else
-    {
-        LIBBOARDGAME_ASSERT(variant == Variant::trigon_3);
-        if (! board_const_trigon_3)
-            board_const_trigon_3.reset(new BoardConst(BoardType::trigon_3,
-                                                      Variant::trigon_3));
-        return *board_const_trigon_3;
-    }
+    static map<BoardType, map<PieceSet, unique_ptr<BoardConst>>> board_const;
+    auto board_type = libpentobi_base::get_board_type(variant);
+    auto piece_set = get_piece_set(variant);
+    auto& bc = board_const[board_type][piece_set];
+    if (! bc)
+        bc.reset(new BoardConst(board_type, piece_set));
+    return *bc;
 }
 
 bool BoardConst::get_piece_by_name(const string& name, Piece& piece) const
@@ -640,24 +637,20 @@ bool BoardConst::find_move(const MovePoints& points, Move& move) const
     if (points.size() == 0)
         return false;
     MovePoints sorted_points = points;
-    sort(sorted_points.begin(), sorted_points.end(),
-         bind(compare_points, m_geo.get_width(), placeholders::_1,
-              placeholders::_2));
+    sort(sorted_points);
     Point p = points[0];
-    if (! m_geo.is_onboard(p))
-        return false;
     for (Piece::IntType i = 0; i < m_pieces.size(); ++i)
     {
         Piece piece(i);
         if (get_piece_info(piece).get_size() == points.size())
         {
-            Board::LocalMovesListRange moves = get_moves(piece, p);
-            for (auto j = moves.begin(); j != moves.end(); ++j)
+            auto moves = get_moves(piece, p);
+            for (auto mv : moves)
                 if (equal(sorted_points.begin(),
                           sorted_points.end(),
-                          m_move_info[j->to_int()].begin()))
+                          m_move_info[mv.to_int()].begin()))
                 {
-                    move = *j;
+                    move = mv;
                     return true;
                 }
         }
@@ -667,9 +660,25 @@ bool BoardConst::find_move(const MovePoints& points, Move& move) const
 
 void BoardConst::init_adj_status()
 {
+    for (Point p : m_geo)
+    {
+        auto& adj_status_list = m_adj_status_list[p];
+        for (Point pp : m_geo.get_adj(p))
+        {
+            if (adj_status_list.size() == PrecompMoves::adj_status_nu_adj)
+                break;
+            adj_status_list.push_back(pp);
+        }
+        for (Point pp : m_geo.get_diag(p))
+        {
+            if (adj_status_list.size() == PrecompMoves::adj_status_nu_adj)
+                break;
+            adj_status_list.push_back(pp);
+        }
+    }
     array<bool, PrecompMoves::adj_status_nu_adj> forbidden;
-    for (GeometryIterator i(m_geo); i; ++i)
-        init_adj_status(*i, forbidden, 0);
+    for (Point p : m_geo)
+        init_adj_status(p, forbidden, 0);
 }
 
 void BoardConst::init_adj_status(
@@ -677,17 +686,23 @@ void BoardConst::init_adj_status(
                        array<bool, PrecompMoves::adj_status_nu_adj>& forbidden,
                        unsigned i)
 {
-    if (i == PrecompMoves::adj_status_nu_adj
-        || i == m_geo.get_adj_diag(p).size())
+    auto& adj_status_list = m_adj_status_list[p];
+    if (i == adj_status_list.size())
     {
         unsigned index = 0;
         for (unsigned j = 0; j < i; ++j)
             if (forbidden[j])
                 index |= (1 << j);
+        s_adj_status[p][index].clear();
         unsigned n = 0;
-        for (AdjDiagIterator j(m_geo, p); n < i; ++j, ++n)
+        for (Point j : adj_status_list)
+        {
+            if (n >= i)
+                return;
             if (forbidden[n])
-                m_adj_status[p][index].push_back(*j);
+                s_adj_status[p][index].push_back(j);
+            ++n;
+        }
         return;
     }
     forbidden[i] = false;
@@ -701,7 +716,7 @@ void BoardConst::init_symmetry_info()
     SymmetricPoints symmetric_points;
     PointTransfRot180<Point> transform;
     symmetric_points.init(m_geo, transform);
-    for (unsigned i = 0; i < m_move_info.size(); ++i)
+    for (unsigned i = 1; i < m_nu_moves; ++i)
     {
         const auto& info = m_move_info[i];
         auto& info_ext_2 = m_move_info_ext_2[i];
@@ -720,66 +735,115 @@ void BoardConst::init_symmetry_info()
 bool BoardConst::is_compatible_with_adj_status(Point p, unsigned adj_status,
                                                const MoveInfo& info) const
 {
-    for (Point p_adj : m_adj_status[p][adj_status])
+    for (Point p_adj : s_adj_status[p][adj_status])
         if (info.contains(p_adj))
             return false;
     return true;
 }
 
-void BoardConst::reserve_info(size_t nu_moves)
-{
-    m_move_info.reserve(nu_moves);
-    m_move_info_ext.reserve(nu_moves);
-    m_move_info_ext_2.reserve(nu_moves);
-}
-
 void BoardConst::set_adj_and_attach_points(const MoveInfo& info,
                                            MoveInfoExt& info_ext)
 {
     auto begin = info.begin();
     auto end = info.end();
-    m_marker.clear();
+    s_marker.clear();
     for (auto i = begin; i != end; ++i)
-        m_marker.set(*i);
+        s_marker.set(*i);
     ArrayList<Point, PieceInfo::max_adj> adj_points;
     for (auto i = begin; i != end; ++i)
-        for (AdjIterator j(m_geo, *i); j; ++j)
-            if (m_geo.is_onboard(*j) && ! m_marker[*j])
+        m_geo.for_each_adj(*i, [&](Point j) {
+            if (! s_marker[j])
             {
-                m_marker.set(*j);
-                adj_points.push_back(*j);
+                s_marker.set(j);
+                adj_points.push_back(j);
             }
+        });
     ArrayList<Point, PieceInfo::max_attach> attach_points;
     for (auto i = begin; i != end; ++i)
-        for (DiagIterator j(m_geo, *i); j; ++j)
-            if (m_geo.is_onboard(*j) && ! m_marker[*j])
+        m_geo.for_each_diag(*i, [&](Point j) {
+            if (! s_marker[j])
             {
-                m_marker.set(*j);
-                attach_points.push_back(*j);
+                s_marker.set(j);
+                attach_points.push_back(j);
             }
+        });
     info_ext.init(adj_points, attach_points);
 }
 
+inline void BoardConst::sort(MovePoints& points) const
+{
+    auto check = [&](unsigned short a, unsigned short b)
+    {
+        if (m_compare_val[points[a]] > m_compare_val[points[b]])
+            swap(points[a], points[b]);
+    };
+    // Minimal number of necessary comparisons with sorting networks
+    auto size = points.size();
+    if (size == 6)
+    {
+        check(1, 2);
+        check(4, 5);
+        check(0, 2);
+        check(3, 5);
+        check(0, 1);
+        check(3, 4);
+        check(2, 5);
+        check(0, 3);
+        check(1, 4);
+        check(2, 4);
+        check(1, 3);
+        check(2, 3);
+    }
+    else if (size == 5)
+    {
+        check(0, 1);
+        check(3, 4);
+        check(2, 4);
+        check(2, 3);
+        check(1, 4);
+        check(0, 3);
+        check(0, 2);
+        check(1, 3);
+        check(1, 2);
+    }
+    else if (size == 4)
+    {
+        check(0, 1);
+        check(2, 3);
+        check(0, 2);
+        check(1, 3);
+        check(1, 2);
+    }
+    else if (size == 3)
+    {
+        check(1, 2);
+        check(0, 2);
+        check(0, 1);
+    }
+    else if (size == 2)
+    {
+        check(0, 1);
+    }
+    else
+        LIBBOARDGAME_ASSERT(size == 1);
+}
+
 string BoardConst::to_string(Move mv, bool with_piece_name) const
 {
     if (mv.is_null())
         return "null";
-    if (mv.is_pass())
-        return "pass";
     auto& info = get_move_info(mv);
     ostringstream s;
     if (with_piece_name)
         s << '[' << get_piece_info(info.get_piece()).get_name() << "]";
     bool is_first = true;
-    auto width = m_geo.get_width();
-    auto height = m_geo.get_height();
     for (Point p : info)
     {
         if (! is_first)
             s << ',';
         else
             is_first = false;
-        p.write(s, width, height);
+        s << m_geo.to_string(p);
     }
     return s.str();
 }
