@@ -9,13 +9,13 @@
 
 #include <array>
 #include <algorithm>
-#include "Variant.h"
 #include "Geometry.h"
 #include "Marker.h"
 #include "MoveInfo.h"
 #include "PieceInfo.h"
 #include "PieceTransforms.h"
 #include "PrecompMoves.h"
+#include "Variant.h"
 #include "libpentobi_base/Color.h"
 #include "libpentobi_base/ColorMap.h"
 #include "libboardgame_util/ArrayList.h"
@@ -27,31 +27,24 @@ using libboardgame_util::ArrayList;
 
 //-----------------------------------------------------------------------------
 
-enum class BoardType
-{
-    classic,
-
-    duo,
-
-    trigon,
-
-    trigon_3
-};
-
-//-----------------------------------------------------------------------------
-
 /** Constant precomputed data that is shared between all instances of Board
     with a given board type and set of unique pieces per color. */
 class BoardConst
 {
 public:
+    /** See get_adj_status_list() */
+    typedef
+    ArrayList<Point, PrecompMoves::adj_status_nu_adj, unsigned short>
+    AdjStatusList;
+
     /** Maximum number of unique pieces per color. */
     static const unsigned max_pieces = 22;
 
     static const unsigned max_moves_at_point = 40;
 
     /** Get the single instance for a given board size.
-        The instance is created the first time this function is called. */
+        The instance is created the first time this function is called.
+        This function is not thread-safe. */
     static const BoardConst& get(Variant variant);
 
     Piece::IntType get_nu_pieces() const;
@@ -70,7 +63,7 @@ public:
 
     /** Get move info.
         @pre move.is_regular() */
-    const MoveInfo& get_move_info(Move move) const;
+    const MoveInfo& get_move_info(Move mv) const;
 
     /** Get pointer to move info array.
         Can be used to speed up the access to the move info by avoiding the
@@ -84,20 +77,20 @@ public:
 
     /** Get extended move info.
         @pre move.is_regular() */
-    const MoveInfoExt& get_move_info_ext(Move move) const;
+    const MoveInfoExt& get_move_info_ext(Move mv) const;
 
     const MoveInfoExt2& get_move_info_ext_2(Move mv) const;
 
     const MoveInfoExt2* get_move_info_ext_2_array() const;
 
-    unsigned get_nu_all_moves() const;
+    unsigned get_nu_moves() const;
 
     bool find_move(const MovePoints& points, Move& move) const;
 
     /** Get all moves of a piece at a point constrained by the forbidden
         status of adjacent points. */
-    PrecompMoves::LocalMovesListRange get_moves(Piece piece, Point p,
-                                                unsigned adj_status = 0) const
+    PrecompMoves::Range get_moves(Piece piece, Point p,
+                                  unsigned adj_status = 0) const
     {
         return m_precomp_moves.get_moves(piece, p, adj_status);
     }
@@ -108,6 +101,14 @@ public:
 
     const Geometry& get_geometry() const;
 
+    /** List containing the points used for the adjacent status.
+        Contains the first PrecompMoves::adj_status_nu_adj points of
+        Geometry::get_adj() concatenated with Geometry::get_diag(). */
+    const AdjStatusList& get_adj_status_list(Point p) const
+    {
+        return m_adj_status_list[p];
+    }
+
     /** Convert a move to its string representation.
         The string representation is a comma-separated list of points (without
         spaces between the commas or points). If with_piece_name is true,
@@ -118,6 +119,9 @@ public:
 
     Move from_string(const string& s) const;
 
+    /** Sort move points using the ordering used in blksgf files. */
+    void sort(MovePoints& points) const;
+
 private:
     typedef ArrayList<Move, max_moves_at_point> LocalMovesList;
 
@@ -125,27 +129,42 @@ private:
     typedef array<PieceMap<Grid<LocalMovesList>>, PrecompMoves::nu_adj_status>
         FullMoveTable;
 
+    /** Local variable used during construction.
+        Making this variable static slightly speeds up construction and
+        a thread-safe construction is not needed. */
+    static Marker s_marker;
+
+    /** Local variable used during construction.
+        See s_marker why this variable is static. */
+    static Grid<array<ArrayList<Point, PrecompMoves::adj_status_nu_adj>,
+                     PrecompMoves::nu_adj_status>>
+        s_adj_status;
+
     Piece::IntType m_nu_pieces;
 
     unsigned m_total_piece_points;
 
     unsigned m_max_piece_size;
 
+    unsigned m_nu_moves;
+
     BoardType m_board_type;
 
     const Geometry& m_geo;
 
     vector<PieceInfo> m_pieces;
 
+    Grid<AdjStatusList> m_adj_status_list;
+
     unique_ptr<PieceTransforms> m_transforms;
 
-    PieceMap<unsigned> m_nu_attach_points;
+    PieceMap<unsigned> m_nu_attach_points{0};
 
-    vector<MoveInfo> m_move_info;
+    unique_ptr<MoveInfo[]> m_move_info;
 
-    vector<MoveInfoExt> m_move_info_ext;
+    unique_ptr<MoveInfoExt[]> m_move_info_ext;
 
-    vector<MoveInfoExt2> m_move_info_ext_2;
+    unique_ptr<MoveInfoExt2[]> m_move_info_ext_2;
 
     /** Non-compact representation of lists of moves of a piece at a point
         constrained by the forbidden status of adjacent points.
@@ -154,23 +173,20 @@ private:
 
     PrecompMoves m_precomp_moves;
 
-    /** Local variable reused for efficiency. */
-    Marker m_marker;
+    /** Value for comparing points using the ordering used in blksgf files.
+        As specified in doc/blksgf/Pentobi-SGF.html, the order should be
+        (a1, b1, ..., a2, b2, ...) with y going upwards whereas the convention
+        for Point is that y goes downwards. */
+    Grid<unsigned> m_compare_val;
 
-    /** Forbidden neighbors for a given adjacent status index at a given point.
-        Only used during construction. */
-    Grid<array<ArrayList<Point, PrecompMoves::adj_status_nu_adj>,
-               PrecompMoves::nu_adj_status>>
-        m_adj_status;
+    BoardConst(BoardType board_type, PieceSet piece_set);
 
-    BoardConst(BoardType board_type, Variant variant);
-
-    void create_move(Piece piece, const PiecePoints& coord_points,
-                     Point label_pos);
+    void create_move(unsigned& moves_created, Piece piece,
+                     const PiecePoints& coord_points, Point label_pos);
 
     void create_moves();
 
-    void create_moves(Piece piece);
+    void create_moves(unsigned& moves_created, Piece piece);
 
     void init_adj_status();
 
@@ -184,8 +200,6 @@ private:
     bool is_compatible_with_adj_status(Point p, unsigned adj_status,
                                        const MoveInfo& info) const;
 
-    void reserve_info(size_t nu_moves);
-
     void set_adj_and_attach_points(const MoveInfo& info,
                                    MoveInfoExt& info_ext);
 };
@@ -205,42 +219,42 @@ inline unsigned BoardConst::get_max_piece_size() const
     return m_max_piece_size;
 }
 
-inline const MoveInfo& BoardConst::get_move_info(Move move) const
+inline const MoveInfo& BoardConst::get_move_info(Move mv) const
 {
-    LIBBOARDGAME_ASSERT(move.to_int() < m_move_info.size());
-    return m_move_info[move.to_int()];
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_nu_moves);
+    return m_move_info[mv.to_int()];
 }
 
 inline const MoveInfo* BoardConst::get_move_info_array() const
 {
-    return &m_move_info.front();
+    return m_move_info.get();
 }
 
-inline const MoveInfoExt& BoardConst::get_move_info_ext(Move move) const
+inline const MoveInfoExt& BoardConst::get_move_info_ext(Move mv) const
 {
-    LIBBOARDGAME_ASSERT(move.to_int() < m_move_info_ext.size());
-    return m_move_info_ext[move.to_int()];
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_nu_moves);
+    return m_move_info_ext[mv.to_int()];
 }
 
 inline const MoveInfoExt2& BoardConst::get_move_info_ext_2(Move mv) const
 {
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_move_info_ext_2.size());
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_nu_moves);
     return m_move_info_ext_2[mv.to_int()];
 }
 
 inline const MoveInfoExt* BoardConst::get_move_info_ext_array() const
 {
-    return &m_move_info_ext.front();
+    return m_move_info_ext.get();
 }
 
 inline const MoveInfoExt2* BoardConst::get_move_info_ext_2_array() const
 {
-    return &m_move_info_ext_2.front();
+    return m_move_info_ext_2.get();
 }
 
-inline unsigned BoardConst::get_nu_all_moves() const
+inline unsigned BoardConst::get_nu_moves() const
 {
-    return static_cast<unsigned>(m_move_info.size());
+    return m_nu_moves;
 }
 
 inline unsigned BoardConst::get_nu_attach_points(Piece piece) const
