@@ -38,10 +38,8 @@ string dump(const Board& bd)
         writer.begin_node();
         auto mv = bd.get_move(i);
         auto id = get_color_id(variant, mv.color);
-        if (! mv.is_pass())
+        if (! mv.is_null())
             writer.write_property(id, bd.to_string(mv.move, false));
-        else
-            writer.write_property(id, "");
         writer.end_node();
     }
     writer.end_tree();
@@ -55,22 +53,33 @@ void get_current_position_as_setup(const Board& bd, Setup& setup)
     for (unsigned i = 0; i < bd.get_nu_moves(); ++i)
     {
         auto mv = bd.get_move(i);
-        if (! mv.is_pass())
-            setup.placements[mv.color].push_back(mv.move);
+        setup.placements[mv.color].push_back(mv.move);
     }
     setup.to_play = bd.get_to_play();
 }
 
+Move get_transformed(const Board& bd, Move mv,
+                     const PointTransform<Point>& transform)
+{
+    auto& geo = bd.get_geometry();
+    MovePoints points;
+    for (auto p : bd.get_move_info(mv))
+        points.push_back(transform.get_transformed(p, geo));
+    Move transformed_mv;
+    bd.find_move(points, transformed_mv);
+    return transformed_mv;
+}
+
 void write_setup(Writer& writer, Variant variant, const Setup& setup)
 {
     auto& board_const = BoardConst::get(variant);
-    for (ColorIterator i(get_nu_colors(variant)); i; ++i)
-        if (! setup.placements[*i].empty())
+    for (Color c : get_colors(variant))
+        if (! setup.placements[c].empty())
         {
             vector<string> values;
-            for (Move mv : setup.placements[*i])
+            for (Move mv : setup.placements[c])
                 values.push_back(board_const.to_string(mv, false));
-            writer.write_property(get_setup_id(variant, *i), values);
+            writer.write_property(get_setup_id(variant, c), values);
         }
 }
 
