@@ -13,12 +13,12 @@
 #include "libboardgame_sgf/MissingProperty.h"
 #include "libboardgame_sgf/TreeReader.h"
 #include "libboardgame_util/Log.h"
+#include "libpentobi_base/BoardUtil.h"
 
 //-----------------------------------------------------------------------------
 
 namespace libpentobi_base {
 
-using namespace std;
 using libboardgame_base::PointTransfIdent;
 using libboardgame_base::PointTransfRefl;
 using libboardgame_base::PointTransfReflRot180;
@@ -32,16 +32,21 @@ using libboardgame_base::PointTransfTrigonRot60;
 using libboardgame_base::PointTransfTrigonRot120;
 using libboardgame_base::PointTransfTrigonRot240;
 using libboardgame_base::PointTransfTrigonRot300;
-using libboardgame_sgf::ChildIterator;
 using libboardgame_sgf::InvalidPropertyValue;
 using libboardgame_sgf::MissingProperty;
 using libboardgame_sgf::TreeReader;
 using libboardgame_util::log;
+using boardutil::get_transformed;
 
 //-----------------------------------------------------------------------------
 
 Book::Book(Variant variant)
     : m_tree(variant)
+{
+    get_transforms(variant, m_transforms, m_inv_transforms);
+}
+
+Book::~Book()
 {
 }
 
@@ -51,64 +56,9 @@ Move Book::genmove(const Board& bd, Color c)
         // Book cannot handle setup positions
         return Move::null();
     Move mv;
-    if (genmove(bd, c, mv,
-                PointTransfIdent<Point>(),
-                PointTransfIdent<Point>()))
-        return mv;
-    auto board_type = bd.get_board_type();
-    if (board_type == BoardType::duo)
-        if (genmove(bd, c, mv,
-                    PointTransfRot270Refl<Point>(),
-                    PointTransfRot270Refl<Point>()))
+    for (unsigned i = 0; i < m_transforms.size(); ++i)
+        if (genmove(bd, c, mv, *m_transforms[i], *m_inv_transforms[i]))
             return mv;
-    if (board_type == BoardType::trigon
-        || board_type == BoardType::trigon_3)
-    {
-        if (genmove(bd, c, mv,
-                    PointTransfTrigonRot60<Point>(),
-                    PointTransfTrigonRot300<Point>()))
-            return mv;
-        if (genmove(bd, c, mv,
-                    PointTransfTrigonRot120<Point>(),
-                    PointTransfTrigonRot240<Point>()))
-            return mv;
-        if (genmove(bd, c, mv,
-                    PointTransfRot180<Point>(),
-                    PointTransfRot180<Point>()))
-            return mv;
-        if (genmove(bd, c, mv,
-                    PointTransfTrigonRot240<Point>(),
-                    PointTransfTrigonRot120<Point>()))
-            return mv;
-        if (genmove(bd, c, mv,
-                    PointTransfTrigonRot300<Point>(),
-                    PointTransfTrigonRot60<Point>()))
-            return mv;
-        if (genmove(bd, c, mv,
-                    PointTransfRefl<Point>(),
-                    PointTransfRefl<Point>()))
-            return mv;
-        if (genmove(bd, c, mv,
-                    PointTransfTrigonReflRot60<Point>(),
-                    PointTransfTrigonReflRot60<Point>()))
-            return mv;
-        if (genmove(bd, c, mv,
-                    PointTransfTrigonReflRot120<Point>(),
-                    PointTransfTrigonReflRot120<Point>()))
-            return mv;
-        if (genmove(bd, c, mv,
-                    PointTransfReflRot180<Point>(),
-                    PointTransfReflRot180<Point>()))
-            return mv;
-        if (genmove(bd, c, mv,
-                    PointTransfTrigonReflRot240<Point>(),
-                    PointTransfTrigonReflRot240<Point>()))
-            return mv;
-        if (genmove(bd, c, mv,
-                    PointTransfTrigonReflRot300<Point>(),
-                    PointTransfTrigonReflRot300<Point>()))
-            return mv;
-    }
     return Move::null();
 }
 
@@ -123,31 +73,16 @@ bool Book::genmove(const Board& bd, Color c, Move& mv,
         ColorMove color_mv = bd.get_move(i);
         color_mv.move = get_transformed(bd, color_mv.move, transform);
         node = m_tree.find_child_with_move(*node, color_mv);
-        if (node == nullptr)
+        if (! node)
             return false;
     }
     node = select_child(bd, c, m_tree, *node, inv_transform);
-    if (node == nullptr)
+    if (! node)
         return false;
     mv = get_transformed(bd, m_tree.get_move(*node).move, inv_transform);
     return true;
 }
 
-Move Book::get_transformed(const Board& bd, Move mv,
-                           const PointTransform& transform) const
-{
-    if (mv.is_pass())
-        return mv;
-    unsigned width = bd.get_geometry().get_width();
-    unsigned height = bd.get_geometry().get_height();
-    MovePoints points;
-    for (Point p : bd.get_move_info(mv))
-        points.push_back(transform.get_transformed(p, width, height));
-    Move transformed_mv;
-    bd.find_move(points, transformed_mv);
-    return transformed_mv;
-}
-
 void Book::load(istream& in)
 {
     TreeReader reader;
@@ -157,10 +92,11 @@ void Book::load(istream& in)
     }
     catch (const TreeReader::ReadError& e)
     {
-        throw Exception(string("could not read book: ") + e.what());
+        throw runtime_error(string("could not read book: ") + e.what());
     }
     unique_ptr<SgfNode> root = reader.get_tree_transfer_ownership();
     m_tree.init(root);
+    get_transforms(m_tree.get_variant(), m_transforms, m_inv_transforms);
 }
 
 const SgfNode* Book::select_child(const Board& bd, Color c,
@@ -203,7 +139,7 @@ const SgfNode* Book::select_child(const Board& bd, Color c,
         return nullptr;
     log("Book moves: ", good_moves.size());
     unsigned nu_good_moves = static_cast<unsigned>(good_moves.size());
-    return good_moves[m_random.generate_small_int(nu_good_moves)];
+    return good_moves[m_random.generate() % nu_good_moves];
 }
 
 //-----------------------------------------------------------------------------
