@@ -9,14 +9,13 @@
 
 #include <cstdint>
 #include <iosfwd>
+#include <stdexcept>
 #include <string>
 #include "libboardgame_util/Assert.h"
-#include "libboardgame_util/Exception.h"
 
 namespace libpentobi_base {
 
 using namespace std;
-using libboardgame_util::Exception;
 
 //-----------------------------------------------------------------------------
 
@@ -26,31 +25,67 @@ public:
     typedef uint_fast8_t IntType;
 
     class InvalidString
-        : public Exception
+        : public runtime_error
     {
     public:
-        InvalidString(const string& s);
+        InvalidString(const string& s)
+            : runtime_error(s)
+        { }
     };
 
     class Iterator
     {
-        friend class Color;
-
     public:
-        Iterator(IntType nu_colors);
+        Iterator(IntType i)
+        {
+            m_i = i;
+        }
 
-        operator bool() const;
+        bool operator==(Iterator it) const
+        {
+            return m_i == it.m_i;
+        }
 
-        void operator++();
+        bool operator!=(Iterator it) const
+        {
+            return m_i != it.m_i;
+        }
 
-        Color operator*() const;
+        void operator++()
+        {
+            ++m_i;
+        }
+
+        Color operator*() const
+        {
+            return Color(m_i);
+        }
 
     private:
-        const IntType m_nu_colors;
-
         IntType m_i;
     };
 
+    class Range
+    {
+    public:
+        Range(IntType nu_colors)
+            : m_nu_colors(nu_colors)
+        { }
+
+        Iterator begin() const
+        {
+            return 0;
+        }
+
+        Iterator end() const
+        {
+            return m_nu_colors;
+        }
+
+    private:
+        IntType m_nu_colors;
+    };
+
     static const IntType range = 4;
 
     Color();
@@ -79,34 +114,6 @@ private:
     bool is_initialized() const;
 };
 
-typedef Color::Iterator ColorIterator;
-
-inline Color::InvalidString::InvalidString(const string& s)
-    : Exception(s)
-{
-}
-
-inline Color::Iterator::Iterator(IntType nu_colors)
-    : m_nu_colors(nu_colors),
-      m_i(0)
-{
-}
-
-inline Color::Iterator::operator bool() const
-{
-    return m_i < m_nu_colors;
-}
-
-inline void Color::Iterator::operator++()
-{
-    ++m_i;
-}
-
-inline Color Color::Iterator::operator*() const
-{
-    LIBBOARDGAME_ASSERT(*this);
-    return Color(m_i);
-}
 
 inline Color::Color()
 {
@@ -125,7 +132,7 @@ inline bool Color::operator==(const Color& c) const
 {
     LIBBOARDGAME_ASSERT(is_initialized());
     LIBBOARDGAME_ASSERT(c.is_initialized());
-    return (m_i == c.m_i);
+    return m_i == c.m_i;
 }
 
 inline bool Color::operator!=(const Color& c) const
@@ -137,7 +144,7 @@ inline bool Color::operator<(const Color& c) const
 {
     LIBBOARDGAME_ASSERT(is_initialized());
     LIBBOARDGAME_ASSERT(c.is_initialized());
-    return (m_i < c.m_i);
+    return m_i < c.m_i;
 }
 
 inline Color Color::get_next(IntType nu_colors) const
@@ -180,15 +187,16 @@ istream& operator>>(istream& in, Color& c);
 
 //-----------------------------------------------------------------------------
 
-/** Macro to unroll a loop over all colors. */
-#define LIBPENTOBI_FOREACH_COLOR(c, statement)               \
-    {                                                        \
-        static_assert(Color::range == 4, "");                \
-        { Color c(0);  statement; }                          \
-        { Color c(1);  statement; }                          \
-        { Color c(2);  statement; }                          \
-        { Color c(3);  statement; }                          \
-    }
+/** Unrolled loop over all colors. */
+template<class FUNCTION>
+inline void for_each_color(FUNCTION f)
+{
+    static_assert(Color::range == 4, "");
+    f(Color(0));
+    f(Color(1));
+    f(Color(2));
+    f(Color(3));
+}
 
 //-----------------------------------------------------------------------------
 
