@@ -10,7 +10,6 @@
 
 #include "Engine.h"
 
-#include "MoveList.h"
 #include "libboardgame_sgf/TreeReader.h"
 #include "libboardgame_sgf/SgfUtil.h"
 #include "libboardgame_util/Log.h"
@@ -29,11 +28,7 @@ using libboardgame_util::RandomGenerator;
 //-----------------------------------------------------------------------------
 
 Engine::Engine(Variant variant)
-    : m_accept_illegal(false),
-      m_show_board(false),
-      m_resign(true),
-      m_game(variant),
-      m_player(nullptr)
+    : m_game(variant)
 {
     add("all_legal", &Engine::cmd_all_legal);
     add("clear_board", &Engine::cmd_clear_board);
@@ -59,8 +54,9 @@ void Engine::board_changed()
 void Engine::cmd_all_legal(const Arguments& args, Response& response)
 {
     auto& bd = get_board();
-    unique_ptr<MoveList> moves(new MoveList());
-    bd.gen_moves(get_color_arg(args), *moves);
+    unique_ptr<MoveList> moves(new MoveList);
+    unique_ptr<MoveMarker> marker(new MoveMarker);
+    bd.gen_moves(get_color_arg(args), *marker, *moves);
     for (Move mv : *moves)
         response << bd.to_string(mv, false) << '\n';
 }
@@ -76,8 +72,8 @@ void Engine::cmd_final_score(Response& response)
     auto& bd = get_board();
     if (get_nu_players(bd.get_variant()) > 2)
     {
-        for (ColorIterator i(bd.get_nu_colors()); i; ++i)
-            response << bd.get_points(*i) << ' ';
+        for (Color c : bd.get_colors())
+            response << bd.get_points(c) << ' ';
     }
     else
     {
@@ -93,7 +89,7 @@ void Engine::cmd_final_score(Response& response)
 
 void Engine::cmd_g(Response& response)
 {
-    genmove(get_board().get_to_play(), response);
+    genmove(get_board().get_effective_to_play(), response);
 }
 
 void Engine::cmd_genmove(const Arguments& args, Response& response)
@@ -128,12 +124,12 @@ void Engine::cmd_loadsgf(const Arguments& args)
         const SgfNode* node = nullptr;
         if (move_number != -1)
             node = m_game.get_tree().get_node_before_move_number(move_number);
-        if (node == nullptr)
+        if (! node)
             node = &get_last_node(m_game.get_root());
         m_game.goto_node(*node);
         board_changed();
     }
-    catch (const Exception& e)
+    catch (const runtime_error& e)
     {
         throw Failure(e.what());
     }
@@ -154,7 +150,7 @@ void Engine::cmd_move_info(const Arguments& args, Response& response)
         {
             mv = bd.from_string(args.get());
         }
-        catch (Exception&)
+        catch (const runtime_error&)
         {
             ostringstream msg;
             msg << "invalid argument '" << args.get()
@@ -162,6 +158,7 @@ void Engine::cmd_move_info(const Arguments& args, Response& response)
             throw Failure(msg.str());
         }
     }
+    auto& geo = bd.get_geometry();
     auto& info = bd.get_move_info(mv);
     Piece piece = info.get_piece();
     auto& info_ext = bd.get_move_info_ext(mv);
@@ -173,17 +170,17 @@ void Engine::cmd_move_info(const Arguments& args, Response& response)
         << " (" << bd.get_piece_info(piece).get_name() << ")\n"
         << "Points:";
     for (Point p : info)
-        response << ' ' << bd.write(p);
+        response << ' ' << geo.to_string(p);
     response
         << "\n"
         << "Adj:    ";
     for (auto i = info_ext.begin_adj(); i != info_ext.end_adj(); ++i)
-        response << bd.write(*i) << " ";
+        response << geo.to_string(*i) << " ";
     response
         << "\n"
         << "Attach: ";
     for (auto i = info_ext.begin_attach(); i != info_ext.end_attach(); ++i)
-        response << bd.write(*i) << " ";
+        response << geo.to_string(*i) << " ";
     response
         << "\n"
         << "BrkSym: " << info_ext_2.breaks_symmetry << "\n"
@@ -227,8 +224,8 @@ void Engine::cmd_point_integers(Response& response)
 {
     auto& geo = get_board().get_geometry();
     Grid<int> grid;
-    for (GeometryIterator i(geo); i; ++i)
-        grid[*i] = (*i).to_int();
+    for (Point p : geo)
+        grid[p] = p.to_int();
     response << '\n' << grid.to_string(geo);
 }
 
@@ -277,7 +274,10 @@ void Engine::genmove(Color c, Response& response)
     auto& player = get_player();
     auto mv = player.genmove(bd, c);
     if (mv.is_null())
-        throw Failure("player failed to generate a move");
+    {
+        response << "pass";
+        return;
+    }
     if (! bd.is_legal(c, mv))
     {
         ostringstream msg;
@@ -291,8 +291,7 @@ void Engine::genmove(Color c, Response& response)
     }
     m_game.play(c, mv, true);
     response << bd.to_string(mv, false);
-    if (! mv.is_pass())
-        board_changed();
+    board_changed();
 }
 
 Color Engine::get_color_arg(const Arguments& args) const
@@ -337,7 +336,7 @@ Color Engine::get_color_arg(const Arguments& args, unsigned i) const
 
 PlayerBase& Engine::get_player() const
 {
-    if (m_player == nullptr)
+    if (! m_player)
         throw Failure("no player set");
     return *m_player;
 }
@@ -355,15 +354,16 @@ void Engine::play(Color c, const Arguments& args, unsigned arg_move_begin)
         else
             mv = bd.from_string(args.get_remaining_line(arg_move_begin - 1));
     }
-    catch (const Exception& e)
+    catch (const runtime_error& e)
     {
         throw Failure(e.what());
     }
+    if (mv.is_null())
+        throw Failure("play pass not supported (anymore)");
     if (! m_accept_illegal && ! bd.is_legal(c, mv))
         throw Failure("illegal move");
     m_game.play(c, mv, true);
-    if (! mv.is_pass())
-        board_changed();
+    board_changed();
 }
 
 void Engine::set_player(PlayerBase& player)
