@@ -16,7 +16,6 @@
 
 namespace libpentobi_base {
 
-using namespace std;
 using libboardgame_sgf::InvalidTree;
 using libboardgame_sgf::util::back_to_main_variation;
 using libboardgame_sgf::util::is_main_variation;
@@ -31,13 +30,6 @@ Game::Game(Variant variant)
     init(variant);
 }
 
-Game::Game(unique_ptr<SgfNode>& root)
-  : m_bd(new Board(m_tree.get_variant())),
-    m_tree(m_tree.get_variant())
-{
-    init(root);
-}
-
 void Game::add_setup(Color c, Move mv)
 {
     auto& node = m_tree.add_setup(*m_current, c, mv);
@@ -61,7 +53,7 @@ void Game::goto_node(const SgfNode& node)
     catch (const InvalidTree&)
     {
         // Try to restore the old state.
-        if (old == nullptr)
+        if (! old)
             m_current = &node;
         else
         {
@@ -114,7 +106,7 @@ void Game::play(ColorMove mv, bool always_create_new_node)
     const SgfNode* child = nullptr;
     if (! always_create_new_node)
         child = m_tree.find_child_with_move(*m_current, mv);
-    if (child != nullptr)
+    if (child)
         m_current = child;
     else
     {
