@@ -17,7 +17,6 @@ namespace node_util {
 
 using libboardgame_sgf::InvalidPropertyValue;
 using libboardgame_sgf::InvalidTree;
-using libboardgame_sgf::PropertyIterator;
 using libboardgame_util::split;
 using libboardgame_util::trim;
 
@@ -103,8 +102,7 @@ bool get_move(const SgfNode& node, Variant variant, Color& c,
     // values instead of a single value as used by Pentobi <= 0.2, but it
     // is deprecated
     points.clear();
-    auto width = get_geometry(variant).get_width();
-    auto height = get_geometry(variant).get_height();
+    auto& geo = get_geometry(variant);
     for (const auto& s : values)
     {
         if (trim(s).empty())
@@ -113,14 +111,8 @@ bool get_move(const SgfNode& node, Variant variant, Color& c,
         for (const auto& p_str : v)
         {
             Point p;
-            try
-            {
-                p = Point::from_string(p_str, width, height);
-            }
-            catch (const Point::InvalidString&)
-            {
+            if (! geo.from_string(p_str, p))
                 throw InvalidPropertyValue(id, p_str);
-            }
             points.push_back(p);
         }
     }
@@ -147,9 +139,9 @@ bool get_player(const SgfNode& node, Color& c)
 
 bool has_setup(const SgfNode& node)
 {
-    for (PropertyIterator i(node); i; ++i)
-        if (i->id == "AB" || i->id == "AW" || i->id == "A1" || i->id == "A2"
-            || i->id == "A3" || i->id == "A4" || i->id == "AE")
+    for (auto& i : node.get_properties())
+        if (i.id == "AB" || i.id == "AW" || i.id == "A1" || i.id == "A2"
+                || i.id == "A3" || i.id == "A4" || i.id == "AE")
             return true;
     return false;
 }
