@@ -36,11 +36,12 @@ public:
     /** The number of neighbors used for computing the adjacent status.
         The adjacent status is a single number that encodes the forbidden
         status of the first adj_status_nu_adj neighbors (from the list
-        Geometry::get_adj_diag()). It is used for speeding up the matching of
-        moves at a given point. Increasing this number will make the
-        precomputed lists shorter but exponentially increase the number of
-        lists and the total memory used for all lists. Therefore, the optimal
-        value for speeding up the matching depends on the CPU cache size. */
+        Geometry::get_adj() concatenated with Geometry::get_diag()). It is used
+        for speeding up the matching of moves at a given point. Increasing this
+        number will make the precomputed lists shorter but exponentially
+        increase the number of lists and the total memory used for all lists.
+        Therefore, the optimal value for speeding up the matching depends on
+        the CPU cache size. */
 #if PENTOBI_LOW_RESOURCES
     static const unsigned adj_status_nu_adj = 4;
 #else
@@ -51,23 +52,15 @@ public:
 
     /** Begin/end range for lists with moves at a given point.
         See get_moves(). */
-    class LocalMovesListRange
+    struct Range
     {
-    public:
-        LocalMovesListRange(const Move* begin, const Move* end)
-        {
-            m_begin = begin;
-            m_end = end;
-        }
+        const Move* m_begin;
+
+        const Move* m_end;
 
         const Move* begin() const { return m_begin; }
 
         const Move* end() const { return m_end; }
-
-    private:
-        const Move* m_begin;
-
-        const Move* m_end;
     };
 
     /** Clear storage of moves for all move lists during construction. */
@@ -83,18 +76,17 @@ public:
     void set_list_range(Point p, unsigned adj_status, Piece piece,
                         unsigned begin, unsigned size)
     {
-        m_moves_range[p][adj_status][piece] = ListIndex(begin, size);
+        m_moves_range[p][adj_status][piece] = CompressedRange(begin, size);
     }
 
     /** Get all moves of a piece at a point constrained by the forbidden
         status of adjacent points. */
-    LocalMovesListRange get_moves(Piece piece, Point p,
-                                  unsigned adj_status = 0) const
+    Range get_moves(Piece piece, Point p, unsigned adj_status = 0) const
     {
-        ListIndex idx = m_moves_range[p][adj_status][piece];
-        auto begin = m_move_lists.begin() + idx.begin;
-        auto end = begin + idx.size;
-        return LocalMovesListRange(begin, end);
+        auto& range = m_moves_range[p][adj_status][piece];
+        auto begin = move_lists_begin() + range.begin;
+        auto end = begin + range.size;
+        return Range{begin, end};
     }
 
     /** Begin of storage for move lists.
@@ -103,21 +95,18 @@ public:
         index of old iterators with the current get_size() to ensure that
         we don't overwrite any old content that we still need to read
         during the construction. */
-    const Move* move_lists_begin() const { return m_move_lists.begin(); }
+    const Move* move_lists_begin() const { return &(*m_move_lists.begin()); }
 
 private:
-    /** Compressed begin/end range for lists with moves at a given point.
-        This struct will be unpacked into a LocalMovesListRange as a return
-        value for get_moves(). */
-    struct ListIndex
+    struct CompressedRange
     {
         unsigned begin : 24;
 
         unsigned size : 8;
 
-        ListIndex() { }
+        CompressedRange() { }
 
-        ListIndex(unsigned begin, unsigned size)
+        CompressedRange(unsigned begin, unsigned size)
         {
             LIBBOARDGAME_ASSERT(begin < max_move_lists_sum_length);
             LIBBOARDGAME_ASSERT(begin + size <= max_move_lists_sum_length);
@@ -129,7 +118,7 @@ private:
     };
 
     /** See m_move_lists. */
-    Grid<array<PieceMap<ListIndex>, nu_adj_status>> m_moves_range;
+    Grid<array<PieceMap<CompressedRange>, nu_adj_status>> m_moves_range;
 
     /** Compact representation of lists of moves of a piece at a point
         constrained by the forbidden status of adjacent points.
