@@ -14,24 +14,15 @@
 #include <cmath>
 #include "Util.h"
 #include "libboardgame_util/Log.h"
-#include "libpentobi_base/AdjIterator.h"
-#include "libpentobi_base/DiagIterator.h"
 
 using namespace std;
 using libboardgame_util::log;
-using libpentobi_base::AdjIterator;
-using libpentobi_base::BoardIterator;
-using libpentobi_base::DiagIterator;
-using libpentobi_base::GeometryIterator;
 using libpentobi_base::Move;
 using libpentobi_base::PointState;
 
 //-----------------------------------------------------------------------------
 
 BoardPainter::BoardPainter()
-    : m_hasPainted(false),
-      m_coordinates(false),
-      m_coordinateColor(Qt::black)
 {
     m_font.setFamily("Helvetica");
     m_font.setStyleHint(QFont::SansSerif);
@@ -45,8 +36,7 @@ BoardPainter::BoardPainter()
 }
 
 BoardPainter::~BoardPainter()
-{
-}
+{ }
 
 void BoardPainter::drawCoordinates(QPainter& painter, bool isTrigon)
 {
@@ -124,31 +114,74 @@ void BoardPainter::drawLabel(QPainter& painter, qreal x, qreal y,
 
 void BoardPainter::drawLabels(QPainter& painter,
                               const Grid<PointState>& pointState,
-                              Variant variant,
-                              const Grid<QString>* labels)
+                              Variant variant, const Grid<QString>& labels)
 {
-    if (labels == 0)
-        return;
     bool isTrigon = (variant == Variant::trigon
                      || variant == Variant::trigon_2
                      || variant == Variant::trigon_3);
-    for (GeometryIterator i(*m_geo); i; ++i)
-        if (! (*labels)[*i].isEmpty())
+    for (Point p : *m_geo)
+        if (! labels[p].isEmpty())
         {
-            PointState s = pointState[*i];
-            painter.setPen(Util::getLabelColor(variant, s));
-            qreal x = i->get_x(m_geo->get_width()) * m_fieldWidth;
-            qreal y = i->get_y(m_geo->get_width()) * m_fieldHeight;
+            painter.setPen(Util::getLabelColor(variant, pointState[p]));
+            qreal x = m_geo->get_x(p) * m_fieldWidth;
+            qreal y = m_geo->get_y(p) * m_fieldHeight;
             qreal width = m_fieldWidth;
             qreal height = m_fieldHeight;
             if (isTrigon)
             {
-                bool isUpward = (m_geo->get_point_type(*i) == 0);
+                bool isUpward = (m_geo->get_point_type(p) == 0);
                 if (isUpward)
                     y += 0.333 * height;
                 height = 0.666 * height;
             }
-            drawLabel(painter, x, y, width, height, (*labels)[*i], false);
+            drawLabel(painter, x, y, width, height, labels[p], false);
+        }
+}
+
+void BoardPainter::drawMarks(QPainter& painter,
+                             const Grid<PointState>& pointState,
+                             Variant variant, const Grid<int>& marks)
+{
+    bool isTrigon = (variant == Variant::trigon
+                     || variant == Variant::trigon_2
+                     || variant == Variant::trigon_3);
+    for (Point p : *m_geo)
+        if (marks[p] & (dot | circle))
+        {
+            qreal x = (static_cast<float>(m_geo->get_x(p)) + 0.5f)
+                    * m_fieldWidth;
+            qreal y = (static_cast<float>(m_geo->get_y(p)) + 0.5f)
+                    * m_fieldHeight;
+            qreal size;
+            if (isTrigon)
+            {
+                bool isUpward = (m_geo->get_point_type(p) == 0);
+                if (isUpward)
+                    y += 0.167 * m_fieldHeight;
+                else
+                    y -= 0.167 * m_fieldHeight;
+                size = 0.1 * m_fieldHeight;
+            }
+            else
+                size = 0.12 * m_fieldHeight;
+            QColor color = Util::getMarkColor(variant, pointState[p]);
+            qreal penWidth = 0.05 * m_fieldHeight;
+            if (marks[p] & dot)
+            {
+                color.setAlphaF(0.5);
+                painter.setPen(Qt::NoPen);
+                painter.setBrush(color);
+                size *= (1 + 0.25 * penWidth);
+            }
+            else
+            {
+                color.setAlphaF(0.6);
+                QPen pen(color);
+                pen.setWidthF(penWidth);
+                painter.setPen(pen);
+                painter.setBrush(Qt::NoBrush);
+            }
+            painter.drawEllipse(QPointF(x, y), size, size);
         }
 }
 
@@ -206,36 +239,26 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
     m_fieldHeight = ratio * m_fieldWidth;
     m_boardOffset = QPointF(0.5 * (width - m_fieldWidth * m_width),
                             0.5 * (height - m_fieldHeight * m_height));
-    if (m_isTrigon)
-    {
-        m_font.setPointSizeF(0.6 * m_fieldWidth);
-        m_fontSemiCondensed.setPointSizeF(0.6 * m_fieldWidth);
-        m_fontCondensed.setPointSizeF(0.6 * m_fieldWidth);
-        m_fontCoordLabels.setPointSizeF(0.6 * m_fieldWidth);
-    }
-    else
-    {
-        m_font.setPointSizeF(0.4 * m_fieldWidth);
-        m_fontSemiCondensed.setPointSizeF(0.4 * m_fieldWidth);
-        m_fontCondensed.setPointSizeF(0.4 * m_fieldWidth);
-        m_fontCoordLabels.setPointSizeF(0.36 * m_fieldWidth);
-    }
+    int fontSize = static_cast<int>((m_isTrigon ? 0.7 : 0.5) * m_fieldWidth);
+    m_font.setPixelSize(fontSize);
+    m_fontSemiCondensed.setPixelSize(fontSize);
+    m_fontCondensed.setPixelSize(fontSize);
+    m_fontCoordLabels.setPixelSize(fontSize);
     painter.save();
     painter.translate(m_boardOffset);
     if (m_coordinates)
         drawCoordinates(painter, m_isTrigon);
     m_startingPoints.init(variant, *m_geo);
-    auto geoWidth = m_geo->get_width();
-    for (GeometryIterator i(*m_geo); i; ++i)
+    for (Point p : *m_geo)
     {
-        int x = i->get_x(geoWidth);
-        int y = i->get_y(geoWidth);
+        int x = m_geo->get_x(p);
+        int y = m_geo->get_y(p);
         qreal fieldX = x * m_fieldWidth;
         qreal fieldY = y * m_fieldHeight;
         if (m_isTrigon)
         {
             bool isUpward = (m_geo->get_point_type(x, y) == 0);
-            if (m_startingPoints.is_colorless_starting_point(*i))
+            if (m_startingPoints.is_colorless_starting_point(p))
                 Util::paintEmptyTriangleStartingPoint(painter, isUpward, fieldX,
                                                       fieldY, m_fieldWidth,
                                                       m_fieldHeight);
@@ -245,9 +268,9 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
         }
         else
         {
-            if (m_startingPoints.is_colored_starting_point(*i))
+            if (m_startingPoints.is_colored_starting_point(p))
             {
-                Color color = m_startingPoints.get_starting_point_color(*i);
+                Color color = m_startingPoints.get_starting_point_color(p);
                 Util::paintEmptySquareStartingPoint(painter, variant,
                                                     color, fieldX, fieldY,
                                                     m_fieldWidth);
@@ -261,17 +284,17 @@ void BoardPainter::paintEmptyBoard(QPainter& painter, unsigned width,
 
 void BoardPainter::paintPieces(QPainter& painter,
                                const Grid<PointState>& pointState,
-                               const Grid<QString>* labels)
+                               const Grid<QString>* labels,
+                               const Grid<int>* marks)
 {
     painter.setRenderHint(QPainter::Antialiasing, true);
     painter.save();
     painter.translate(m_boardOffset);
-    auto geoWidth = m_geo->get_width();
-    for (GeometryIterator i(*m_geo); i; ++i)
+    for (Point p : *m_geo)
     {
-        int x = i->get_x(geoWidth);
-        int y = i->get_y(geoWidth);
-        PointState s = pointState[*i];
+        int x = m_geo->get_x(p);
+        int y = m_geo->get_y(p);
+        PointState s = pointState[p];
         qreal fieldX = x * m_fieldWidth;
         qreal fieldY = y * m_fieldHeight;
         if (m_isTrigon)
@@ -289,7 +312,10 @@ void BoardPainter::paintPieces(QPainter& painter,
                                        fieldX, fieldY, m_fieldWidth);
         }
     }
-    drawLabels(painter, pointState, m_variant, labels);
+    if (marks)
+        drawMarks(painter, pointState, m_variant, *marks);
+    if (labels)
+        drawLabels(painter, pointState, m_variant, *labels);
     painter.restore();
 }
 
@@ -314,11 +340,12 @@ void BoardPainter::paintSelectedPiece(QPainter& painter, Color c,
         saturation = 0.5;
         flat = true;
     }
-    auto geoWidth = m_geo->get_width();
     for (Point p : points)
     {
-        qreal fieldX = p.get_x(geoWidth) * m_fieldWidth;
-        qreal fieldY = p.get_y(geoWidth) * m_fieldHeight;
+        if (p.is_null())
+            continue;
+        qreal fieldX = m_geo->get_x(p) * m_fieldWidth;
+        qreal fieldY = m_geo->get_y(p) * m_fieldHeight;
         if (m_isTrigon)
         {
             bool isUpward = (m_geo->get_point_type(p) == 0);
