@@ -34,6 +34,13 @@ using libpentobi_base::StartingPoints;
 class BoardPainter
 {
 public:
+    enum
+    {
+        dot = 1 << 1,
+
+        circle = 1 << 2
+    };
+
     BoardPainter();
 
     ~BoardPainter();
@@ -50,7 +57,8 @@ public:
         uses the arguments from the paintEmptyBoard() function to determine the
         board properties. */
     void paintPieces(QPainter& painter, const Grid<PointState>& pointState,
-                     const Grid<QString>* labels = nullptr);
+                     const Grid<QString>* labels = nullptr,
+                     const Grid<int>* marks = nullptr);
 
     /** Paint the selected piece.
         Paints the selected piece either transparent (if not legal) or opaque
@@ -68,9 +76,9 @@ public:
     bool hasPainted() const;
 
 private:
-    bool m_hasPainted;
+    bool m_hasPainted = false;
 
-    bool m_coordinates;
+    bool m_coordinates = false;
 
     bool m_isTrigon;
 
@@ -84,7 +92,7 @@ private:
     /** The height of the last board painted. */
     int m_height;
 
-    QColor m_coordinateColor;
+    QColor m_coordinateColor = Qt::black;
 
     qreal m_fieldWidth;
 
@@ -108,7 +116,10 @@ private:
                    qreal height, const QString& label, bool isCoordLabel);
 
     void drawLabels(QPainter& painter, const Grid<PointState>& pointState,
-                    Variant variant, const Grid<QString>* labels);
+                    Variant variant, const Grid<QString>& labels);
+
+    void drawMarks(QPainter& painter, const Grid<PointState>& pointState,
+                   Variant variant, const Grid<int>& marks);
 };
 
 inline void BoardPainter::setCoordinateColor(const QColor& color)
