@@ -21,7 +21,6 @@
 using namespace std;
 using libboardgame_base::Transform;
 using libboardgame_util::log;
-using libpentobi_base::BoardIterator;
 using libpentobi_base::BoardType;
 using libpentobi_base::Geometry;
 using libpentobi_base::MovePoints;
@@ -36,25 +35,31 @@ namespace {
 bool allPointEmpty(const Board& bd, Move mv)
 {
     for (Point p : bd.get_move_info(mv))
-        if (! bd.is_empty(p))
+        if (! bd.get_point_state(p).is_empty())
             return false;
     return true;
 }
 
+QPixmap* createPixmap(const QPainter& painter, const QSize& size)
+{
+#if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
+    auto devicePixelRatio = painter.device()->devicePixelRatio();
+    auto pixmap = new QPixmap(devicePixelRatio * size);
+    pixmap->setDevicePixelRatio(devicePixelRatio);
+#else
+    LIBBOARDGAME_UNUSED(painter);
+    auto pixmap = new QPixmap(size);
+#endif
+    return pixmap;
+}
+
 } // namespace
 
 //-----------------------------------------------------------------------------
 
 GuiBoard::GuiBoard(QWidget* parent, const Board& bd)
     : QWidget(parent),
-      m_bd(bd),
-      m_isInitialized(false),
-      m_freePlacement(false),
-      m_emptyBoardDirty(true),
-      m_dirty(true),
-      m_selectedPiece(Piece::null()),
-      m_selectedPieceTransform(0),
-      m_isMoveShown(false)
+      m_bd(bd)
 {
     setMinimumWidth(350);
     setMinimumHeight(350);
@@ -70,11 +75,14 @@ void GuiBoard::changeEvent(QEvent* event)
 
 void GuiBoard::clearMarkup()
 {
-    for (Geometry::Iterator i(m_bd.get_geometry()); i; ++i)
-        setLabel(*i, "");
+    for (Point p : m_bd)
+    {
+        m_marks[p] = 0;
+        setLabel(p, "");
+    }
 }
 
-void GuiBoard::clearSelectedPiece()
+void GuiBoard::clearPiece()
 {
     m_selectedPiece = Piece::null();
     m_selectedPieceTransform = nullptr;
@@ -89,15 +97,14 @@ void GuiBoard::copyFromBoard(const Board& bd)
     {
         m_variant = bd.get_variant();
         m_isInitialized = true;
-        m_pointState = bd.get_grid();
+        m_pointState.copy_from(bd.get_point_state(), geo);
         m_labels.fill("", geo);
+        m_marks.fill(0, geo);
         setEmptyBoardDirty();
     }
     else
     {
-        for (BoardIterator i(bd); i; ++i)
-            if (m_pointState[*i] != bd.get_point_state(*i))
-                m_pointState[*i] = bd.get_point_state(*i);
+        m_pointState.copy_from(bd.get_point_state(), geo);
         setDirty();
     }
 }
@@ -108,10 +115,10 @@ Move GuiBoard::findSelectedPieceMove()
         return Move::null();
     const PiecePoints& points =
         m_bd.get_piece_info(m_selectedPiece).get_points();
+    auto& geo = m_bd.get_geometry();
+    int width = static_cast<int>(geo.get_width());
+    int height = static_cast<int>(geo.get_height());
     MovePoints movePoints;
-    auto geoWidth = m_bd.get_geometry().get_width();
-    int width = static_cast<int>(geoWidth);
-    int height = static_cast<int>(m_bd.get_geometry().get_height());
     for (CoordPoint p : points)
     {
         p = m_selectedPieceTransform->get_transformed(p);
@@ -119,12 +126,16 @@ Move GuiBoard::findSelectedPieceMove()
         int y = p.y + m_selectedPieceOffset.y;
         if (x < 0 || x >= width || y < 0 || y >= height)
             return Move::null();
-        movePoints.push_back(Point(x, y, geoWidth));
+        Point pp = geo.get_point(x, y);
+        if (pp.is_null())
+            return Move::null();
+        movePoints.push_back(pp);
     }
     Move mv;
     if (! m_bd.find_move(movePoints, mv)
-        || (m_freePlacement && ! allPointEmpty(m_bd, mv))
-        || (! m_freePlacement && ! m_bd.is_legal(m_selectedPieceColor, mv)))
+            || (m_freePlacement && ! allPointEmpty(m_bd, mv))
+            || (! m_freePlacement
+                && ! m_bd.is_legal(m_selectedPieceColor, mv)))
         return Move::null();
     else
         return mv;
@@ -158,14 +169,14 @@ void GuiBoard::mousePressEvent(QMouseEvent* event)
         CoordPoint p = m_boardPainter.getCoordPoint(event->x(), event->y());
         auto& geo = m_bd.get_geometry();
         if (geo.is_onboard(p))
-            emit pointClicked(Point(p.x, p.y, geo.get_width()));
+            emit pointClicked(geo.get_point(p.x, p.y));
         return;
     }
     setSelectedPieceOffset(*event);
-    placeSelectedPiece();
+    placePiece();
 }
 
-void GuiBoard::moveSelectedPieceDown()
+void GuiBoard::movePieceDown()
 {
     if (m_selectedPiece.is_null())
         return;
@@ -199,7 +210,7 @@ void GuiBoard::moveSelectedPieceDown()
     }
 }
 
-void GuiBoard::moveSelectedPieceLeft()
+void GuiBoard::movePieceLeft()
 {
     if (m_selectedPiece.is_null())
         return;
@@ -227,7 +238,7 @@ void GuiBoard::moveSelectedPieceLeft()
     }
 }
 
-void GuiBoard::moveSelectedPieceRight()
+void GuiBoard::movePieceRight()
 {
     if (m_selectedPiece.is_null())
         return;
@@ -255,7 +266,7 @@ void GuiBoard::moveSelectedPieceRight()
     }
 }
 
-void GuiBoard::moveSelectedPieceUp()
+void GuiBoard::movePieceUp()
 {
     if (m_selectedPiece.is_null())
         return;
@@ -293,14 +304,15 @@ void GuiBoard::paintEvent(QPaintEvent*)
 {
     if (! m_isInitialized)
         return;
-    if (m_emptyBoardPixmap == nullptr || m_emptyBoardPixmap->size() != size())
+    QPainter painter(this);
+    if (! m_emptyBoardPixmap || m_emptyBoardPixmap->size() != size())
     {
-        m_emptyBoardPixmap.reset(new QPixmap(size()));
+        m_emptyBoardPixmap.reset(createPixmap(painter, size()));
         m_emptyBoardDirty = true;
     }
-    if (m_boardPixmap == nullptr || m_boardPixmap->size() != size())
+    if (! m_boardPixmap || m_boardPixmap->size() != size())
     {
-        m_boardPixmap.reset(new QPixmap(size()));
+        m_boardPixmap.reset(createPixmap(painter, size()));
         m_dirty = true;
     }
     if (m_emptyBoardDirty)
@@ -319,10 +331,9 @@ void GuiBoard::paintEvent(QPaintEvent*)
         m_boardPixmap->fill(Qt::transparent);
         QPainter painter(m_boardPixmap.get());
         painter.drawPixmap(0, 0, *m_emptyBoardPixmap);
-        m_boardPainter.paintPieces(painter, m_pointState, &m_labels);
+        m_boardPainter.paintPieces(painter, m_pointState, &m_labels, &m_marks);
         m_dirty = false;
     }
-    QPainter painter(this);
     painter.drawPixmap(0, 0, *m_boardPixmap);
     if (m_isMoveShown)
     {
@@ -341,7 +352,7 @@ void GuiBoard::paintEvent(QPaintEvent*)
     }
 }
 
-void GuiBoard::placeSelectedPiece()
+void GuiBoard::placePiece()
 {
     auto mv = findSelectedPieceMove();
     if (! mv.is_null())
@@ -398,6 +409,17 @@ void GuiBoard::setLabel(Point p, const QString& text)
     }
 }
 
+void GuiBoard::setMark(Point p, int mark, bool enable)
+{
+    if (! m_isInitialized)
+        return;
+    if (((m_marks[p] & mark) != 0) != enable)
+    {
+        m_marks[p] ^= mark;
+        setDirty();
+    }
+}
+
 void GuiBoard::setSelectedPieceOffset(const QMouseEvent& event)
 {
     setSelectedPieceOffset(m_boardPainter.getCoordPoint(event.x(), event.y()));
@@ -473,7 +495,7 @@ void GuiBoard::setSelectedPiecePoints()
             int x = p.x + m_selectedPieceOffset.x;
             int y = p.y + m_selectedPieceOffset.y;
             if (x >= 0 && x < width && y >= 0 && y < height)
-                m_selectedPiecePoints.push_back(Point(x, y, geo.get_width()));
+                m_selectedPiecePoints.push_back(geo.get_point(x, y));
         }
     }
     update();
