@@ -59,24 +59,26 @@ public:
 
     void setLabel(Point p, const QString& text);
 
+    void setMark(Point p, int mark, bool enable = true);
+
     void clearMarkup();
 
     void setFreePlacement(bool enable);
 
 public slots:
-    void clearSelectedPiece();
+    void clearPiece();
 
     void selectPiece(Color color, Piece piece);
 
-    void moveSelectedPieceLeft();
+    void movePieceLeft();
 
-    void moveSelectedPieceRight();
+    void movePieceRight();
 
-    void moveSelectedPieceUp();
+    void movePieceUp();
 
-    void moveSelectedPieceDown();
+    void movePieceDown();
 
-    void placeSelectedPiece();
+    void placePiece();
 
 signals:
     void play(Color color, Move mv);
@@ -97,27 +99,29 @@ protected:
 private:
     const Board& m_bd;
 
-    bool m_isInitialized;
+    bool m_isInitialized = false;
 
-    bool m_freePlacement;
+    bool m_freePlacement = false;
 
     /** Does the empty board need redrawing? */
-    bool m_emptyBoardDirty;
+    bool m_emptyBoardDirty = true;
 
     /** Do the pieces and markup on the board need redrawing?
         If true, the cached board pixmap needs to be repainted. This does not
         include the selected piece (the selected piece is always painted). */
-    bool m_dirty;
+    bool m_dirty = true;
+
+    bool m_isMoveShown = false;
 
     Variant m_variant;
 
     Board::PointStateGrid m_pointState;
 
-    Piece m_selectedPiece;
+    Piece m_selectedPiece = Piece::null();
 
     Color m_selectedPieceColor;
 
-    const Transform* m_selectedPieceTransform;
+    const Transform* m_selectedPieceTransform = nullptr;
 
     CoordPoint m_selectedPieceOffset;
 
@@ -125,14 +129,14 @@ private:
 
     Grid<QString> m_labels;
 
+    Grid<int> m_marks;
+
     BoardPainter m_boardPainter;
 
     unique_ptr<QPixmap> m_emptyBoardPixmap;
 
     unique_ptr<QPixmap> m_boardPixmap;
 
-    bool m_isMoveShown;
-
     Color m_currentMoveShownColor;
 
     MovePoints m_currentMoveShownPoints;
