@@ -10,6 +10,7 @@
 
 #include "GuiBoardUtil.h"
 
+#include "libboardgame_sgf/SgfUtil.h"
 #include "libboardgame_util/Log.h"
 #include "libboardgame_util/StringUtil.h"
 
@@ -18,10 +19,9 @@ namespace gui_board_util {
 using libpentobi_base::ColorMove;
 using libpentobi_base::PentobiTree;
 using libboardgame_sgf::SgfNode;
-using libboardgame_sgf::ChildIterator;
+using libboardgame_sgf::util::is_main_variation;
 using libboardgame_util::get_letter_coord;
 using libboardgame_util::log;
-using libboardgame_util::Exception;
 
 //-----------------------------------------------------------------------------
 
@@ -75,15 +75,15 @@ bool getVariationIndex(const PentobiTree& tree, const SgfNode& node,
                        unsigned& moveIndex)
 {
     auto parent = node.get_parent_or_null();
-    if (parent == nullptr || parent->has_single_child())
+    if (! parent || parent->has_single_child())
         return false;
     unsigned nuSiblingMoves = 0;
     moveIndex = 0;
-    for (ChildIterator i(*parent); i; ++i)
+    for (auto& i : parent->get_children())
     {
-        if (! tree.has_move(*i))
+        if (! tree.has_move(i))
             continue;
-        if (&(*i) == &node)
+        if (&i == &node)
             moveIndex = nuSiblingMoves;
         ++nuSiblingMoves;
     }
@@ -92,13 +92,22 @@ bool getVariationIndex(const PentobiTree& tree, const SgfNode& node,
     return true;
 }
 
-void setMoveLabel(GuiBoard& guiBoard, const Game& game, const SgfNode& node,
-                  unsigned moveNumber, ColorMove mv, bool markVariations)
+void markMove(GuiBoard& guiBoard, const Game& game, const SgfNode& node,
+              unsigned moveNumber, ColorMove mv, bool markVariations,
+              bool markWithDot)
 {
-    if (! mv.is_regular())
+    if (mv.is_null())
         return;
     auto& bd = game.get_board();
     Point p = bd.get_move_info_ext_2(mv.move).label_pos;
+    if (markWithDot)
+    {
+        if (markVariations && ! is_main_variation(game.get_current()))
+            guiBoard.setMark(p, BoardPainter::circle);
+        else
+            guiBoard.setMark(p, BoardPainter::dot);
+        return;
+    }
     QString label;
     label.setNum(moveNumber);
     if (markVariations)
@@ -116,44 +125,28 @@ void setMoveLabel(GuiBoard& guiBoard, const Game& game, const SgfNode& node,
 //-----------------------------------------------------------------------------
 
 void setMarkup(GuiBoard& guiBoard, const Game& game, unsigned markMovesBegin,
-               unsigned markMovesEnd, bool markVariations)
+               unsigned markMovesEnd, bool markVariations, bool markWithDot)
 {
     guiBoard.clearMarkup();
-    if (markMovesBegin > 0)
-    {
+    if (markMovesBegin == 0)
+        return;
         auto& tree = game.get_tree();
         auto& bd = game.get_board();
-        unsigned displayedMoveNumber = 0; // pass moves have no number
-        auto node = &game.get_current();
-        do
-        {
-            auto mv = tree.get_move_ignore_invalid(*node);
-            if (mv.is_regular())
-                ++displayedMoveNumber;
-            node = node->get_parent_or_null();
-        }
-        while (node != nullptr);
         unsigned moveNumber = bd.get_nu_moves();
-        node = &game.get_current();
+        auto node = &game.get_current();
         do
         {
             auto mv = tree.get_move_ignore_invalid(*node);
             if (! mv.is_null())
             {
-                if (! mv.move.is_pass())
-                {
-                    if (moveNumber >= markMovesBegin
-                        && moveNumber <= markMovesEnd)
-                        setMoveLabel(guiBoard, game, *node, moveNumber, mv,
-                                     markVariations);
-                    --displayedMoveNumber;
-                }
+                if (moveNumber >= markMovesBegin && moveNumber <= markMovesEnd)
+                    markMove(guiBoard, game, *node, moveNumber, mv,
+                             markVariations, markWithDot);
                 --moveNumber;
             }
             node = node->get_parent_or_null();
         }
-        while (node != nullptr);
-    }
+        while (node);
 }
 
 //-----------------------------------------------------------------------------
