@@ -12,6 +12,7 @@
 
 #include <QAction>
 #include <QFile>
+#include <QLocale>
 #include <QSettings>
 #include <QTextBrowser>
 #include <QToolBar>
@@ -33,11 +34,12 @@ void setIcon(QAction* action, const QString& name)
 
 //-----------------------------------------------------------------------------
 
-HelpWindow::HelpWindow(QWidget* parent, const QString& mainPage)
+HelpWindow::HelpWindow(QWidget* parent, const QString& title,
+                       const QString& mainPage)
     : QMainWindow(parent)
 {
     log("Loading ", mainPage.toLocal8Bit().constData());
-    setWindowTitle(tr("Pentobi User Manual"));
+    setWindowTitle(title);
     if (QIcon::hasThemeIcon("help-browser"))
         setWindowIcon(QIcon::fromTheme("help-browser"));
     m_mainPageUrl = QUrl::fromLocalFile(mainPage);
@@ -82,17 +84,18 @@ HelpWindow::HelpWindow(QWidget* parent, const QString& mainPage)
         adjustSize();
 }
 
-QString HelpWindow::findMainPage(QString dir, QString file, QString locale)
+QString HelpWindow::findMainPage(QString helpDir, QString appName)
 {
-    QString path;
-    path = dir + "/" + locale + "/" + file;
+    auto locale = QLocale::system().name();
+    auto path = QString("%1/%2/%3/index.html")
+            .arg(helpDir).arg(locale).arg(appName);
     if (QFile(path).exists())
         return path;
-    QStringList list = locale.split("_");
-    path = dir + "/" + list[0] + "/" + file;
+    path = QString("%1/%2/%3/index.html")
+            .arg(helpDir).arg(locale.split("_")[0]).arg(appName);
     if (QFile(path).exists())
         return path;
-    return dir + "/en/" + file;
+    return QString("%1/C/%3/index.html").arg(helpDir).arg(appName);
 }
 
 void HelpWindow::closeEvent(QCloseEvent* event)
