@@ -27,10 +27,7 @@ using libpentobi_base::PiecePoints;
 
 OrientationDisplay::OrientationDisplay(QWidget* parent, const Board& bd)
     : QWidget(parent),
-      m_bd(bd),
-      m_piece(Piece::null()),
-      m_transform(0),
-      m_isColorSelected(false)
+      m_bd(bd)
 {
     setMinimumWidth(30);
     setMinimumHeight(30);
@@ -45,7 +42,7 @@ void OrientationDisplay::clearSelectedColor()
     }
 }
 
-void OrientationDisplay::clearSelectedPiece()
+void OrientationDisplay::clearPiece()
 {
     if (m_piece.is_null())
         return;
