@@ -74,17 +74,14 @@ void PieceSelector::checkUpdate()
 {
     bool disabledStatus[maxColumns][maxRows];
     setDisabledStatus(disabledStatus);
-    bool changed = false;
     for (unsigned x = 0; x < m_nuColumns; ++x)
         for (unsigned y = 0; y < m_nuRows; ++y)
             if (! m_piece[x][y].is_null()
-                && disabledStatus[x][y] != m_disabledStatus[x][y])
+                    && disabledStatus[x][y] != m_disabledStatus[x][y])
             {
-                changed = true;
-                break;
+                update();
+                return;
             }
-    if (changed)
-        update();
 }
 
 void PieceSelector::findPiecePoints(Piece piece, unsigned x, unsigned y,
@@ -96,8 +93,8 @@ void PieceSelector::findPiecePoints(Piece piece, unsigned x, unsigned y,
         return;
     points.push_back(p);
     // This assumes that no Trigon pieces touch at the corners, otherwise
-    // we would need to iterate over neighboring CoordPoint's like AdjIterator
-    // iterates over neighboring Point's
+    // we would need to iterate over neighboring CoordPoint's corresponding to
+    // Geometry::get_adj()
     findPiecePoints(piece, x + 1, y, points);
     findPiecePoints(piece, x - 1, y, points);
     findPiecePoints(piece, x, y + 1, points);
@@ -135,6 +132,8 @@ void PieceSelector::init()
         m_nuColumns = 33;
         m_nuRows = 6;
     }
+    LIBBOARDGAME_ASSERT(m_nuColumns <= maxColumns);
+    LIBBOARDGAME_ASSERT(m_nuRows <= maxRows);
     for (unsigned y = 0; y < m_nuRows; ++y)
         for (unsigned x = 0; x < m_nuColumns; ++x)
         {
@@ -160,7 +159,7 @@ void PieceSelector::init()
             type_match_shift(geo, points.begin(), points.end(), 0);
             m_transform[x][y] =
                 m_bd.get_piece_info(piece).find_transform(geo, points);
-            LIBBOARDGAME_ASSERT(m_transform[x][y] != 0);
+            LIBBOARDGAME_ASSERT(m_transform[x][y]);
         }
     setDisabledStatus(m_disabledStatus);
     update();
