@@ -20,7 +20,6 @@ using libboardgame_sgf::SgfNode;
 using libboardgame_util::log;
 using libboardgame_util::clear_abort;
 using libboardgame_util::get_abort;
-using libboardgame_util::Exception;
 using libboardgame_util::WallTimeSource;
 using libpentobi_base::BoardUpdater;
 
@@ -39,7 +38,7 @@ void AnalyzeGame::run(const Game& game, Search& search, size_t nu_simulations,
     auto& root = game.get_root();
     auto node = &root;
     unsigned total_moves = 0;
-    while (node != nullptr)
+    while (node)
     {
         if (tree.has_move(*node))
             ++total_moves;
@@ -49,10 +48,10 @@ void AnalyzeGame::run(const Game& game, Search& search, size_t nu_simulations,
     clear_abort();
     node = &root;
     unsigned move_number = 0;
-    while (node != nullptr)
+    while (node)
     {
         auto mv = tree.get_move(*node);
-        if (mv.is_regular())
+        if (! mv.is_null())
         {
             if (! node->has_parent())
             {
@@ -81,8 +80,7 @@ void AnalyzeGame::run(const Game& game, Search& search, size_t nu_simulations,
                                   min_simulations, max_time, time_source);
                     if (get_abort())
                         break;
-                    auto& search_root = search.get_tree().get_root();
-                    if (search_root.get_visit_count() == 0)
+                    if (search.get_root_visit_count() == 0)
                     {
                         m_moves.push_back(mv);
                         m_has_value.push_back(false);
@@ -92,10 +90,10 @@ void AnalyzeGame::run(const Game& game, Search& search, size_t nu_simulations,
                     {
                         m_moves.push_back(mv);
                         m_has_value.push_back(true);
-                        m_values.push_back(search_root.get_value());
+                        m_values.push_back(search.get_root_val().get_mean());
                     }
                 }
-                catch (const Exception&)
+                catch (const runtime_error&)
                 {
                     // BoardUpdater::update() can throw on invalid SGF tree
                     // read from external file. We simply abort the analysis.
