@@ -17,7 +17,6 @@
 
 namespace libpentobi_mcts {
 
-using namespace std;
 using libboardgame_util::log;
 using libboardgame_util::CpuTimeSource;
 using libboardgame_util::WallTimeSource;
@@ -25,19 +24,19 @@ using libpentobi_base::BoardType;
 
 //-----------------------------------------------------------------------------
 
-Player::Player(Variant initial_variant, const string& books_dir,
-               unsigned nu_threads, size_t memory)
+Player::Player(Variant initial_variant, string  books_dir, unsigned nu_threads,
+               size_t memory)
     : m_is_book_loaded(false),
       m_use_book(true),
       m_resign(false),
-      m_books_dir(books_dir),
+      m_books_dir(move(books_dir)),
       m_level(4),
       m_fixed_simulations(0),
       m_resign_threshold(0.09f),
       m_resign_min_simulations(500),
       m_search(initial_variant, nu_threads, memory),
       m_book(initial_variant),
-      m_time_source(new WallTimeSource())
+      m_time_source(new WallTimeSource)
 {
     for (unsigned i = 0; i < Board::max_player_moves; ++i)
     {
@@ -82,7 +81,7 @@ Move Player::genmove(const Board& bd, Color c)
 {
     m_resign = false;
     if (! bd.has_moves(c))
-        return Move::pass();
+        return Move::null();
     Move mv;
     auto variant = bd.get_variant();
     auto board_type = bd.get_board_type();
@@ -109,7 +108,7 @@ Move Player::genmove(const Board& bd, Color c)
         max_time = m_fixed_time;
     else
     {
-        // Rationale for chosing max_count:
+        // Rationale for choosing max_count:
         // * The number at level 1 is very small in Classic/Duo to avoid that
         //   level 1 is too strong for absolute beginners (searches with such a
         //   small number of simulations still produce reasonable moves because
@@ -117,8 +116,7 @@ Move Player::genmove(const Board& bd, Color c)
         //   with a higher number because the playing strength is weaker there.
         // * The number at the highest level is chosen such that the average
         //   time per game and player is 2 min in Duo, 4 min in Classic, 5 min
-        //   in Trigon on a Intel i3-4130. (This takes into account the
-        //   weighting of max_count with the move number below.)
+        //   in Trigon on an Intel i3-4130.
         // * The numbers for other levels are chosen such that they roughly
         //   correspond to equal Elo differences in self-play experiments.
         switch (variant)
@@ -128,7 +126,7 @@ Move Player::genmove(const Board& bd, Color c)
         case Variant::classic_3:
             {
                 static float counts[] =
-                    { 3, 24, 87, 213, 667, 1989, 10780, 98438, 1321164 };
+                    { 3, 24, 87, 213, 667, 1989, 11319, 101391, 1339615 };
                 max_count = counts[level - 1];
             }
             break;
@@ -136,7 +134,7 @@ Move Player::genmove(const Board& bd, Color c)
         case Variant::junior:
             {
                 static float counts[] =
-                    { 3, 17, 44, 123, 426, 1672, 6240, 51590, 5093330 };
+                    { 3, 17, 44, 123, 426, 1672, 6739, 54169, 5476857 };
                 max_count = counts[level - 1];
             }
             break;
@@ -145,7 +143,7 @@ Move Player::genmove(const Board& bd, Color c)
         case Variant::trigon_3:
             {
                 static float counts[] =
-                    { 228, 433, 727, 1501, 2912, 7395, 20828, 61138, 370066 };
+                    { 228, 433, 727, 1501, 2912, 7395, 22494, 66029, 412757 };
                 max_count = counts[level - 1];
             }
             break;
@@ -182,12 +180,9 @@ Move Player::genmove(const Board& bd, Color c)
         return Move::null();
     // Resign only in two-player game variants
     if (get_nu_players(variant) == 2)
-    {
-        auto& root = m_search.get_tree().get_root();
-        if (root.get_visit_count() > m_resign_min_simulations
-            && root.get_value() < m_resign_threshold)
+        if (m_search.get_root_visit_count() > m_resign_min_simulations
+                && m_search.get_root_val().get_mean() < m_resign_threshold)
             m_resign = true;
-    }
     return mv;
 }
 
@@ -244,7 +239,7 @@ Rating Player::get_rating(Variant variant, int level)
 
 bool Player::is_book_loaded(Variant variant) const
 {
-    return (m_is_book_loaded && m_book.get_tree().get_variant() == variant);
+    return m_is_book_loaded && m_book.get_tree().get_variant() == variant;
 }
 
 void Player::load_book(istream& in)
@@ -275,9 +270,9 @@ bool Player::resign() const
 void Player::use_cpu_time(bool enable)
 {
     if (enable)
-        m_time_source.reset(new CpuTimeSource());
+        m_time_source.reset(new CpuTimeSource);
     else
-        m_time_source.reset(new WallTimeSource());
+        m_time_source.reset(new WallTimeSource);
 }
 
 //-----------------------------------------------------------------------------
