@@ -12,26 +12,25 @@
 
 #include <cmath>
 #include "libboardgame_util/MathUtil.h"
-#include "libpentobi_base/AdjIterator.h"
 
 namespace libpentobi_mcts {
 
-using namespace std;
 using libboardgame_util::fast_exp;
-using libpentobi_base::AdjIterator;
 using libpentobi_base::BoardType;
 using libpentobi_base::Color;
-using libpentobi_base::ColorIterator;
 using libpentobi_base::ColorMove;
-using libpentobi_base::GeometryIterator;
 using libpentobi_base::Point;
 using libpentobi_base::PointState;
 using libpentobi_base::Variant;
 
 //-----------------------------------------------------------------------------
 
+PriorKnowledge::PriorKnowledge()
+{
+    m_is_local.fill_all(false);
+}
+
 void PriorKnowledge::compute_features(const Board& bd, const MoveList& moves,
-                                      const LocalValue& local_value,
                                       bool check_dist_to_center,
                                       bool check_connect)
 {
@@ -62,43 +61,49 @@ void PriorKnowledge::compute_features(const Board& bd, const MoveList& moves,
     Grid<Float> point_value;
     Grid<Float> attach_point_value;
     Grid<Float> adj_point_value;
-    for (GeometryIterator i(geo); i; ++i)
+    for (Point p : geo)
     {
-        point_value[*i] = 1;
-        auto s = bd.get_point_state(*i);
-        if (is_forbidden[*i] && s != to_play)
-            attach_point_value[*i] = -2.5;
+        point_value[p] = 1;
+        auto s = bd.get_point_state(p);
+        if (is_forbidden[p] && s != to_play)
+            attach_point_value[p] = -2.5;
         else
-            attach_point_value[*i] = 0.5;
-        if (! is_forbidden[*i])
+            attach_point_value[p] = 0.5;
+        if (! is_forbidden[p])
         {
-            if (bd.is_attach_point(*i, to_play))
+            if (bd.is_attach_point(p, to_play))
                 // Making own attach point forbidden is especially bad
-                adj_point_value[*i] = -1;
+                adj_point_value[p] = -1;
             else
                 // Creating new forbidden points is a bad thing
-                adj_point_value[*i] = -0.1f;
+                adj_point_value[p] = -0.1f;
         }
         else if (s == connect_color)
-            adj_point_value[*i] = 1;
+            // Connecting own colors is good
+            adj_point_value[p] = 1;
+        else if (! s.is_empty())
+            // Touching opponent is better than playing elsewhere (no need to
+            // check if s == to_play, such moves are illegal)
+            adj_point_value[p] = 0.4f;
         else
-            adj_point_value[*i] = 0;
+            adj_point_value[p] = 0;
     }
-    for (ColorIterator i(bd.get_nu_colors()); i; ++i)
+    for (Color c : bd.get_colors())
     {
-        if (*i == to_play || *i == second_color)
+        if (c == to_play || c == second_color)
             continue;
-        auto& is_forbidden = bd.is_forbidden(*i);
-        for (Point p : bd.get_attach_points(*i))
+        auto& is_forbidden = bd.is_forbidden(c);
+        for (Point p : bd.get_attach_points(c))
             if (! is_forbidden[p])
             {
                 // Occupying opponent attach points is very good
                 point_value[p] = 3.2f;
-                for (AdjIterator j(geo, p); j; ++j)
-                    if (! is_forbidden[*j])
+                geo.for_each_adj(p, [&](Point j) {
+                    if (! is_forbidden[j])
                         // Occupying points adjacent to opponent attach points
                         // is good
-                        point_value[*j] = max(point_value[*j], Float(2.5));
+                        point_value[j] = max(point_value[j], Float(2.5));
+                });
             }
     }
     if (variant == Variant::classic_2
@@ -116,7 +121,7 @@ void PriorKnowledge::compute_features(const Board& bd, const MoveList& moves,
             }
     }
     m_max_heuristic = -numeric_limits<Float>::max();
-    m_min_dist_to_center = numeric_limits<unsigned>::max();
+    m_min_dist_to_center = numeric_limits<unsigned short>::max();
     m_has_connect_move = false;
     for (unsigned i = 0; i < moves.size(); ++i)
     {
@@ -127,12 +132,12 @@ void PriorKnowledge::compute_features(const Board& bd, const MoveList& moves,
         auto j = info.begin();
         auto end = info.end();
         Float heuristic = point_value[*j];
-        LocalValue::Compute local(*j, local_value);
+        bool local = m_is_local[*j];
         if (! check_dist_to_center)
             while (++j != end)
             {
                 heuristic += point_value[*j];
-                local.add_move_point(*j, local_value);
+                local |= m_is_local[*j];
             }
         else
         {
@@ -140,7 +145,7 @@ void PriorKnowledge::compute_features(const Board& bd, const MoveList& moves,
             while (++j != end)
             {
                 heuristic += point_value[*j];
-                local.add_move_point(*j, local_value);
+                local |= m_is_local[*j];
                 features.dist_to_center =
                     min(features.dist_to_center, m_dist_to_center[*j]);
             }
@@ -173,20 +178,23 @@ void PriorKnowledge::compute_features(const Board& bd, const MoveList& moves,
         if (heuristic > m_max_heuristic)
             m_max_heuristic = heuristic;
         features.heuristic = heuristic;
-        features.local_value = local;
+        features.is_local = local;
     }
 }
 
 void PriorKnowledge::gen_children(const Board& bd, const MoveList& moves,
                                   bool is_symmetry_broken,
-                                  const LocalValue& local_value,
                                   Tree::NodeExpander& expander, Float init_val)
 {
     if (moves.empty())
     {
-        expander.add_child(Move::pass(), 0.5, 0);
+        // Add a pass move. The initialization value does not matter for a
+        // single child, but we need to use SearchParamConst::child_min_count
+        // for the count to avoid an assertion.
+        expander.add_child(Move::null(), init_val, 3);
         return;
     }
+    init_local(bd);
     auto to_play = bd.get_to_play();
     auto nu_onboard_pieces = bd.get_nu_onboard_pieces();
     bool check_dist_to_center =
@@ -194,8 +202,7 @@ void PriorKnowledge::gen_children(const Board& bd, const MoveList& moves,
              && nu_onboard_pieces <= m_dist_to_center_max_pieces);
     bool check_connect =
         (bd.get_variant() == Variant::classic_2 && nu_onboard_pieces < 14);
-    compute_features(bd, moves, local_value, check_dist_to_center,
-                     check_connect);
+    compute_features(bd, moves, check_dist_to_center, check_connect);
     if (! m_has_connect_move)
         check_connect = false;
     Move symmetric_mv = Move::null();
@@ -208,8 +215,7 @@ void PriorKnowledge::gen_children(const Board& bd, const MoveList& moves,
             if (nu_moves > 0)
             {
                 ColorMove last = bd.get_move(nu_moves - 1);
-                if (! last.is_pass())
-                    symmetric_mv =
+                symmetric_mv =
                         bd.get_move_info_ext_2(last.move).symmetric_move;
             }
         }
@@ -221,6 +227,10 @@ void PriorKnowledge::gen_children(const Board& bd, const MoveList& moves,
                     break;
                 }
     }
+    auto board_type = bd.get_board_type();
+    bool is_trigon = (board_type == BoardType::trigon
+                      || board_type == BoardType::trigon_3);
+    float max_dist_diff = (is_trigon ? 0.5f : 0.3f);
     for (unsigned i = 0; i < moves.size(); ++i)
     {
         const auto& features = m_features[i];
@@ -229,8 +239,8 @@ void PriorKnowledge::gen_children(const Board& bd, const MoveList& moves,
         // center and moves that don't connect in the middle if connection is
         // possible
         if ((check_dist_to_center
-             && features.dist_to_center != m_min_dist_to_center)
-            || (check_connect && ! features.connect))
+             && features.dist_to_center > m_min_dist_to_center + max_dist_diff)
+                || (check_connect && ! features.connect))
             continue;
 
         auto mv = moves[i];
@@ -246,7 +256,8 @@ void PriorKnowledge::gen_children(const Board& bd, const MoveList& moves,
         heuristic = 0.1f + 0.9f * fast_exp(-0.6f * heuristic);
 
         // Initialize value from heuristic and init_val, each with a count
-        // of 1.5
+        // of 1.5. If this is changed, SearchParamConst::child_min_count
+        // should be updated.
         Float value = 1.5f * heuristic + 1.5f * init_val;
         Float count = 3;
 
@@ -266,7 +277,7 @@ void PriorKnowledge::gen_children(const Board& bd, const MoveList& moves,
 
         // Add 1 win for moves that are local responses to recent opponent
         // moves
-        if (features.local_value.has_local())
+        if (features.is_local)
         {
             value += 1;
             count += 1;
@@ -276,6 +287,45 @@ void PriorKnowledge::gen_children(const Board& bd, const MoveList& moves,
     }
 }
 
+void PriorKnowledge::init_local(const Board& bd)
+{
+    for (Point p : m_local_points)
+        m_is_local[p] = false;
+    unsigned nu_local = 0;
+    Color to_play = bd.get_to_play();
+    Color second_color;
+    if (bd.get_variant() == Variant::classic_3 && to_play.to_int() == 3)
+        second_color = Color(bd.get_alt_player());
+    else
+        second_color = bd.get_second_color(to_play);
+    auto& moves = bd.get_moves();
+    // Consider last 3 moves for local points (i.e. last 2 opponent moves in
+    // two-color variants)
+    auto end = moves.end();
+    auto begin = (end - moves.begin() < 3 ? moves.begin() : end - 3);
+    for (auto i = begin; i != end; ++i)
+    {
+        Color c = i->color;
+        if (c == to_play || c == second_color)
+            continue;
+        auto mv = i->move;
+        auto& is_forbidden = bd.is_forbidden(c);
+        auto& info_ext = bd.get_move_info_ext(mv);
+        auto j = info_ext.begin_attach();
+        auto end = info_ext.end_attach();
+        do
+        {
+            if (is_forbidden[*j])
+                continue;
+            if (! m_is_local[*j])
+                m_local_points.get_unchecked(nu_local++) = *j;
+            m_is_local[*j] = true;
+        }
+        while (++j != end);
+    }
+    m_local_points.resize(nu_local);
+}
+
 void PriorKnowledge::start_search(const Board& bd)
 {
     auto& geo = bd.get_geometry();
@@ -289,20 +339,18 @@ void PriorKnowledge::start_search(const Board& bd)
     bool is_trigon = (board_type == BoardType::trigon
                       || board_type == BoardType::trigon_3);
     float ratio = (is_trigon ? 1.732f : 1);
-    for (GeometryIterator i(geo); i; ++i)
+    for (Point p : geo)
     {
-        float x = static_cast<float>(i->get_x(geo.get_width()));
-        float y = static_cast<float>(i->get_y(geo.get_width()));
+        float x = static_cast<float>(geo.get_x(p));
+        float y = static_cast<float>(geo.get_y(p));
         float dx = x - center_x;
         float dy = ratio * (y - center_y);
-        // Multiply Euklidian distance by 4, so that distances that differ
-        // by max. 0.25 are treated as equal
-        float d = round(4 * sqrt(dx * dx + dy * dy));
+        float d = sqrt(dx * dx + dy * dy);
         if (board_type == BoardType::classic)
             // Don't make a distinction between moves close enough to the
             // center in game variant Classic/Classic2
-            d = max(d, 10.f);
-        m_dist_to_center[*i] = static_cast<unsigned>(d);
+            d = max(d, 2.f);
+        m_dist_to_center[p] = d;
     }
 
     // Init m_check_dist_to_center
@@ -321,7 +369,7 @@ void PriorKnowledge::start_search(const Board& bd)
     case Variant::trigon_2:
     case Variant::trigon_3:
         m_check_dist_to_center.fill(true);
-        m_dist_to_center_max_pieces = 4;
+        m_dist_to_center_max_pieces = 3;
         break;
     default:
         m_check_dist_to_center.fill(false);
@@ -331,19 +379,19 @@ void PriorKnowledge::start_search(const Board& bd)
     // search might not generate any moves (if no moves meet the dist-to-center
     // condition). Even if such positions cannot occur in legal games, we still
     // don't want the move generation to fail.
-    for (ColorIterator i(bd.get_nu_colors()); i; ++i)
+    for (Color c : bd.get_colors())
     {
-        if (bd.get_nu_onboard_pieces(*i) == 0)
+        if (bd.get_nu_onboard_pieces(c) == 0)
             continue;
         bool is_starting_point_covered = false;
-        for (Point p : bd.get_starting_points(*i))
-            if (bd.get_point_state(p) == PointState(*i))
+        for (Point p : bd.get_starting_points(c))
+            if (bd.get_point_state(p) == PointState(c))
             {
                 is_starting_point_covered = true;
                 break;
             }
         if (! is_starting_point_covered)
-            m_check_dist_to_center[*i] = false;
+            m_check_dist_to_center[c] = false;
     }
 }
 
