@@ -8,10 +8,9 @@
 #define LIBPENTOBI_MCTS_PRIOR_KNOWLEDGE_H
 
 #include "Float.h"
-#include "LocalValue.h"
+#include "SearchParamConst.h"
 #include "libboardgame_mcts/Tree.h"
 #include "libpentobi_base/Board.h"
-#include "libpentobi_base/MoveList.h"
 
 namespace libpentobi_mcts {
 
@@ -21,6 +20,7 @@ using libpentobi_base::ColorMap;
 using libpentobi_base::Grid;
 using libpentobi_base::Move;
 using libpentobi_base::MoveList;
+using libpentobi_base::PointList;
 
 //-----------------------------------------------------------------------------
 
@@ -28,16 +28,19 @@ using libpentobi_base::MoveList;
 class PriorKnowledge
 {
 public:
-    typedef libboardgame_mcts::Node<Move, Float> Node;
+    typedef libboardgame_mcts::Node<Move, Float, SearchParamConst::multithread>
+    Node;
 
     typedef libboardgame_mcts::Tree<Node> Tree;
 
+    PriorKnowledge();
+
     void start_search(const Board& bd);
 
     /** Generate children nodes initialized with prior knowledge. */
     void gen_children(const Board& bd, const MoveList& moves,
-                      bool is_symmetry_broken, const LocalValue& local_value,
-                      Tree::NodeExpander& expander, Float init_val);
+                      bool is_symmetry_broken, Tree::NodeExpander& expander,
+                      Float init_val);
 
 private:
     struct MoveFeatures
@@ -45,13 +48,13 @@ private:
         /** Heuristic value of the move expressed in score points. */
         Float heuristic;
 
-        LocalValue::Compute local_value;
-
-        /** Only used on Classic and Trigon boards. */
-        unsigned dist_to_center;
+        bool is_local;
 
         /** Does the move touch a piece of the same player? */
         bool connect;
+
+        /** Only used on Classic and Trigon boards. */
+        float dist_to_center;
     };
 
     array<MoveFeatures, Move::range> m_features;
@@ -65,14 +68,21 @@ private:
 
     unsigned m_dist_to_center_max_pieces;
 
-    unsigned m_min_dist_to_center;
+    float m_min_dist_to_center;
+
+    /** Marker for attach points of recent opponent moves. */
+    Grid<bool> m_is_local;
+
+    /** Points in m_is_local with value greater zero. */
+    PointList m_local_points;
 
     /** Distance to center heuristic. */
-    Grid<unsigned> m_dist_to_center;
+    Grid<float> m_dist_to_center;
 
     void compute_features(const Board& bd, const MoveList& moves,
-                          const LocalValue& local_value,
                           bool check_dist_to_center, bool check_connect);
+
+    void init_local(const Board& bd);
 };
 
 //-----------------------------------------------------------------------------
