@@ -7,46 +7,20 @@
 #ifndef LIBPENTOBI_MCTS_SEARCH_H
 #define LIBPENTOBI_MCTS_SEARCH_H
 
+#include "History.h"
+#include "SearchParamConst.h"
 #include "State.h"
 #include "libboardgame_mcts/SearchBase.h"
-#include "libpentobi_base/GameStateHistory.h"
 
 namespace libpentobi_mcts {
 
 using namespace std;
 using libboardgame_mcts::PlayerInt;
-using libboardgame_mcts::SearchBase;
-using libboardgame_util::Timer;
 using libboardgame_util::TimeSource;
-using libpentobi_base::GameStateHistory;
 using libpentobi_base::Setup;
 
 //-----------------------------------------------------------------------------
 
-/** Optional compile-time parameters for libboardgame_mcts::Search.
-    See libboardgame_mcts::SearchParamConstDefault for the meaning of the
-    members. */
-struct SearchParamConst
-{
-    typedef libpentobi_mcts::Float Float;
-
-    static const PlayerInt max_players = 6;
-
-    static const unsigned max_moves = Board::max_game_moves;
-
-    static const bool rave = true;
-
-    static const bool rave_check_same = false;
-
-    static const bool rave_dist_weighting = true;
-
-    static const bool use_last_good_reply = true;
-
-    static const bool virtual_loss = true;
-};
-
-//-----------------------------------------------------------------------------
-
 /** Monte-Carlo tree search implementation for Blokus.
     Multiple colors per player (e.g. in Classic 2) are handled by using the
     same game result for each color of a player.
@@ -56,8 +30,8 @@ struct SearchParamConst
     The maximum number of players is 6, which occurs in Classic 3 with 3
     real players and 3 pseudo-players for the 4th color.
     @note @ref libboardgame_avoid_stack_allocation */
-class Search
-    : public SearchBase<State, Move, SearchParamConst>
+class Search final
+    : public libboardgame_mcts::SearchBase<State, Move, SearchParamConst>
 {
 public:
     Search(Variant initial_variant, unsigned nu_threads, size_t memory);
@@ -102,7 +76,7 @@ public:
     /** Get color to play at root node of the last search. */
     Color get_to_play() const;
 
-    const GameStateHistory& get_last_state() const;
+    const History& get_last_history() const;
 
     /** Get board position of last search at root node as setup.
         @param[out] variant
@@ -125,9 +99,9 @@ private:
     SharedConst m_shared_const;
 
     /** Local variable reused for efficiency. */
-    GameStateHistory m_state;
+    History m_history;
 
-    GameStateHistory m_last_state;
+    History m_last_history;
 
     const Board& get_board() const;
 
@@ -149,9 +123,9 @@ inline const Board& Search::get_board() const
     return *m_shared_const.board;
 }
 
-inline const GameStateHistory& Search::get_last_state() const
+inline const History& Search::get_last_history() const
 {
-    return m_last_state;
+    return m_last_history;
 }
 
 inline PlayerInt Search::get_nu_players() const
