@@ -16,11 +16,9 @@
 
 namespace libpentobi_mcts {
 
-using namespace std;
 using libboardgame_util::fast_exp;
 using libpentobi_base::get_multiplayer_result;
 using libpentobi_base::BoardType;
-using libpentobi_base::ColorIterator;
 using libpentobi_base::PointState;
 
 //-----------------------------------------------------------------------------
@@ -36,62 +34,45 @@ inline Float sigmoid(Float steepness, Float x)
 
 //-----------------------------------------------------------------------------
 
-SharedConst::SharedConst(const Color& to_play)
-    : board(nullptr),
-      to_play(to_play),
-      avoid_symmetric_draw(true)
-{
-    // Game variant and position dependent variables are initialized in
-    // libpentobi_mcts::Search::start_search()
-}
-
-//-----------------------------------------------------------------------------
-
 State::State(Variant initial_variant, const SharedConst& shared_const)
   : m_shared_const(shared_const),
     m_bd(initial_variant)
 {
 }
 
-inline void State::add_move(MoveList& moves, Move mv, double gamma)
-{
-    m_total_gamma += gamma;
-    m_cumulative_gamma[moves.size()] = m_total_gamma;
-    moves.push_back(mv);
-}
-
 inline void State::add_moves(Point p, Color c,
                              const Board::PiecesLeftList& pieces_considered)
 {
     auto& moves = m_moves[c];
+    auto nu_moves = moves.size();
     auto& marker = m_marker[c];
+    auto& playout_features = m_playout_features[c];
     auto adj_status = m_bd.get_adj_status(p, c);
-    auto& is_forbidden = m_bd.is_forbidden(c);
-    double gamma;
+    double total_gamma =
+            moves.empty() ? 0 : m_cumulative_gamma[moves.size() - 1];
     for (Piece piece : pieces_considered)
         for (Move mv : get_moves(c, piece, p, adj_status))
             if (! marker[mv]
-                    && check_move(is_forbidden, get_move_info(mv), gamma))
-            {
+                    && check_move(mv, get_move_info(mv), moves, nu_moves,
+                                  playout_features, total_gamma))
                 marker.set(mv);
-                add_move(moves, mv, gamma);
-            }
-    m_moves_added_at[c].set(p);
+    moves.resize(nu_moves);
+    m_moves_added_at[c][p] = true;
 }
 
 inline void State::add_moves(Point p, Color c, Piece piece,
-                             unsigned adj_status)
+                             unsigned adj_status, double& total_gamma)
 {
     auto& moves = m_moves[c];
+    auto nu_moves = moves.size();
     auto& marker = m_marker[c];
-    auto& is_forbidden = m_bd.is_forbidden(c);
-    double gamma;
+    auto& playout_features = m_playout_features[c];
     for (Move mv : get_moves(c, piece, p, adj_status))
-        if (! marker[mv] && check_move(is_forbidden, get_move_info(mv), gamma))
-        {
+        if (! marker[mv]
+                && check_move(mv, get_move_info(mv), moves, nu_moves,
+                              playout_features, total_gamma))
             marker.set(mv);
-            add_move(moves, mv, gamma);
-        }
+    moves.resize(nu_moves);
 }
 
 void State::add_starting_moves(Color c,
@@ -106,61 +87,75 @@ void State::add_starting_moves(Color c,
     if (p.is_null())
         return;
     auto& moves = m_moves[c];
+    LIBBOARDGAME_ASSERT(moves.empty());
+    unsigned nu_moves = 0;
     auto& marker = m_marker[c];
     auto& is_forbidden = m_bd.is_forbidden(c);
+    double total_gamma = 0;
     for (Piece piece : pieces_considered)
-    {
         for (Move mv : get_moves(c, piece, p, 0))
         {
             LIBBOARDGAME_ASSERT(! marker[mv]);
-            if (check_move_without_gamma(is_forbidden, mv))
+            if (check_forbidden(is_forbidden, mv, moves, nu_moves))
             {
                 marker.set(mv);
                 if (with_gamma)
-                    add_move(moves, mv, m_gamma_piece[piece]);
-                else
-                    moves.push_back(mv);
+                {
+                    total_gamma += m_gamma_piece[piece];
+                    m_cumulative_gamma[nu_moves - 1] = total_gamma;
+                }
             }
         }
-    }
+    moves.resize(nu_moves);
 }
 
-/** Check if move is not forbidden and compute/handle its local value in the
-    same loop. */
-bool State::check_move(const Grid<bool>& is_forbidden, const MoveInfo& info,
-                       double& gamma)
+bool State::check_forbidden(const Grid<bool>& is_forbidden, Move mv,
+                            MoveList& moves, unsigned& nu_moves)
 {
-    auto i = info.begin();
-    if (is_forbidden[*i])
+    auto& info = get_move_info(mv);
+    auto p = info.begin();
+    if (is_forbidden[*p])
         return false;
-    LocalValue::Compute local(*i, m_local_value);
     auto end = info.end();
-    while (++i != end)
+    for (unsigned i = 1; i < PieceInfo::max_size; ++i)
     {
-        if (is_forbidden[*i])
+        if (++p == end)
+            break;
+        if (is_forbidden[*p])
             return false;
-        local.add_move_point(*i, m_local_value);
-    }
-    gamma = m_gamma_piece[info.get_piece()];
-    if (local.has_local())
-    {
-        gamma *= m_gamma_nu_attach[local.get_nu_attach()];
-        if (local.has_adj_attach())
-            gamma *= 1e5;
     }
+    LIBBOARDGAME_ASSERT(nu_moves < MoveList::max_size);
+    moves.get_unchecked(nu_moves) = mv;
+    ++nu_moves;
     return true;
 }
 
-bool State::check_move_without_gamma(const Grid<bool>& is_forbidden, Move mv)
+bool State::check_move(Move mv, const MoveInfo& info, MoveList& moves,
+                       unsigned& nu_moves,
+                       const PlayoutFeatures& playout_features,
+                       double& total_gamma)
 {
-    auto& info = get_move_info(mv);
-    auto i = info.begin();
-    if (is_forbidden[*i])
+    auto p = info.begin();
+    PlayoutFeatures::Compute features(*p, playout_features);
+    if (features.is_forbidden())
         return false;
     auto end = info.end();
-    while (++i != end)
-        if (is_forbidden[*i])
+    for (unsigned i = 1; i < PieceInfo::max_size; ++i)
+    {
+        if (++p == end)
+            break;
+        if (! features.add(*p, playout_features))
             return false;
+    }
+    double gamma = m_gamma_piece[info.get_piece()];
+    auto nu_local = features.get_nu_local();
+    if (nu_local > 0)
+        gamma *= m_gamma_nu_attach[nu_local];
+    total_gamma += gamma;
+    m_cumulative_gamma[nu_moves] = total_gamma;
+    LIBBOARDGAME_ASSERT(nu_moves < MoveList::max_size);
+    moves.get_unchecked(nu_moves) = mv;
+    ++nu_moves;
     return true;
 }
 
@@ -173,8 +168,7 @@ string State::dump() const
 }
 #endif
 
-/** Evaluation function for game variants with 2 players and 1 color per
-    player. */
+/** Evaluation function for Duo and Junior. */
 void State::evaluate_duo(array<Float, 6>& result)
 {
     LIBBOARDGAME_ASSERT(m_bd.get_nu_players() == 2);
@@ -198,7 +192,7 @@ void State::evaluate_duo(array<Float, 6>& result)
         res = 0.5;
     if (log_simulations)
         log("Result color ", c, ": sco=", s, " game_res=", res);
-    res += get_eval_bonus(c, res, s);
+    res += get_quality_bonus(c, res, s, false);
     if (log_simulations)
         log("res=", res);
     result[0] = res;
@@ -235,7 +229,7 @@ void State::evaluate_multicolor(array<Float, 6>& result)
         res = 0.5;
     if (log_simulations)
         log("Result color ", c, ": sco=", s, " game_res=", res);
-    res += get_eval_bonus(c, res, s);
+    res += get_quality_bonus(c, res, s, true);
     if (log_simulations)
         log("res=", res);
     result[0] = result[2] = res;
@@ -264,7 +258,8 @@ void State::evaluate_multiplayer(array<Float, 6>& result)
     {
         Color c(i);
         auto s = static_cast<Float>(m_bd.get_score(c));
-        result[i] = game_result[i] + get_eval_bonus(c, game_result[i], s);
+        result[i] =
+                game_result[i] + get_quality_bonus(c, game_result[i], s, true);
         if (log_simulations)
             log("Result color ", c, ": sco=", s, " game_res=", game_result[i],
                 " res=", result[i]);
@@ -287,23 +282,22 @@ Point State::find_best_starting_point(Color c) const
     bool is_trigon = (m_bd.get_board_type() == BoardType::trigon
                       || m_bd.get_board_type() == BoardType::trigon_3);
     float ratio = (is_trigon ? 1.732f : 1);
-    auto width = m_bd.get_geometry().get_width();
+    auto& geo = m_bd.get_geometry();
     for (Point p : m_bd.get_starting_points(c))
     {
         if (m_bd.is_forbidden(p, c))
             continue;
-        float px = static_cast<float>(p.get_x(width));
-        float py = static_cast<float>(p.get_y(width));
+        float px = static_cast<float>(geo.get_x(p));
+        float py = static_cast<float>(geo.get_y(p));
         float d = 0;
-        for (ColorIterator i(m_nu_colors); i; ++i)
-        {
-            for (Point pp : m_bd.get_starting_points(*i))
+        for (Color i : Color::Range(m_nu_colors))
+            for (Point pp : m_bd.get_starting_points(i))
             {
                 PointState s = m_bd.get_point_state(pp);
                 if (! s.is_empty())
                 {
-                    float ppx = static_cast<float>(pp.get_x(width));
-                    float ppy = static_cast<float>(pp.get_y(width));
+                    float ppx = static_cast<float>(geo.get_x(pp));
+                    float ppy = static_cast<float>(geo.get_y(pp));
                     float dx = ppx - px;
                     float dy = ratio * (ppy - py);
                     float weight = 1;
@@ -312,7 +306,6 @@ Point State::find_best_starting_point(Color c) const
                     d += weight * sqrt(dx * dx + dy * dy);
                 }
             }
-        }
         if (d > max_distance)
         {
             best = p;
@@ -322,36 +315,9 @@ Point State::find_best_starting_point(Color c) const
     return best;
 }
 
-bool State::gen_playout_move(Move lgr1, Move lgr2, PlayerMove<Move>& mv)
+bool State::gen_playout_move_full(PlayerMove<Move>& mv)
 {
-    if (m_nu_passes == m_nu_colors)
-        return false;
-
-    if (! m_is_symmetry_broken
-            && m_bd.get_nu_onboard_pieces() >= m_symmetry_min_nu_pieces)
-    {
-        // See also the comment in evaluate_playout()
-        if (log_simulations)
-            log("Terminate playout. Symmetry not broken.");
-        return false;
-    }
-
     Color to_play = m_bd.get_to_play();
-    if (lgr2.is_regular() && m_bd.is_legal_nonpass(lgr2))
-    {
-        if (log_simulations)
-            log("Playing last good reply 2");
-        mv = PlayerMove<Move>(to_play.to_int(), lgr2);
-        return true;
-    }
-    if (lgr1.is_regular() && m_bd.is_legal_nonpass(lgr1))
-    {
-        if (log_simulations)
-            log("Playing last good reply 1");
-        mv = PlayerMove<Move>(to_play.to_int(), lgr1);
-        return true;
-    }
-
     while (true)
     {
         if (! m_is_move_list_initialized[to_play])
@@ -376,40 +342,19 @@ bool State::gen_playout_move(Move lgr1, Move lgr2, PlayerMove<Move>& mv)
     }
 
     auto& moves = m_moves[to_play];
+    double total_gamma = m_cumulative_gamma[moves.size() - 1];
     if (log_simulations)
-        log("Moves: ", moves.size(), ", total gamma: ", m_total_gamma);
+        log("Moves: ", moves.size(), ", total_gamma: ", total_gamma);
     auto begin = m_cumulative_gamma.begin();
     auto end = begin + moves.size();
-    auto random = m_total_gamma * m_random.generate_double();
+    auto random = m_random.generate_double(0, total_gamma);
     auto pos = lower_bound(begin, end, random);
     LIBBOARDGAME_ASSERT(pos != end);
-    mv = PlayerMove<Move>(to_play.to_int(),
+    mv = PlayerMove<Move>(get_player(),
                           moves[static_cast<unsigned>(pos - begin)]);
     return true;
 }
 
-/** Bonus added to the result to encorage wins with larger scores and
-    fewer number of moves.
-    See also: Pepels et al.: Quality-based Rewards for Monte-Carlo Tree Search
-    Simulations. ECAI 2014. */
-inline Float State::get_eval_bonus(Color c, Float result, Float score)
-{
-    Float bonus = 0;
-    Float l = static_cast<Float>(m_bd.get_nu_moves());
-    m_stat_len.add(l);
-    Float dev = m_stat_len.get_deviation();
-    if (dev > 0)
-        bonus +=
-                (result == 1 ? -0.06f : 0.06f)
-                * sigmoid(2.f, (l - m_stat_len.get_mean()) / dev);
-    auto& stat = m_stat_score[c];
-    stat.add(score);
-    dev = stat.get_deviation();
-    if (dev > 0)
-        bonus += 0.3f * sigmoid(2.f, (score - stat.get_mean()) / dev);
-    return bonus;
-}
-
 inline const PieceMap<bool>& State::get_pieces_considered() const
 {
     // Use number of on-board pieces for move number to handle the case where
@@ -422,11 +367,65 @@ inline const PieceMap<bool>& State::get_pieces_considered() const
         return *m_shared_const.is_piece_considered[nu_moves];
 }
 
+/** Bonus added to the result for quality-based rewards.
+    See also: Pepels et al.: Quality-based Rewards for Monte-Carlo Tree Search
+    Simulations. ECAI 2014. */
+inline Float State::get_quality_bonus(Color c, Float result, Float score,
+                                      bool use_nu_attach)
+{
+    Float bonus = 0;
+
+    // Game length
+    Float l = static_cast<Float>(m_bd.get_nu_moves());
+    m_stat_len.add(l);
+    Float dev = m_stat_len.get_deviation();
+    if (dev > 0)
+        bonus +=
+                (result == 1 ? -0.06f : 0.06f)
+                * sigmoid(2.f, (l - m_stat_len.get_mean()) / dev);
+
+    // Game score
+    auto& stat = m_stat_score[c];
+    stat.add(score);
+    dev = stat.get_deviation();
+    if (dev > 0)
+        bonus += 0.3f * sigmoid(2.f, (score - stat.get_mean()) / dev);
+
+    // Number of non-forbidden attach points is another feature of a superior
+    // final position. Not used in Duo/Junior, mainly helps in Trigon.
+    if (! use_nu_attach)
+        return bonus;
+    auto second_color = m_bd.get_second_color(c);
+    Float opp_weight = (m_bd.get_nu_players() == 2 ? 1.f : 1.f / m_nu_colors);
+    Float attach = 0;
+    for (Color i : Color::Range(m_nu_colors))
+        if (i == c || i == second_color)
+        {
+            for (Point p : m_bd.get_attach_points(i))
+                if (! m_bd.is_forbidden(p, i))
+                    ++attach;
+        }
+        else
+        {
+            Float n = 0;
+            for (Point p : m_bd.get_attach_points(i))
+                if (! m_bd.is_forbidden(p, i))
+                    ++n;
+            attach -= opp_weight * n;
+        }
+    m_stat_attach.add(attach);
+    dev = m_stat_attach.get_deviation();
+    if (dev > 0)
+        bonus += 0.1f
+                * sigmoid(2.f, (attach - m_stat_attach.get_mean()) / dev);
+    return bonus;
+}
+
 void State::init_moves_with_gamma(Color c)
 {
     m_is_piece_considered[c] = &get_pieces_considered();
-    m_local_value.init(m_bd);
-    m_total_gamma = 0;
+    auto& is_piece_considered = *m_is_piece_considered[c];
+    m_playout_features[c].set_local(m_bd);
     auto& marker = m_marker[c];
     auto& moves = m_moves[c];
     marker.clear(moves);
@@ -442,8 +441,10 @@ void State::init_moves_with_gamma(Color c)
             if (! m_bd.is_forbidden(p, c))
                 add_moves(p, c, pieces_considered);
     m_is_move_list_initialized[c] = true;
-    m_new_moves[c].clear();
-    if (moves.empty() && ! m_force_consider_all_pieces)
+    m_nu_new_moves[c] = 0;
+    m_last_attach_points_end[c] = m_bd.get_attach_points(c).end();
+    if (moves.empty()
+            && &is_piece_considered != &m_shared_const.is_piece_considered_all)
     {
         m_force_consider_all_pieces = true;
         init_moves_with_gamma(c);
@@ -453,18 +454,21 @@ void State::init_moves_with_gamma(Color c)
 void State::init_moves_without_gamma(Color c)
 {
     m_is_piece_considered[c] = &get_pieces_considered();
+    auto& is_piece_considered = *m_is_piece_considered[c];
     auto& marker = m_marker[c];
     auto& moves = m_moves[c];
     marker.clear(moves);
     moves.clear();
     Board::PiecesLeftList pieces_considered;
     for (Piece piece : m_bd.get_pieces_left(c))
-        if ((*m_is_piece_considered[c])[piece])
+        if (is_piece_considered[piece])
             pieces_considered.push_back(piece);
     auto& is_forbidden = m_bd.is_forbidden(c);
     if (m_bd.is_first_piece(c))
         add_starting_moves(c, pieces_considered, false);
     else
+    {
+        unsigned nu_moves = 0;
         for (Point p : m_bd.get_attach_points(c))
             if (! is_forbidden[p])
             {
@@ -472,16 +476,18 @@ void State::init_moves_without_gamma(Color c)
                 for (Piece piece : pieces_considered)
                     for (Move mv : get_moves(c, piece, p, adj_status))
                         if (! marker[mv]
-                                && check_move_without_gamma(is_forbidden, mv))
-                        {
+                                && check_forbidden(is_forbidden, mv, moves,
+                                                   nu_moves))
                             marker.set(mv);
-                            moves.push_back(mv);
-                        }
-                m_moves_added_at[c].set(p);
+                m_moves_added_at[c][p] = true;
             }
+        moves.resize(nu_moves);
+    }
     m_is_move_list_initialized[c] = true;
-    m_new_moves[c].clear();
-    if (moves.empty() && ! m_force_consider_all_pieces)
+    m_nu_new_moves[c] = 0;
+    m_last_attach_points_end[c] = m_bd.get_attach_points(c).end();
+    if (moves.empty()
+            && &is_piece_considered != &m_shared_const.is_piece_considered_all)
     {
         m_force_consider_all_pieces = true;
         init_moves_without_gamma(c);
@@ -492,7 +498,7 @@ void State::play_expanded_child(Move mv)
 {
     if (log_simulations)
         log("Playing expanded child");
-    if (! mv.is_pass())
+    if (! mv.is_null())
         play_playout(mv);
     else
     {
@@ -514,8 +520,10 @@ void State::start_search()
     m_bd.copy_from(bd);
     m_bd.set_to_play(m_shared_const.to_play);
     m_bd.take_snapshot();
-    m_bc = &m_bd.get_board_const();
     m_nu_colors = bd.get_nu_colors();
+    for (Color c : Color::Range(m_nu_colors))
+        m_playout_features[c].init_snapshot(m_bd, c);
+    m_bc = &m_bd.get_board_const();
     m_move_info_array = m_bc->get_move_info_array();
     m_move_info_ext_array = m_bc->get_move_info_ext_array();
     m_check_terminate_early =
@@ -540,10 +548,11 @@ void State::start_search()
 
     m_prior_knowledge.start_search(bd);
     m_stat_len.clear();
-    for (ColorIterator i(m_nu_colors); i; ++i)
-        m_stat_score[*i].clear();
+    m_stat_attach.clear();
+    for (Color c : Color::Range(m_nu_colors))
+        m_stat_score[c].clear();
 
-    // Init precomputed gamma values
+    // Init gamma values
     double gamma_size_factor = 1;
     double gamma_nu_attach_factor = 1;
     switch (bd.get_board_type())
@@ -583,49 +592,42 @@ void State::start_simulation(size_t n)
             "=========================================================");
     m_bd.restore_snapshot();
     m_force_consider_all_pieces = false;
-    for (ColorIterator i(m_nu_colors); i; ++i)
+    auto& geo = m_bd.get_geometry();
+    for (Color c : Color::Range(m_nu_colors))
     {
-        m_has_moves[*i] = true;
-        m_is_move_list_initialized[*i] = false;
-        m_new_moves[*i].clear();
-        m_moves_added_at[*i].clear();
+        m_has_moves[c] = true;
+        m_is_move_list_initialized[c] = false;
+        m_playout_features[c].restore_snapshot(m_bd);
+        m_moves_added_at[c].fill(false, geo);
     }
     m_nu_passes = 0;
-    for (unsigned i = m_bd.get_nu_moves(); i > 0; --i)
-    // Should m_nu_passes be initialized without assuming alternating
-    // colors in the board's move history?
-    {
-        if (! m_bd.get_move(i - 1).move.is_pass())
-            break;
-        ++m_nu_passes;
-    }
 }
 
 void State::update_moves(Color c)
 {
-    m_local_value.init(m_bd);
-    m_total_gamma = 0;
+    auto& playout_features = m_playout_features[c];
+    playout_features.set_local(m_bd);
+
     auto& marker = m_marker[c];
 
     // Find old moves that are still legal
     auto& is_forbidden = m_bd.is_forbidden(c);
-    auto& new_moves = m_new_moves[c];
     auto& moves = m_moves[c];
     auto old_size = moves.size();
     moves.clear();
-    double gamma;
-    if (new_moves.size() == 1 && m_bd.get_nu_piece_instances() == 1)
+    unsigned nu_moves = 0;
+    double total_gamma = 0;
+    if (m_nu_new_moves[c] == 1 && m_bd.get_nu_piece_instances() == 1)
     {
-        Piece piece = get_move_info(new_moves[0]).get_piece();
+        Piece piece = get_move_info(m_last_move[c]).get_piece();
         for (unsigned i = 0; i < old_size; ++i)
         {
             LIBBOARDGAME_ASSERT(i >= moves.size());
             Move mv = moves.get_unchecked(i);
             auto& info = get_move_info(mv);
-            if (info.get_piece() != piece
-                    && check_move(is_forbidden, info, gamma))
-                add_move(moves, mv, gamma);
-            else
+            if (info.get_piece() == piece
+                    || ! check_move(mv, info, moves, nu_moves,
+                                    playout_features, total_gamma))
                 marker.clear(mv);
         }
     }
@@ -639,30 +641,27 @@ void State::update_moves(Color c)
             LIBBOARDGAME_ASSERT(i >= moves.size());
             Move mv = moves.get_unchecked(i);
             auto& info = get_move_info(mv);
-            if (is_piece_left[info.get_piece()]
-                    && check_move(is_forbidden, info, gamma))
-                add_move(moves, mv, gamma);
-            else
+            if (! is_piece_left[info.get_piece()]
+                    || ! check_move(mv, info, moves,  nu_moves,
+                                    playout_features, total_gamma))
                 marker.clear(mv);
         }
     }
+    moves.resize(nu_moves);
 
     // Find new legal moves because of new pieces played by this color
     Board::PiecesLeftList pieces_considered;
     for (Piece piece : m_bd.get_pieces_left(c))
         if ((*m_is_piece_considered[c])[piece])
             pieces_considered.push_back(piece);
-    for (Move mv : new_moves)
-    {
-        auto& info_ext = get_move_info_ext(mv);
-        auto i = info_ext.begin_attach();
-        auto end = info_ext.end_attach();
-        do
-            if (! is_forbidden[*i] && ! m_moves_added_at[c][*i])
-                add_moves(*i, c, pieces_considered);
-        while (++i != end);
-    }
-    new_moves.clear();
+    auto& attach_points = m_bd.get_attach_points(c);
+    auto begin = m_last_attach_points_end[c];
+    auto end = attach_points.end();
+    for (auto i = begin; i != end; ++i)
+        if (! is_forbidden[*i] && ! m_moves_added_at[c][*i])
+            add_moves(*i, c, pieces_considered);
+    m_nu_new_moves[c] = 0;
+    m_last_attach_points_end[c] = end;
 
     // Generate moves for pieces not considered in the last position
     auto& is_piece_considered = *m_is_piece_considered[c];
@@ -678,12 +677,14 @@ void State::update_moves(Color c)
                 if (! is_piece_considered[piece]
                     && is_piece_considered_new[piece])
                     pieces_considered.push_back(piece);
+            double total_gamma =
+                    moves.empty() ? 0 : m_cumulative_gamma[moves.size() - 1];
             for (Point p : m_bd.get_attach_points(c))
                 if (! is_forbidden[p])
                 {
                     auto adj_status = m_bd.get_adj_status(p, c);
                     for (Piece piece : pieces_considered)
-                        add_moves(p, c, piece, adj_status);
+                        add_moves(p, c, piece, adj_status, total_gamma);
                 }
             m_is_piece_considered[c] = &is_piece_considered_new;
         }
@@ -692,7 +693,6 @@ void State::update_moves(Color c)
 
 void State::update_symmetry_broken(Move mv)
 {
-    LIBBOARDGAME_ASSERT(! mv.is_pass());
     Color to_play = m_bd.get_to_play();
     Color second_color = m_bd.get_second_color(to_play);
     auto& info = get_move_info(mv);
