@@ -7,88 +7,33 @@
 #ifndef LIBPENTOBI_MCTS_STATE_H
 #define LIBPENTOBI_MCTS_STATE_H
 
-#include "LocalValue.h"
+#include "PlayoutFeatures.h"
 #include "PriorKnowledge.h"
+#include "SharedConst.h"
 #include "StateUtil.h"
+#include "libboardgame_mcts/LastGoodReply.h"
 #include "libboardgame_mcts/PlayerMove.h"
 #include "libboardgame_util/Log.h"
 #include "libboardgame_util/RandomGenerator.h"
 #include "libboardgame_util/Statistics.h"
-#include "libpentobi_base/Board.h"
-#include "libpentobi_base/SymmetricPoints.h"
 
 namespace libpentobi_mcts {
 
-using namespace std;
+using libboardgame_mcts::LastGoodReply;
 using libboardgame_mcts::PlayerInt;
 using libboardgame_mcts::PlayerMove;
 using libboardgame_util::log;
-using libboardgame_util::ArrayList;
 using libboardgame_util::RandomGenerator;
 using libboardgame_util::Statistics;
-using libpentobi_base::Board;
 using libpentobi_base::BoardConst;
-using libpentobi_base::Color;
-using libpentobi_base::ColorMap;
-using libpentobi_base::Marker;
 using libpentobi_base::MoveInfo;
 using libpentobi_base::MoveInfoExt;
-using libpentobi_base::MoveMarker;
 using libpentobi_base::Piece;
 using libpentobi_base::PieceInfo;
-using libpentobi_base::PieceMap;
-using libpentobi_base::PrecompMoves;
-using libpentobi_base::SymmetricPoints;
 using libpentobi_base::Variant;
 
 //-----------------------------------------------------------------------------
 
-/** Constant data shared between the search states. */
-struct SharedConst
-{
-    /** Precomputed moves additionally constrained by moves that are
-        non-forbidden at root position. */
-    ColorMap<PrecompMoves> precomp_moves;
-
-    /** The game board.
-        Contains the current position. */
-    const Board* board;
-
-    /** The color to play at the root of the search. */
-    const Color& to_play;
-
-    bool avoid_symmetric_draw;
-
-    /** Lookup table for symmetric points (only used in Duo and Trigon). */
-    SymmetricPoints symmetric_points;
-
-    /** Precomputed information if move is forbidden at the start position (and
-        therefore in all positions in the search). */
-    ColorMap<MoveMarker> is_forbidden_at_root;
-
-    /** Minimum move number where all pieces are considered until the rest
-        of the simulation. */
-    unsigned min_move_all_considered;
-
-    /** Precomputed lists of considered pieces depending on the move number.
-        Only initialized for move numbers less than min_move_all_considered.
-        Contains pointers to unique values auch that the comparison of the
-        lists can be done by comparing the pointers to the lists. */
-    array<const PieceMap<bool>*, Board::max_game_moves> is_piece_considered;
-
-    /** List of unique values for is_piece_considered. */
-    ArrayList<PieceMap<bool>, Board::max_game_moves> is_piece_considered_list;
-
-    /** Precomputed lists of considered pieces if all pieces are enforced to be
-        considered (because using the restricted set of pieces would generate
-        no moves). */
-    PieceMap<bool> is_piece_considered_all;
-
-    SharedConst(const Color& to_play);
-};
-
-//-----------------------------------------------------------------------------
-
 /** A state of a simulation.
     This class contains modifiable data used in a simulation. In multi-threaded
     search (not yet implemented), each thread uses its own instance of this
@@ -100,10 +45,17 @@ struct SharedConst
 class State
 {
 public:
-    typedef libboardgame_mcts::Node<Move, Float> Node;
+    typedef libboardgame_mcts::Node<Move, Float, SearchParamConst::multithread>
+        Node;
 
     typedef libboardgame_mcts::Tree<Node> Tree;
 
+    typedef libboardgame_mcts::LastGoodReply<Move,
+                                             SearchParamConst::max_players,
+                                             SearchParamConst::lgr_hash_table_size,
+                                             SearchParamConst::multithread>
+        LastGoodReply;
+
     /** Constructor.
         @param initial_variant Game variant to initialize the internal
         board with (may avoid unnecessary BoardConst creation for game variant
@@ -125,7 +77,8 @@ public:
     /** Finish in-tree phase without expanding a node. */
     void finish_in_tree_no_expansion();
 
-    PlayerInt get_to_play() const;
+    /** Get current player to play. */
+    PlayerInt get_player() const;
 
     void start_search();
 
@@ -138,7 +91,8 @@ public:
     /** Generate a playout move.
         @return @c false if end of game was reached, and no move was
         generated. */
-    bool gen_playout_move(Move lgr1, Move lgr2, PlayerMove<Move>& move);
+    bool gen_playout_move(const LastGoodReply& lgr, Move last,
+                          Move second_last, PlayerMove<Move>& move);
 
     void evaluate_playout(array<Float, 6>& result);
 
@@ -182,17 +136,22 @@ private:
 
     PriorKnowledge m_prior_knowledge;
 
-    /** The sum of the gamma values of all moves. */
-    double m_total_gamma;
-
-    /** Precomputed gamma value for a piece. */
+    /** Gamma value for a piece. */
     PieceMap<double> m_gamma_piece;
 
-    /** Precomputed gamma value for LocalValue::get_nu_attach(). */
+    /** Gamma value for PlayoutFeatures::get_nu_attach(). */
     array<double, PieceInfo::max_size + 1> m_gamma_nu_attach;
 
-    /** Moves played by a color since the last update of its move list. */
-    ColorMap<ArrayList<Move, Board::max_nonpass_player_moves>> m_new_moves;
+    /** Number of moves played by a color since the last update of its move
+        list. */
+    ColorMap<unsigned> m_nu_new_moves;
+
+    /** Board::get_attach_points().end() for a color at the last update of
+        its move list. */
+    ColorMap<PointList::const_iterator> m_last_attach_points_end;
+
+    /** Last move played by a color since the last update of its move list. */
+    ColorMap<Move> m_last_move;
 
     ColorMap<bool> m_is_move_list_initialized;
 
@@ -201,18 +160,19 @@ private:
     /** Marks moves contained in m_moves. */
     ColorMap<MoveMarker> m_marker;
 
-    LocalValue m_local_value;
+    ColorMap<PlayoutFeatures> m_playout_features;
 
     RandomGenerator m_random;
 
-    /** Statistics of the score for each color.
-        Used for normalizing the score modification of the game result. */
+    /** Used in get_quality_bonus(). */
     ColorMap<Statistics<Float>> m_stat_score;
 
-    /** Statistics of the length of a simulation.
-        Used for normalizing the length modification of the game result. */
+    /** Used in get_quality_bonus(). */
     Statistics<Float> m_stat_len;
 
+    /** Used in get_quality_bonus(). */
+    Statistics<Float> m_stat_attach;
+
     bool m_check_symmetric_draw;
 
     bool m_check_terminate_early;
@@ -235,15 +195,15 @@ private:
         Allows the incremental update of the move lists to skip attach points
         of newly played pieces that were already attach points of previously
         played pieces. */
-    ColorMap<Marker> m_moves_added_at;
-
-    void add_move(MoveList& moves, Move mv, double gamma);
+    ColorMap<Grid<bool>> m_moves_added_at;
 
     void add_moves(Point p, Color c,
                    const Board::PiecesLeftList& pieces_considered);
 
-    void add_moves(Point p, Color c, Piece piece, unsigned adj_status);
+    void add_moves(Point p, Color c, Piece piece, unsigned adj_status,
+                   double& total_gamma);
 
+    LIBBOARDGAME_NOINLINE
     void add_starting_moves(Color c,
                             const Board::PiecesLeftList& pieces_considered,
                             bool with_gamma);
@@ -256,7 +216,8 @@ private:
 
     Point find_best_starting_point(Color c) const;
 
-    Float get_eval_bonus(Color c, Float result, Float score);
+    Float get_quality_bonus(Color c, Float result, Float score,
+                            bool use_nu_attach);
 
     /** Equivalent to but faster than m_bd.get_move_info() */
     const MoveInfo& get_move_info(Move move) const;
@@ -264,8 +225,8 @@ private:
     /** Equivalent to but faster than m_bd.get_move_info_ext() */
     const MoveInfoExt& get_move_info_ext(Move move) const;
 
-    PrecompMoves::LocalMovesListRange get_moves(Color c, Piece piece, Point p,
-                                                unsigned adj_status) const;
+    PrecompMoves::Range get_moves(Color c, Piece piece, Point p,
+                                  unsigned adj_status) const;
 
     const PieceMap<bool>& get_pieces_considered() const;
 
@@ -273,13 +234,20 @@ private:
 
     void init_moves_without_gamma(Color c);
 
-    bool check_move(const Grid<bool>& is_forbidden, const MoveInfo& info,
-                    double& gamma);
+    bool check_forbidden(const Grid<bool>& is_forbidden, Move mv,
+                         MoveList& moves, unsigned& nu_moves);
 
-    bool check_move_without_gamma(const Grid<bool>& is_forbidden, Move mv);
+    bool check_move(Move mv, const MoveInfo& info, MoveList& moves,
+                    unsigned& nu_moves,
+                    const PlayoutFeatures& playout_features,
+                    double& total_gamma);
+
+    bool gen_playout_move_full(PlayerMove<Move>& mv);
 
     void update_moves(Color c);
 
+    void update_playout_features(Color c, Move mv);
+
     void update_symmetry_broken(Move mv);
 };
 
@@ -312,33 +280,64 @@ inline void State::gen_children(Tree::NodeExpander& expander, Float init_val)
         return;
     Color to_play = m_bd.get_to_play();
     init_moves_without_gamma(to_play);
-    m_local_value.init(m_bd);
     m_prior_knowledge.gen_children(m_bd, m_moves[to_play],
-                                   m_is_symmetry_broken, m_local_value,
-                                   expander, init_val);
+                                   m_is_symmetry_broken, expander, init_val);
+}
+
+inline bool State::gen_playout_move(const LastGoodReply& lgr, Move last,
+                                    Move second_last, PlayerMove<Move>& mv)
+{
+    if (m_nu_passes == m_nu_colors)
+        return false;
+    if (! m_is_symmetry_broken
+            && m_bd.get_nu_onboard_pieces() >= m_symmetry_min_nu_pieces)
+    {
+        // See also the comment in evaluate_playout()
+        if (log_simulations)
+            log("Terminate playout. Symmetry not broken.");
+        return false;
+    }
+    PlayerInt player = get_player();
+    Move lgr2 = lgr.get_lgr2(player, last, second_last);
+    if (! lgr2.is_null() && m_bd.is_legal(lgr2))
+    {
+        if (log_simulations)
+            log("Playing last good reply 2");
+        mv = PlayerMove<Move>(player, lgr2);
+        return true;
+    }
+    Move lgr1 = lgr.get_lgr1(player, last);
+    if (! lgr1.is_null() && m_bd.is_legal(lgr1))
+    {
+        if (log_simulations)
+            log("Playing last good reply 1");
+        mv = PlayerMove<Move>(player, lgr1);
+        return true;
+    }
+    return gen_playout_move_full(mv);
 }
 
 inline const MoveInfo& State::get_move_info(Move mv) const
 {
     LIBBOARDGAME_ASSERT(! mv.is_null());
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_all_moves());
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
     return *(m_move_info_array + mv.to_int());
 }
 
 inline const MoveInfoExt& State::get_move_info_ext(Move mv) const
 {
     LIBBOARDGAME_ASSERT(! mv.is_null());
-    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_all_moves());
+    LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
     return *(m_move_info_ext_array + mv.to_int());
 }
 
-inline PrecompMoves::LocalMovesListRange State::get_moves(
-                      Color c, Piece piece, Point p, unsigned adj_status) const
+inline PrecompMoves::Range State::get_moves(Color c, Piece piece, Point p,
+                                            unsigned adj_status) const
 {
     return m_shared_const.precomp_moves[c].get_moves(piece, p, adj_status);
 }
 
-inline PlayerInt State::get_to_play() const
+inline PlayerInt State::get_player() const
 {
     unsigned player = m_bd.get_to_play().to_int();
     if ( m_bd.get_variant() == Variant::classic_3 && player == 3)
@@ -349,15 +348,16 @@ inline PlayerInt State::get_to_play() const
 inline void State::play_in_tree(Move mv)
 {
     Color to_play = m_bd.get_to_play();
-    if (! mv.is_pass())
+    if (! mv.is_null())
     {
         LIBBOARDGAME_ASSERT(m_bd.is_legal(to_play, mv));
-        m_bd.play_nonpass(to_play, mv);
+        m_bd.play(to_play, mv);
         m_nu_passes = 0;
+        update_playout_features(to_play, mv);
     }
     else
     {
-        m_bd.play_pass(to_play);
+        m_bd.set_to_play(to_play.get_next(m_nu_colors));
         ++m_nu_passes;
     }
     if (log_simulations)
@@ -366,14 +366,17 @@ inline void State::play_in_tree(Move mv)
 
 inline void State::play_playout(Move mv)
 {
-    LIBBOARDGAME_ASSERT(m_bd.is_legal_nonpass(mv));
-    m_new_moves[m_bd.get_to_play()].push_back(mv);
-    m_bd.play_nonpass(mv);
+    auto to_play = m_bd.get_to_play();
+    LIBBOARDGAME_ASSERT(m_bd.is_legal(to_play, mv));
+    m_bd.play(to_play, mv);
+    ++m_nu_new_moves[to_play];
+    m_last_move[to_play] = mv;
     m_nu_passes = 0;
     if (! m_is_symmetry_broken)
         update_symmetry_broken(mv);
     if (log_simulations)
         log(m_bd);
+    update_playout_features(to_play, mv);
 }
 
 inline void State::start_playout()
@@ -386,6 +389,14 @@ inline bool State::skip_rave(Move mv) const
     return false;
 }
 
+inline void State::update_playout_features(Color c, Move mv)
+{
+    auto& info = get_move_info(mv);
+    for (Color i : Color::Range(m_nu_colors))
+        m_playout_features[i].set_forbidden(info);
+    m_playout_features[c].set_forbidden(get_move_info_ext(mv));
+}
+
 //-----------------------------------------------------------------------------
 
 } // namespace libpentobi_mcts
