@@ -20,7 +20,6 @@
 namespace libpentobi_mcts {
 namespace util {
 
-using libboardgame_mcts::ChildIterator;
 using libboardgame_mcts::Node;
 using libboardgame_mcts::Tree;
 using libboardgame_sgf::Writer;
@@ -44,8 +43,8 @@ void dump_tree_recurse(Writer& writer, Variant variant,
     writer.end_node();
     Color next_to_play = to_play.get_next(get_nu_colors(variant));
     vector<const Search::Node*> children;
-    for (Search::ChildIterator i(tree, node); i; ++i)
-        children.push_back(&(*i));
+    for (auto& i : tree.get_children(node))
+        children.push_back(&i);
     sort(children.begin(), children.end(), compare_node);
     for (const auto i : children)
     {
@@ -56,10 +55,8 @@ void dump_tree_recurse(Writer& writer, Variant variant,
         {
             auto& board_const = BoardConst::get(variant);
             auto id = get_color_id(variant, to_play);
-            if (! mv.is_pass())
+            if (! mv.is_null())
                 writer.write_property(id, board_const.to_string(mv, false));
-            else
-                writer.write_property(id, "");
         }
         dump_tree_recurse(writer, variant, tree, *i, next_to_play);
         writer.end_tree();
@@ -83,7 +80,7 @@ size_t get_memory()
 {
     size_t memory;
     size_t total_mem = libboardgame_sys::get_memory();
-    // Use half of the system memory but not more than 1.3 GB
+    // Use half of the system memory but not more than 1.4 GB
     if (total_mem == 0)
     {
         log("WARNING: could not determine system memory (assuming 512 MB)");
@@ -95,8 +92,8 @@ size_t get_memory()
 #else
         memory = total_mem / 2;
 #endif
-    if (memory > 1300000000)
-        memory = 1300000000;
+    if (memory > 1400000000)
+        memory = 1400000000;
     log("Using ", memory, " of ", total_mem, " bytes");
     return memory;
 }
