@@ -51,14 +51,7 @@ void AnalyzeGameWidget::initSize()
     m_borderX = width() / 50;
     m_borderY = height() / 20;
     m_maxX = width() - 2 * m_borderX;
-    // Assume max. Board::max_nonpass_game_moves. This is not really true
-    // because the tree may contain pass moves but such trees are only written
-    // in search state dumps and cannot be created by the user. Using
-    // max_game_moves would result in too small m_dX (size of the color dots)
-    // and adapting m_dX to the number of moves in this game would result in
-    // color dots of different sizes for different analyses, which also looks
-    // bad.
-    m_dX = qreal(m_maxX) / Board::max_nonpass_game_moves;
+    m_dX = qreal(m_maxX) / Board::max_game_moves;
     m_maxY = height() - 2 * m_borderY;
 }
 
@@ -83,7 +76,7 @@ void AnalyzeGameWidget::paintEvent(QPaintEvent*)
     QPainter painter(this);
     QFont font;
     font.setStyleStrategy(QFont::PreferOutline);
-    font.setPointSizeF(0.05 * height());
+    font.setPixelSize(static_cast<int>(0.06 * height()));
     QFontMetrics metrics(font);
     painter.translate(m_borderX, m_borderY);
     painter.setPen(Qt::NoPen);
@@ -155,7 +148,7 @@ void AnalyzeGameWidget::setCurrentPosition(const Game& game,
         ArrayList<ColorMove,Board::max_game_moves> moves;
         auto& tree = game.get_tree();
         auto current = &find_root(node);
-        while (current != 0)
+        while (current)
         {
             auto mv = tree.get_move(*current);
             if (! mv.is_null() && moves.size() < Board::max_game_moves)
