@@ -24,6 +24,8 @@ class AnalyzeSpeedDialog
 public:
     AnalyzeSpeedDialog(QWidget* parent, const QString& title);
 
+    ~AnalyzeSpeedDialog();
+
     /** Get return value if dialog was accepted.
         0 = fast, 1 = normal, 2 = slow */
     int getSpeedValue()
@@ -32,10 +34,10 @@ public:
     }
 
 public slots:
-    void accept();
+    void accept() override;
 
 private:
-    int m_speedValue;
+    int m_speedValue = 0;
 
     QStringList m_items;
 };
