@@ -13,6 +13,7 @@
 #include <QLibraryInfo>
 #include <QMessageBox>
 #include <QSettings>
+#include <QStyle>
 #include <QTranslator>
 #include "Application.h"
 #include "MainWindow.h"
@@ -68,7 +69,10 @@ int main(int argc, char* argv[])
     QCoreApplication::setOrganizationName("Pentobi");
     QCoreApplication::setApplicationName("Pentobi");
     Application app(argc, argv);
-
+#if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
+    app.setAttribute(Qt::AA_UseHighDpiPixmaps);
+    Q_INIT_RESOURCE(libpentobi_gui_resources_2x);
+#endif
     try
     {
         // For some reason, labels in the status bar have a border on
@@ -77,16 +81,16 @@ int main(int argc, char* argv[])
 
         // Allow the user to override installation paths with a config file in
         // the directory of the executable to test it without installation
-        QString manualDir;
+        QString helpDir;
         QString booksDir;
         QString translationsPentobiDir;
         QString translationsLibPentobiGuiDir;
         QString appDir = QCoreApplication::applicationDirPath();
-#ifdef PENTOBI_MANUAL_DIR
-        manualDir = PENTOBI_MANUAL_DIR;
+#ifdef PENTOBI_HELP_DIR
+        helpDir = PENTOBI_HELP_DIR;
 #endif
-        if (manualDir.isEmpty())
-            manualDir = appDir + "/manual";
+        if (helpDir.isEmpty())
+            helpDir = appDir + "/help";
 #ifdef PENTOBI_BOOKS_DIR
         booksDir = PENTOBI_BOOKS_DIR;
 #endif
@@ -104,7 +108,7 @@ int main(int argc, char* argv[])
         if (QFileInfo(overrideConfigFile).exists())
         {
             QSettings settings(overrideConfigFile, QSettings::IniFormat);
-            manualDir = settings.value("ManualDir", manualDir).toString();
+            helpDir = settings.value("HelpDir", helpDir).toString();
             booksDir = settings.value("BooksDir", booksDir).toString();
             translationsPentobiDir =
                 settings.value("TranslationsPentobiDir",
@@ -150,11 +154,10 @@ int main(int argc, char* argv[])
             threads = opt.get<unsigned>("threads");
             if (threads == 0)
                 throw OptionError("Number of threads must be greater zero.");
-#ifdef LIBBOARDGAME_MCTS_SINGLE_THREAD
-            if (threads > 1)
+            if (! libpentobi_mcts::SearchParamConst::multithread
+                    && threads > 1)
                 throw OptionError("This version of Pentobi was compiled"
                                   " without support for multi-threading.");
-#endif
         }
         if (! opt.contains("verbose"))
             set_log_null();
@@ -168,8 +171,14 @@ int main(int argc, char* argv[])
         QString initialFile;
         auto& args = opt.get_args();
         if (! args.empty())
-            initialFile = args[0].c_str();
-        MainWindow mainWindow(initialFile, manualDir, booksDir, noBook,
+            initialFile = QString::fromLocal8Bit(args[0].c_str());
+        QSettings settings;
+        auto variantString = settings.value("variant", "").toString();
+        Variant variant;
+        if (! parse_variant_id(variantString.toLocal8Bit().constData(),
+                               variant))
+            variant = Variant::classic;
+        MainWindow mainWindow(variant, initialFile, helpDir, booksDir, noBook,
                               threads, memory);
         if (opt.contains("seed"))
             mainWindow.setDeterministic();
