@@ -39,16 +39,17 @@ using namespace std;
 using libboardgame_sgf::SgfNode;
 using libboardgame_base::Transform;
 using libboardgame_util::ArrayList;
-using libboardgame_util::Exception;
 using libboardgame_util::RandomGenerator;
 using libpentobi_base::Board;
 using libpentobi_base::ColorMap;
 using libpentobi_base::ColorMove;
 using libpentobi_base::Game;
+using libpentobi_base::Move;
+using libpentobi_base::MoveList;
+using libpentobi_base::MoveMarker;
 using libpentobi_base::Piece;
 using libpentobi_base::Point;
 using libpentobi_base::Variant;
-using libpentobi_base::Move;
 using libpentobi_mcts::Player;
 
 //-----------------------------------------------------------------------------
@@ -59,15 +60,16 @@ class MainWindow
     Q_OBJECT
 
 public:
-    MainWindow(const QString& initialFile = "", const QString& manualDir = "",
-               const QString& booksDir = "", bool noBook = false,
-               unsigned nu_threads = 0, size_t memory = 0);
+    MainWindow(Variant variant, const QString& initialFile = "",
+               const QString& helpDir = "", const QString& booksDir = "",
+               bool noBook = false, unsigned nu_threads = 0,
+               size_t memory = 0);
 
     ~MainWindow();
 
-    bool eventFilter(QObject* object, QEvent* event);
+    bool eventFilter(QObject* object, QEvent* event) override;
 
-    QSize sizeHint() const;
+    QSize sizeHint() const override;
 
 public slots:
     void about();
@@ -76,15 +78,13 @@ public slots:
 
     void backward();
 
-    void backward10();
-
     void backToMainVariation();
 
     void beginning();
 
     void beginningOfBranch();
 
-    void clearSelectedPiece();
+    void clearPiece();
 
     void computerColors();
 
@@ -100,14 +100,12 @@ public slots:
 
     void findNextComment();
 
-    void flipPieceHorizontally();
+    void flipHorizontally();
 
-    void flipPieceVertically();
+    void flipVertically();
 
     void forward();
 
-    void forward10();
-
     void gotoMove();
 
     /** Go to a node if a node with a position defined by a sequence of moves
@@ -138,8 +136,6 @@ public slots:
 
     void nextVariation();
 
-    void nextVariation10();
-
     void nextPiece();
 
     void nextTransform();
@@ -162,13 +158,11 @@ public slots:
 
     void previousVariation();
 
-    void previousVariation10();
-
     void ratedGame();
 
-    void rotatePieceAnticlockwise();
+    void rotateAnticlockwise();
 
-    void rotatePieceClockwise();
+    void rotateClockwise();
 
     void save();
 
@@ -258,34 +252,39 @@ private:
 
     static const int maxRecentFiles = 9;
 
-    unique_ptr<Game> m_game;
+    Game m_game;
+
+    const Board& m_bd;
 
     Color m_currentColor;
 
     unique_ptr<Player> m_player;
 
-    bool m_noDelay;
+    bool m_noDelay = false;
 
-    bool m_isGenMoveRunning;
+    /** Was window maximized before entering fullscreen. */
+    bool m_wasMaximized = false;
 
-    bool m_isAnalyzeRunning;
+    bool m_isGenMoveRunning = false;
+
+    bool m_isAnalyzeRunning = false;
 
     /** Should the computer generate a move if it is its turn?
         Enabled on game start (if the computer plays at least one color)
         or after selecting Play. Disabled when navigating in the game. */
-    bool m_autoPlay;
+    bool m_autoPlay = false;
 
     /** Flag indicating that the position after the last move played was
         a terminal position. */
     bool m_gameFinished;
 
-    bool m_isRated;
+    bool m_isRated = false;
 
     /** Flag set while setting the text in m_comment for fast return in the
         textChanged() handler.
         Used because QPlainTextEdit does not have a textEdited() signal and
         we only need to handle edits. */
-    bool m_ignoreCommentTextChanged;
+    bool m_ignoreCommentTextChanged = false;
 
     /** Color played by the user in a rated game.
         Only defined if m_isRated is true. In game variants with multiple
@@ -295,7 +294,7 @@ private:
 
     /** Integer ID assigned to the currently running move generation.
         Used to ignore finished events from canceled move generations. */
-    unsigned m_genMoveId;
+    unsigned m_genMoveId = 0;
 
     /** Beginning of move number range of last moves played by the computer
         without pause.
@@ -304,13 +303,13 @@ private:
         it would be hard to see what the last moves were if the computer plays
         quickly. A value of 0 means that there are no last moves played by
         the computer to be marked. */
-    unsigned m_lastComputerMovesBegin;
+    unsigned m_lastComputerMovesBegin = 0;
 
     /** End (inclusive) of move number range of last moves played by the
         computer without pause.
         Only defined if m_lastComputerMovesBegin is not 0.
         @see m_lastComputerMovesBegin */
-    unsigned m_lastComputerMovesEnd;
+    unsigned m_lastComputerMovesEnd = 0;
 
     /** Current playing level of m_player.
         Only use if m_useTimeLimit is false. Possible values for m_level are in
@@ -324,11 +323,13 @@ private:
 
     unique_ptr<RatingHistory> m_history;
 
-    QToolBar* m_toolBar;
+    /** Local variable in findMove().
+        Reused for efficiency. */
+    unique_ptr<MoveMarker> m_marker;
 
     GuiBoard* m_guiBoard;
 
-    QString m_manualDir;
+    QString m_helpDir;
 
     ColorMap<bool> m_computerColors;
 
@@ -342,11 +343,11 @@ private:
 
     QPlainTextEdit* m_comment;
 
-    HelpWindow* m_helpWindow;
+    HelpWindow* m_helpWindow = nullptr;
 
-    RatingDialog* m_ratingDialog;
+    RatingDialog* m_ratingDialog = nullptr;
 
-    AnalyzeGameWindow* m_analyzeGameWindow;
+    AnalyzeGameWindow* m_analyzeGameWindow = nullptr;
 
     QAction* m_actionAbout;
 
@@ -354,8 +355,6 @@ private:
 
     QAction* m_actionBackward;
 
-    QAction* m_actionBackward10;
-
     QAction* m_actionBackToMainVariation;
 
     QAction* m_actionBadMove;
@@ -364,7 +363,7 @@ private:
 
     QAction* m_actionBeginningOfBranch;
 
-    QAction* m_actionClearSelectedPiece;
+    QAction* m_actionClearPiece;
 
     QAction* m_actionComputerColors;
 
@@ -384,14 +383,12 @@ private:
 
     QAction* m_actionFindNextComment;
 
-    QAction* m_actionFlipPieceHorizontally;
+    QAction* m_actionFlipHorizontally;
 
-    QAction* m_actionFlipPieceVertically;
+    QAction* m_actionFlipVertically;
 
     QAction* m_actionForward;
 
-    QAction* m_actionForward10;
-
     QAction* m_actionFullscreen;
 
     QAction* m_actionGameInfo;
@@ -436,21 +433,23 @@ private:
 
     QAction* m_actionMoveDownVariation;
 
-    QAction* m_actionMoveNumbersAll;
+    QAction* m_actionMoveMarkingAllNumber;
 
-    QAction* m_actionMoveNumbersLast;
+    QAction* m_actionMoveMarkingLastDot;
 
-    QAction* m_actionMoveNumbersNone;
+    QAction* m_actionMoveMarkingLastNumber;
+
+    QAction* m_actionMoveMarkingNone;
 
     QAction* m_actionMoveUpVariation;
 
-    QAction* m_actionMoveSelectedPieceLeft;
+    QAction* m_actionMovePieceLeft;
 
-    QAction* m_actionMoveSelectedPieceRight;
+    QAction* m_actionMovePieceRight;
 
-    QAction* m_actionMoveSelectedPieceUp;
+    QAction* m_actionMovePieceUp;
 
-    QAction* m_actionMoveSelectedPieceDown;
+    QAction* m_actionMovePieceDown;
 
     QAction* m_actionNextPiece;
 
@@ -458,8 +457,6 @@ private:
 
     QAction* m_actionNextVariation;
 
-    QAction* m_actionNextVariation10;
-
     QAction* m_actionNew;
 
     QAction* m_actionRatedGame;
@@ -468,7 +465,7 @@ private:
 
     QAction* m_actionOpen;
 
-    QAction* m_actionPlaceSelectedPiece;
+    QAction* m_actionPlacePiece;
 
     QAction* m_actionPlay;
 
@@ -480,15 +477,13 @@ private:
 
     QAction* m_actionPreviousVariation;
 
-    QAction* m_actionPreviousVariation10;
-
     QAction* m_actionQuit;
 
     QAction* m_actionRecentFile[maxRecentFiles];
 
-    QAction* m_actionRotatePieceAnticlockwise;
+    QAction* m_actionRotateAnticlockwise;
 
-    QAction* m_actionRotatePieceClockwise;
+    QAction* m_actionRotateClockwise;
 
     QAction* m_actionSave;
 
@@ -582,13 +577,13 @@ private:
 
     QString m_file;
 
-    unique_ptr<ArrayList<Move, Move::range>> m_legalMoves;
+    unique_ptr<MoveList> m_legalMoves;
 
     unsigned m_legalMoveIndex;
 
     QLabel* m_moveNumber;
 
-    LeaveFullscreenButton* m_leaveFullscreenButton;
+    LeaveFullscreenButton* m_leaveFullscreenButton = nullptr;
 
     int m_lastRemainingSeconds;
 
@@ -647,10 +642,6 @@ private:
 
     void genMove(bool playSingleMove = false);
 
-    const Board& getBoard() const;
-
-    Variant getVariant() const;
-
     QString getFilter() const;
 
     QString getLastDir();
@@ -693,7 +684,7 @@ private:
     void showInfo(const QString& message, const QString& infoText = "",
                   const QString& detailText = "", bool withIcon = false);
 
-    void showInvalidFile(QString file, const Exception& e);
+    void showInvalidFile(QString file, const exception& e);
 
     void showStatus(const QString& text, bool temporary = false);
 
@@ -768,11 +759,13 @@ private slots:
 
     void setLevel(bool checked);
 
-    void setMoveNumbersAll(bool checked);
+    void setMoveMarkingAllNumber(bool checked);
 
-    void setMoveNumbersLast(bool checked);
+    void setMoveMarkingLastNumber(bool checked);
 
-    void setMoveNumbersNone(bool checked);
+    void setMoveMarkingLastDot(bool checked);
+
+    void setMoveMarkingNone(bool checked);
 
     void setSetupPlayer();
 
@@ -782,6 +775,8 @@ private slots:
 
     void toolBarNoText(bool checked);
 
+    void toolBarText(const QString& key, Qt::ToolButtonStyle style);
+
     void toolBarTextBesideIcons(bool checked);
 
     void toolBarTextBelowIcons(bool checked);
@@ -795,16 +790,6 @@ private slots:
     void veryGoodMove(bool checked);
 };
 
-inline const Board& MainWindow::getBoard() const
-{
-    return m_game->get_board();
-}
-
-inline Variant MainWindow::getVariant() const
-{
-    return m_game->get_variant();
-}
-
 //-----------------------------------------------------------------------------
 
 #endif // PENTOBI_MAIN_WINDOW_H
