@@ -38,7 +38,7 @@ RatedGamesList::RatedGamesList(QWidget* parent)
 void RatedGamesList::activateGame(const QModelIndex& index)
 {
     auto item = m_model->item(index.row(), 0);
-    if (item == 0)
+    if (! item)
         return;
     bool ok;
     unsigned n = item->text().toUInt(&ok);
@@ -84,7 +84,7 @@ void RatedGamesList::updateContent(Variant variant,
             << tr("Date");
     m_model->setHorizontalHeaderLabels(headers);
     auto header = horizontalHeader();
-    header->setDefaultAlignment(Qt::AlignLeft);
+    header->setDefaultAlignment(Qt::AlignLeft | Qt::AlignVCenter);
     header->setHighlightSections(false);
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
     header->setSectionResizeMode(QHeaderView::ResizeToContents);
