@@ -17,7 +17,6 @@
 namespace pentobi_gtp {
 
 using libboardgame_gtp::Failure;
-using libboardgame_mcts::ChildIterator;
 using libboardgame_mcts::PlayerMove;
 using libboardgame_sgf::TreeReader;
 using libboardgame_util::log;
@@ -42,7 +41,6 @@ Engine::Engine(Variant variant, int level, bool use_book,
     create_player(variant, books_dir, nu_threads, memory);
     get_mcts_player().set_use_book(use_book);
     get_mcts_player().set_level(level);
-    add("gen_playout_move", &Engine::cmd_gen_playout_move);
     add("get_value", &Engine::cmd_get_value);
     add("name", &Engine::cmd_name);
     add("param", &Engine::cmd_param);
@@ -66,8 +64,8 @@ void Engine::cmd_move_values(Response& response)
     auto& tree = search.get_tree();
     auto& bd = get_board();
     vector<const Search::Node*> children;
-    for (Search::ChildIterator i(tree, tree.get_root()); i; ++i)
-        children.push_back(&(*i));
+    for (auto& i : tree.get_root_children())
+        children.push_back(&i);
     sort(children.begin(), children.end(), libpentobi_mcts::util::compare_node);
     response << fixed;
     for (const auto node : children)
@@ -85,7 +83,7 @@ void Engine::cmd_name(Response& response)
 void Engine::cmd_save_tree(const Arguments& args)
 {
     auto& search = get_search();
-    if (! search.get_last_state().is_valid())
+    if (! search.get_last_history().is_valid())
         throw Failure("no search tree");
     ofstream out(args.get());
     libpentobi_mcts::util::dump_tree(out, search);
@@ -179,19 +177,6 @@ void Engine::create_player(Variant variant, const string& books_dir,
     set_player(*m_player);
 }
 
-void Engine::cmd_gen_playout_move(Response& response)
-{
-    auto& state = get_mcts_player().get_search().get_state(0);
-    state.start_search();
-    state.start_simulation(0);
-    state.finish_in_tree();
-    PlayerMove<Move> mv;
-    if (! state.gen_playout_move(Move::null(), Move::null(), mv))
-        throw Failure("terminal playout position");
-    auto& bd = get_board();
-    response << bd.to_string(mv.move);
-}
-
 Player& Engine::get_mcts_player()
 {
     try
@@ -215,7 +200,7 @@ void Engine::set_deterministic()
     {
         get_search().set_deterministic();
     }
-    catch (Failure&)
+    catch (const Failure&)
     {
         // Ignore if player is not MCTS player
     }
