@@ -13,11 +13,9 @@
 
 using namespace std;
 using libboardgame_util::log;
-using libboardgame_util::Exception;
 using libpentobi_base::to_string_id;
 using libpentobi_base::BoardType;
 using libpentobi_base::Color;
-using libpentobi_base::ColorIterator;
 using libpentobi_base::ColorMap;
 using libpentobi_base::ColorMove;
 using libpentobi_base::CoordPoint;
@@ -58,22 +56,20 @@ int getNuPiecesLeft(const Board& bd, Color c)
 QPointF getGameCoord(const Board& bd, Move mv)
 {
     auto& geo = bd.get_geometry();
-    auto width = geo.get_width();
     auto moveInfo = bd.get_move_info(mv);
     PiecePoints movePoints;
     for (Point p : moveInfo)
-        movePoints.push_back(CoordPoint(p.get_x(width), p.get_y(width)));
+        movePoints.push_back(CoordPoint(geo.get_x(p), geo.get_y(p)));
     return PieceModel::findCenter(bd, movePoints, false);
 }
 
 const Transform* getTransform(const Board& bd, Move mv)
 {
     auto& geo = bd.get_geometry();
-    auto width = geo.get_width();
     auto moveInfo = bd.get_move_info(mv);
     PiecePoints movePoints;
     for (Point p : moveInfo)
-        movePoints.push_back(CoordPoint(p.get_x(width), p.get_y(width)));
+        movePoints.push_back(CoordPoint(geo.get_x(p), geo.get_y(p)));
     auto& pieceInfo = bd.get_piece_info(moveInfo.get_piece());
     return pieceInfo.find_transform(geo, movePoints);
 }
@@ -190,9 +186,7 @@ bool BoardModel::findMove(const PieceModel& piece, QPointF coord,
             return false;
         if (pointTypeChanged && pointType == boardPointType)
             return false;
-        points.push_back(Point(static_cast<unsigned>(x),
-                               static_cast<unsigned>(y),
-                               geo.get_width()));
+        points.push_back(geo.get_point(x, y));
     }
     return m_bd.find_move(points, mv);
 }
@@ -312,17 +306,17 @@ bool BoardModel::loadAutoSave()
         {
             unsigned colorInt = l[i].toUInt();
             if (colorInt >= m_bd.get_nu_colors())
-                throw Exception("invalid color");
+                throw runtime_error("invalid color");
             Color c(colorInt);
             if (i + 1 >= l.length())
-                throw Exception("color without move");
+                throw runtime_error("color without move");
             Move mv = m_bd.from_string(l[i + 1].toLocal8Bit().constData());
             if (! m_bd.is_legal(c, mv))
-                throw Exception("illegal move");
+                throw runtime_error("illegal move");
             m_bd.play(c, mv);
         }
     }
-    catch (const Exception &e)
+    catch (const exception &e)
     {
         qWarning() << "BoardModel: autosave has illegal move: " << e.what();
     }
@@ -525,8 +519,8 @@ void BoardModel::updateProperties()
     }
 
     bool isGameOver = true;
-    for (ColorIterator i(m_nuColors); i; ++i)
-        if (m_bd.has_moves(*i))
+    for (Color c : m_bd.get_colors())
+        if (m_bd.has_moves(c))
             isGameOver = false;
     if (m_isGameOver != isGameOver)
     {
@@ -542,17 +536,17 @@ void BoardModel::updateProperties()
     }
 
     ColorMap<array<bool, Board::max_pieces>> isPlayed;
-    for (ColorIterator i(m_nuColors); i; ++i)
-        isPlayed[*i].fill(false);
+    for (Color c : m_bd.get_colors())
+        isPlayed[c].fill(false);
+#if LIBBOARDGAME_DEBUG
     // Does not handle setup yet
-    for (ColorIterator i(m_nuColors); i; ++i)
-        LIBBOARDGAME_ASSERT(m_bd.get_setup().placements[*i].empty());
+    for (Color c : m_bd.get_colors())
+        LIBBOARDGAME_ASSERT(m_bd.get_setup().placements[c].empty());
+#endif
     m_lastMovePieceModel = nullptr;
     for (unsigned i = 0; i < m_bd.get_nu_moves(); ++i)
     {
         auto mv = m_bd.get_move(i);
-        if (mv.is_pass())
-            continue;
         Piece piece = m_bd.get_move_info(mv.move).get_piece();
         auto& pieceInfo = m_bd.get_piece_info(piece);
         auto gameCoord = getGameCoord(m_bd, mv.move);
@@ -594,12 +588,12 @@ void BoardModel::updateProperties()
         if (i == m_bd.get_nu_moves() - 1)
             m_lastMovePieceModel = pieceModel;
     }
-    for (ColorIterator i(m_nuColors); i; ++i)
+    for (Color c : m_bd.get_colors())
     {
-        auto& pieceModels = getPieceModels(*i);
-        for (int j = 0; j < pieceModels.length(); ++j)
-            if (! isPlayed[*i][j])
-                pieceModels[j]->setIsPlayed(false);
+        auto& pieceModels = getPieceModels(c);
+        for (int i = 0; i < pieceModels.length(); ++i)
+            if (! isPlayed[c][i])
+                pieceModels[i]->setIsPlayed(false);
     }
 
     int toPlay = m_isGameOver ? 0 : m_bd.get_effective_to_play().to_int();
