@@ -10,8 +10,6 @@
 
 #include "libboardgame_mcts/Node.h"
 
-#include <cmath>
-#include <cstring>
 #include "libboardgame_test/Test.h"
 
 using namespace std;
@@ -20,7 +18,7 @@ using namespace std;
 
 LIBBOARDGAME_TEST_CASE(mcts_node_add_value)
 {
-    libboardgame_mcts::Node<int, float> node;
+    libboardgame_mcts::Node<int, float, true> node;
     node.init(0, 0.5, 0);
     node.add_value(5);
     LIBBOARDGAME_CHECK_CLOSE(node.get_value(), 5., 1e-4);
