@@ -11,7 +11,6 @@
 #include "libpentobi_base/Board.h"
 
 #include "libboardgame_test/Test.h"
-#include "libpentobi_base/MoveList.h"
 
 using namespace std;
 using namespace libpentobi_base;
@@ -125,36 +124,14 @@ LIBBOARDGAME_TEST_CASE(pentobi_base_board_classic_2)
     LIBBOARDGAME_CHECK_EQUAL(bd->get_nu_onboard_pieces(Color(1)), 3u);
     LIBBOARDGAME_CHECK_EQUAL(bd->get_nu_onboard_pieces(Color(2)), 10u);
     LIBBOARDGAME_CHECK_EQUAL(bd->get_nu_onboard_pieces(Color(3)), 3u);
-    // Make sure that bonus computation still works if after the 1-piece an
-    // additional pass move was played
-    bd->play_pass(Color(0));
-    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(0)), 109u);
-    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(1)), 7u);
-    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(2)), 38u);
-    LIBBOARDGAME_CHECK_EQUAL(bd->get_points(Color(3)), 7u);
 }
 
-/** Check the number of generated moves at each attach point in a known
-    case. */
-LIBBOARDGAME_TEST_CASE(pentobi_base_board_gen_moves)
+LIBBOARDGAME_TEST_CASE(pentobi_base_board_gen_moves_classic_initial)
 {
     unique_ptr<Board> bd(new Board(Variant::classic));
-    Color c(0);
-    play(*bd, c, "i11,j11,k11,l11");
-    unique_ptr<MoveList> moves(new MoveList());
-    for (Point p : bd->get_attach_points(c))
-    {
-        bd->gen_moves(c, p, *moves);
-        LIBBOARDGAME_CHECK_EQUAL(moves->size(), 124u);
-    }
-}
-
-/** Check the number of generated moves at a starting point. */
-LIBBOARDGAME_TEST_CASE(pentobi_base_board_gen_moves_2)
-{
-    unique_ptr<Board> bd(new Board(Variant::classic));
-    unique_ptr<MoveList> moves(new MoveList());
-    bd->gen_moves(Color(0), Point("A20", 20, 20), *moves);
+    unique_ptr<MoveList> moves(new MoveList);
+    unique_ptr<MoveMarker> marker(new MoveMarker);
+    bd->gen_moves(Color(0), *marker, *moves);
     LIBBOARDGAME_CHECK_EQUAL(moves->size(), 58u);
 }
 
